# $Log: not supported by cvs2svn $
# Revision 1.183  2011/12/12 21:18:49  soliday
# Increased the minimum L1 klystron forward power the RFGUN_Startup procedure
# will wait for before continuing to power up RF1 per Pasky's instructions.
#
# Revision 1.182  2011/08/03 18:30:14  soliday
# The fixedCurrent script is now resumed at the end of the RFGUN_Startup procedure.
#
# Revision 1.181  2011/08/02 17:29:03  soliday
# Modified the call to APSRFGUN_SetCathodeHeaterStandbyCurrent so that it now
# sets RG1 to 4.1 amps and RG2 to 3.1 amps.
#
# Revision 1.180  2010/12/10 21:13:20  soliday
# Removed all the code that dealt with the smart monitor per Pasky's request.
#
# Revision 1.179  2010/11/16 20:03:54  soliday
# When starting the gun and the modulators, it now will turn on the LLRF after
# it brings up the cathode heater current to avoid a bunch of reflected power.
#
# Revision 1.178  2010/11/16 16:51:50  soliday
# Changed a hard coded value for a target klystron forward power for RG2.
#
# Revision 1.177  2010/10/29 14:38:45  soliday
# Prior to raising the cathode heater current it now resets the ACIS interlock.
#
# Revision 1.176  2008/08/13 17:58:46  soliday
# Fixed a few small issues.
#
# Revision 1.175  2008/07/17 15:22:09  soliday
# It seemed to be running some code that was commented out so I deleted the old
# code to see if the problem still exists.
#
# Revision 1.174  2008/06/09 15:24:56  soliday
# Removed the restoring of the PAR timing values.
#
# Revision 1.173  2008/06/06 20:37:45  soliday
# Updated so that when it is running the RFGUN_Startup procedure it will not
# restore any L4 or L5 PVs if those modulators have not been selected.
#
# Revision 1.172  2008/03/07 20:42:41  soliday
# Updated to relink the linacBeamRfPhase test files.
#
# Revision 1.171  2007/12/13 16:59:39  soliday
# Updated dialog when modulator forward power has trouble reaching a minimum value.
#
# Revision 1.170  2007/03/02 16:56:36  soliday
# Added code to relink the linacBeamRfPhase controllaw input files prior
# to starting them.
#
# Revision 1.169  2007/03/01 16:14:11  soliday
# Added calls to APSLogScriptAction to store initialization values
# for LINAC_SwitchMode and LINAC_SwitchModeWithPS.
#
# Revision 1.167  2006/11/03 17:38:49  soliday
# Changed the minimum L1 klystron threshold because the previous value was
# too high.
#
# Revision 1.166  2006/10/23 15:21:35  soliday
# Improved an error message explaining why it could not detect the
# selected gun.
#
# Revision 1.165  2006/08/16 18:17:28  soliday
# Updated to fix a few problems seen when going into direct injection.
#
# Revision 1.164  2006/07/25 16:11:24  soliday
# Updated durring studies to bring the L1 forward power and RF gun cathode heater
# up slower in direct injection mode.
#
# Revision 1.163  2006/07/14 15:22:34  soliday
# Fixed a problem with LINAC_SwitchMode. It would condition the PAR PS when it
# was not suppose to.
#
# Revision 1.162  2006/05/23 17:38:31  soliday
# The cathode heater is not lowered if the sector powering L1 is not selected.
# Also added additional checks after resetting the kicker interlock.
#
# Revision 1.161  2006/05/23 14:26:43  soliday
# Fixed acis checks.
#
# Revision 1.160  2006/05/22 14:35:31  soliday
# Removed debugging statement.
#
# Revision 1.159  2006/05/19 21:28:21  soliday
# Modified the conditioning of the LTP, PAR, PTB, and PAR Bypass to convert
# the given conditioning cycles into a conditioning minutes value. Before it
# would always condition for 10 minutes now it will condition for
# (2.5 * cycles) minutes.
#
# Revision 1.158  2006/05/19 20:34:40  soliday
# Added code to check for ACIS inhibit and trip states before running
# certain PEMs. Also fixed an issue in RFGUN+LMOCS_StartupWithPS where
# the main PowerSupplies category is selected but all the individual
# PS categories are unselected. The lower level PEM would use a default
# list of PS in this case. I now check for this condition and set the
# PowerSupplies variable to 0 if all the individual PS categories are
# unselected.
#
# Revision 1.157  2006/05/18 19:20:57  soliday
# Removed references to the L2 GV0 gate valve that has been removed.
#
# Revision 1.156  2006/04/20 15:52:11  soliday
# Removed code to turn off PAR RF12 when in bypass mode per CY's request.
#
# Revision 1.155  2006/04/19 16:17:43  soliday
# Added code to improve the procedure to come out of direct injection. It
# now starts and conditions the PAR, selects the correct RF amps and
# restores a Booster file.
#
# Revision 1.154  2006/03/13 18:14:32  soliday
# Added ability to continue past an interlock that cannot be reset.
#
# Revision 1.153  2006/03/08 15:43:57  soliday
# Added CheckModulatorInterlocks procedure
#
# Revision 1.152  2005/12/19 17:08:28  soliday
# The phase setpoints are set now before resuming the controllaws.
#
# Revision 1.151  2005/12/16 21:26:33  soliday
# If the bypass mode is changed then the phase controllaws will not be
# resumed. Additional interaction is required by the operators before
# they can successfully be restarted.
#
# Revision 1.150  2005/12/16 21:11:44  soliday
# Fixed the description of the bypass par system reference file.
# The new linac phase controllaws are used with this version.
#
# Revision 1.149  2005/11/15 18:29:35  soliday
# When going to K1down mode it turns the L2 Sled on.
#
# Revision 1.148  2005/11/04 20:25:06  soliday
# Added the option of turning off the unused modulator if going to K1 or
# K2 down mode using the LINAC_SwitchMode procedure. Previously it was
# assumed that the affected modulator in K1 or K2 down mode would not
# be responsive. This is not always the case.
#
# Revision 1.147  2005/11/04 20:00:28  soliday
# Changed RFGUN+LMOCS_StartupWithPS and RFGUN+LMOCS_SwitchWithPS so that
# the all the modulator power controls are suspended at the completion by default.
#
# Revision 1.146  2005/11/04 19:54:14  soliday
# Changed the LINAC_SwitchMode so that the modulator power controls are disabled
# at the end of the procedure. The previous behavior was to only suspend the
# power control related to a k1 or k2 down mode.
#
# Revision 1.145  2005/09/28 20:29:31  soliday
# Added code to suspend and resume linac rf phase controllaw. Added code to
# turn off the alpha magnets when conditioning the trim magnets.
#
# Revision 1.144  2005/09/28 14:24:06  soliday
# Changed when the smart monitor is enabled and changed how the kicker
# is turned on.
#
# Revision 1.143  2005/09/16 18:28:22  soliday
# Updated to work with the change of the LINAC beamline category in the SCR
# files to the LI beamline. This was done because LINAC was seen as too broad
# of a category.
#
# Revision 1.142  2005/09/13 20:06:28  soliday
# Updated to work with new RF PLC control state variable.
#
# Revision 1.141  2005/08/08 15:44:55  borland
# Fixed bug in RFGUN+LMOCS_StartupWithPS: replaced sequential if, if-else with
# if, else-if, else.  This would apparently result in trying to bring up
# L1 in K1 down mode.
#
# Revision 1.140  2005/08/06 00:51:23  borland
# Added some diagnostic APSMpStep  commands in RFGUN_Startup.
#
# Revision 1.139  2005/04/07 16:14:36  soliday
# Updated to identify limited save files and not allow the operator to use
# them.
#
# Revision 1.138  2005/03/14 19:34:44  soliday
# Removed the 425 and 450 MeV options for direct injection.
#
# Revision 1.137  2004/11/01 20:01:39  soliday
# Updated modulator labels. Updated to work with K1 and K2 down modes.
#
# Revision 1.136  2004/10/18 16:09:29  soliday
# Changed the method for detecting mode switch.
#
# Revision 1.135  2004/10/18 14:28:52  soliday
# Improved error message when an invalid SCR file is selected.
#
# Revision 1.134  2004/10/07 17:24:46  soliday
# Updated the dialog for RFGUN+LMOCS_StartupWithPS.
#
# Revision 1.133  2004/09/01 14:13:26  soliday
# Prior to bringing up the L1 modulator The RFGUN+LMOCS_StartupWithPS sets
# the cathode heater to standby power.
#
# Revision 1.132  2004/07/06 17:32:47  soliday
# Added additional checks of the L1 LLRF state while bringing up the
# heater power.
#
# Revision 1.131  2004/06/30 14:33:58  soliday
# It now forces the user to select an SCR file.
#
# Revision 1.130  2004/06/30 14:29:45  soliday
# Indented code
#
# Revision 1.129  2004/06/29 14:57:03  soliday
# When the Include PAR selection is made the SCR file selection is cleared.
#
# Revision 1.128  2004/05/28 17:27:48  soliday
# Fixed another typo.
#
# Revision 1.127  2004/05/27 21:12:10  soliday
# Fixed typo
#
# Revision 1.126  2004/05/27 20:42:43  soliday
# Added the ability to automatically restore the Booster file when we go to
# direct injection.
#
# Revision 1.125  2004/05/26 19:11:31  soliday
# The kicker voltage is now brought up in four steps instead of two.
#
# Revision 1.124  2004/05/19 15:40:23  soliday
# Fixed a problem with a change I made yesterday.
#
# Revision 1.123  2004/05/18 18:20:38  soliday
# Increased pause time when increasing the gun current.
#
# Revision 1.122  2004/05/17 14:00:41  soliday
# RG2 is now voltage controlled.
#
# Revision 1.121  2004/04/21 14:01:49  soliday
# A switch to K3 Down is considered equivalent to normal mode.
#
# Revision 1.120  2004/02/24 05:42:27  soliday
# Updated after testing in the SCR. It now disables the kicker charge pulse
# instead of turning the kicker off when going into or coming out of bypass mode.
#
# Revision 1.119  2004/02/13 17:22:28  soliday
# Added changes to RFGUN+LMOCS_StartupWithPS so that when the direct injection
# state is changed it turns off the RF kicker. Also when it goes into direct
# injection it disables the 12th harmonic RF system.
#
# Revision 1.118  2004/02/10 21:57:01  borland
# Now accepts "K3 Down" files for "Normal" mode.  Puts a message in the
# dialog indicating that this is being done.
#
# Revision 1.117  2004/02/10 16:10:10  borland
# LinacSwitchModeWithPS now sets PS to condition by default.
#
# Revision 1.116  2004/02/10 15:21:26  borland
# Changed procedure LINAC_SwitchModeInfoWithPS to LINAC_SwitchModeWithPSInfo.
#
# Revision 1.115  2004/02/10 15:18:12  borland
# Fixed suspend/resume messages for LTP controllaw (Soliday).
#
# Revision 1.114  2004/01/29 17:04:17  soliday
# Updated with more error/warning dialogs.
#
# Revision 1.113  2004/01/26 22:31:18  soliday
# Modified APSPEMTKDialog and increased the number of places where it
# is used.
#
# Revision 1.112  2003/12/17 18:42:25  soliday
# Reverted some of the last changes in the case where the timeout was 0
#
# Revision 1.111  2003/12/17 04:42:50  soliday
# Increased the timeouts to 30 seconds.
#
# Revision 1.110  2003/11/17 16:17:35  soliday
# When RFGUN+LMOCS_SwitchWithPS calles RFGUN+LMOCS_StartupWithPS prior to the
# gun switch, it does not complain about an invalid SCR file.
#
# Revision 1.109  2003/11/14 18:38:48  soliday
# If RFGUN is not selected in LINAC_SwitchMode it will not bring down the gun.
#
# Revision 1.108  2003/11/14 06:22:48  soliday
# Added LINAC_SwitchModeWithPS
#
# Revision 1.107  2003/10/28 23:18:47  soliday
# Fixed warning box from poping up in RFGUN+LMOCS_SwitchWithPS when the
# correct SCR file is chosen.
#
# Revision 1.106  2003/10/28 14:43:19  soliday
# PAR Bypass valves are no longer opened in this procedure. A PAR Bypass startup
# pem handles this now.
#
# Revision 1.105  2003/10/27 20:22:21  soliday
# Replaced existing warning dialogs with calls to APSPEMTKDialog.
# Added logging of the selections on the initial dialog.
#
# Revision 1.104  2003/10/14 21:04:55  soliday
# Removed commented out code and added K2 down related code.
# Added the option of not standardizing the LTP,PTB when starting them.
#
# Revision 1.103  2003/10/14 16:55:10  soliday
# The RFGUN Shutdown pem can be lanuched with k1down settings so that it does not ramp down the LLRF.
#
# Revision 1.102  2003/10/06 16:21:20  soliday
# Added more changes to work better with K1 Down.
#
# Revision 1.101  2003/10/02 21:51:33  soliday
# Added PAR Bypass buttons and debugged K1 down mode.
#
# Revision 1.100  2003/08/20 14:39:51  soliday
# Fixed problem setting LTP:B1 and PTB:B1 into standby mode.
#
# Revision 1.99  2003/07/15 14:14:14  soliday
# No longer turn of PTB:B2 with bypass setup.
#
# Revision 1.98  2003/07/14 20:56:33  soliday
# Added some extra steps when going to bypass mode.
#
# Revision 1.97  2003/07/09 22:29:58  soliday
# Incorporated the include/bypass par switch into the RFGUN+LMOCS_StartupWithPS
#
# Revision 1.96  2003/05/19 17:56:17  soliday
# Changed CathodeHeaterCurrent to CathodeHeaterSetpoint.
#
# Revision 1.95  2003/04/17 20:48:16  soliday
# Fixed problem when deselecting LTP and PTB.
#
# Revision 1.94  2003/04/17 18:15:05  soliday
# Changed during studies.
#
# Revision 1.93  2003/04/15 20:49:20  soliday
# Changed so that burtwb does not restore the L:L?Stabilizer:SDDS* PVs.
#
# Revision 1.92  2003/04/15 18:38:01  soliday
# Added LTP and PTB to the LINAC_SwitchMode
#
# Revision 1.91  2003/03/25 16:27:37  soliday
# Worked out more kinks during studies.
#
# Revision 1.90  2003/03/24 17:59:07  soliday
# A few minor diagnosic changes.
#
# Revision 1.89  2003/03/24 17:33:13  soliday
# Added the ability to startup the LTP and PTB with the linac.
#
# Revision 1.88  2003/02/04 21:53:28  soliday
# Updated the method for turning on the 400W RF Amp
#
# Revision 1.87  2003/01/29 18:01:09  soliday
# Finally fixed the problem with parbypass timing.
#
# Revision 1.86  2003/01/29 17:30:16  soliday
# Fix related to last change.
#
# Revision 1.85  2003/01/29 17:16:46  soliday
# Fixed bug with last change.
#
# Revision 1.84  2003/01/29 17:06:47  soliday
# When in parbypass mode it does not restore some timing PVs
#
# Revision 1.83  2002/11/25 19:36:18  soliday
# Turns off the L2 sled when going into K2 down mode.
#
# Revision 1.82  2002/11/25 19:32:28  soliday
# Ned changed the way the lockSLED PVs work.
#
# Revision 1.81  2002/11/15 22:46:49  soliday
# Added K2 down mode updated the par bypass pem
#
# Revision 1.80  2002/11/13 19:39:26  soliday
# Commited old changes
#
# Revision 1.79  2002/10/22 16:49:58  soliday
# Changed error message email address
#
# Revision 1.78  2002/10/22 16:42:16  soliday
# Fixed a few bugs in the MCR
#
# Revision 1.77  2002/10/22 15:38:23  soliday
# Fixed an error message
#
# Revision 1.76  2002/10/10 21:25:24  soliday
# Commited changes after PEM studies.
#
# Revision 1.75  2002/07/12 19:04:41  soliday
# Made changes made during studies period.
#
# Revision 1.74  2002/07/11 20:04:10  soliday
# Added the ability to set the mode on the modulators.
#
# Revision 1.73  2002/05/29 16:52:51  soliday
# Commited changes made during PEM studies.
#
# Revision 1.72  2002/05/23 21:48:31  soliday
# Temporary version.
#
# Revision 1.71  2002/03/04 16:14:17  soliday
# It now restores the PAR timing.
#
# Revision 1.70  2002/02/19 16:11:31  soliday
# The minimum sector forward power is read from the rfg smart monitor.
#
# Revision 1.69  2002/02/19 04:38:36  borland
# Now requires only 5.2MW for L1 klystron forward power for RG1.
#
# Revision 1.68  2001/11/27 16:13:57  borland
# Modified check of heater power in RFGUN_Startup.  Now triggered on values of
# 9W or less.  Dialog gives user a choice to abort or continue.
#
# Revision 1.67  2001/08/15 22:41:25  soliday
# Fixed the cathode heater problem. The Query PV is backwards. I will have to
# modify code if it is fixed.
#
# Revision 1.66  2001/08/08 23:44:10  soliday
# Added RFGUN+LMOCS_SwitchWithPS
#
# Revision 1.65  2001/05/17 17:42:00  soliday
# The klystron min forward power for RG1 is now 10.0e6
#
# Revision 1.64  2001/05/17 16:51:12  soliday
# It now disables the RF Gun Monitor instead of just Suspending it.
#
# Revision 1.63  2001/03/01 20:05:10  soliday
# Added the ability to select individual power supplies.
#
# Revision 1.62  2001/02/22 19:00:21  soliday
# Changed all the PEMS to use the LPL SCR files.
#
# Revision 1.61  2001/02/07 21:52:00  soliday
# Did more pem testing. Also used procheck to clean up code.
#
# Revision 1.60  2001/01/24 21:58:40  soliday
# Tested RFGUN+LMOCS startup and shutdown.
#
# Revision 1.59  2000/12/04 22:52:11  soliday
# New versions tested in control room.
#
# Revision 1.58  2000/09/29 18:08:34  soliday
# Added RFGUN+LMOCS_StartupWithPS and changed RFGUN_Startup so that it
# resets the Kicker interlock.
#
# Revision 1.57  2000/08/13 13:05:47  borland
# Added more diagnostics for SCR problems.  Added processing to remove invalid
# lines from several instances where burtwb is used.
#
# Revision 1.56  2000/07/28 14:22:25  soliday
# No longer turns the kicker on. It leaves this up to the operator.
#
# Revision 1.55  2000/07/27 20:47:14  soliday
# The Kicker charge pulse is turned off and the kicker itself is only turned on
# when the rfgun startup pem is run.
#
# Revision 1.54  2000/07/26 21:33:01  soliday
# Fixed bug with cavput call
#
# Revision 1.53  2000/07/25 19:34:48  soliday
# Added L3 power supplies
#
# Revision 1.52  2000/06/27 15:13:00  borland
# Now degausses RG2 alpha magnet when starting up RG1.
#
# Revision 1.51  2000/05/24 21:52:00  soliday
# Fixed error message that incorrectly said it was from RFGUN_Shutdown.
#
# Revision 1.50  2000/05/15 17:57:03  soliday
# Switch the order that procedures are joined.
#
# Revision 1.49  2000/05/11 16:55:55  soliday
# The gate start time and gate width are not restored with the burt call.
#
# Revision 1.48  2000/05/11 16:32:44  soliday
# Added Timing catagory to the burt command.
#
# Revision 1.47  2000/05/02 20:44:36  soliday
# New version for new PVs.
#
# Revision 1.46  2000/03/20 20:58:41  soliday
# Added pop up dialog for SCR restore warnings.
#
# Revision 1.45  2000/03/15 00:18:17  soliday
# Now works in bypass par mode.
#
# Revision 1.44  2000/03/06 16:41:49  soliday
# I now use sddsselect to filter the SCR file.
#
# Revision 1.43  2000/03/06 16:15:01  soliday
# Changed SteeringSetPoint to SteeringSetpoint.
#
# Revision 1.42  2000/02/10 21:33:12  soliday
# Improved error message when Cathode heater power is too low.
#
# Revision 1.41  2000/01/27 18:38:43  soliday
# Added some procedures that will allow startup of the PAR at the same time.
#
# Revision 1.40  2000/01/21 20:51:55  soliday
# Fixed bug with restoring PFN for modulators.
#
# Revision 1.39  1999/12/13 19:59:12  soliday
# Added description lines
#
# Revision 1.38  1999/12/13 15:19:38  soliday
# Changed typo of APSMpResult to APSMpReturn
#
# Revision 1.37  1999/12/10 20:47:55  soliday
# Now does a burtwb call twice because of goofy problems
#
# Revision 1.36  1999/12/06 20:29:46  soliday
# Now does a restore operation
#
# Revision 1.35  1999/11/10 20:24:10  soliday
# Finished testing new code
#
# Revision 1.34  1999/11/08 22:30:11  soliday
# Now works with new Modulator L1
#
# Revision 1.33  1999/09/22 16:26:09  soliday
# Modified the display when in manual mode
#
# Revision 1.32  1999/09/18 03:57:50  soliday
# It now displays the value it waiting for with the Klystron fwd power
#
# Revision 1.31  1999/09/01 14:53:29  soliday
# Improved the method used to start the rfGunInterlock and fixedCurrent
#
# Revision 1.30  1999/08/26 22:26:11  soliday
# Moved APSLINAC_CheckModulatorCurrents from PCGUN_Startup to here
#
# Revision 1.29  1999/08/26 21:32:52  soliday
# Added RFGUN_StartupWithPS
#
# Revision 1.28  1999/08/11 16:37:33  soliday
# This procedure can now be called by others
#
# Revision 1.27  1999/08/10 14:36:27  borland
# Use caput instead of cavput in RFGUN_ResetTimingSystem, because the latter
# gets a CA problem for those PVs.
#
# Revision 1.26  1999/08/10 14:18:05  borland
# Added a missing exec for rf gun timing reset.
#
# Revision 1.25  1999/08/09 16:42:17  soliday
# Added the ResetTimingSystem procedure
#
# Revision 1.24  1999/08/09 16:27:11  soliday
# Changed power target for RFG2 to 13.00 MW from 13.92 MW
#
# Revision 1.23  1999/08/06 18:14:27  borland
# No longer sets the alpha magnet current.  No longer turns the kicker on.
#
# Revision 1.22  1999/08/06 14:36:56  borland
# Changed power target for RFG2 to 13.92 MW from 16.5MW.
#
# Revision 1.21  1999/08/02 18:12:16  borland
# Added all modules to makefile.
# Startup scripts using new APSListSCRMatchSetArray, giving access to
# preferred/reference files.
#
# Revision 1.20  1999/06/24 15:40:52  soliday
# It now restores values from the SCR file.
#
# Revision 1.19  1999/06/19 17:40:01  soliday
# Changed timeout for Kicker on button
#
# Revision 1.18  1999/06/04 18:02:50  soliday
# Changed cathode Heather current to 2.21 from 2.2
#
# Revision 1.17  1999/06/04 17:53:45  soliday
# more changes requested by pasky
#
# Revision 1.16  1999/04/29 16:08:44  soliday
# changed cathode heater current to 2.2 A
#
# Revision 1.15  1999/04/28 15:24:42  soliday
# It now checks the PFN Voltage and the "Ready to Pulse" indicator
#
# Revision 1.14  1999/04/02 21:40:14  soliday
# Some minor modifications
#
# Revision 1.13  1999/04/02 21:05:57  soliday
# Tested RG1 code
#
# Revision 1.12  1999/03/26 20:17:52  soliday
# Altered setpoints for alpha magnets
#
# Revision 1.11  1999/03/18 18:35:08  soliday
# First working version after pv name changes
#
# Revision 1.10  1999/03/18 18:16:08  soliday
# fixed another pv problem
#
# Revision 1.9  1999/03/18 18:00:36  soliday
# fixed gate valve problem
# Revision 1.8  1999/03/18 16:47:13  soliday
# pv state changed so had to change the software
#
# Revision 1.7  1999/03/16 17:13:28  soliday
# Converted procedures to work with pv name changes.
#

# RFGun Startup from Standby of Fault Mode.
# This procedure is to be used when restoring the RFGun to
# operation from standby mode or in a case when you would
# experience a vacuum, BL or TW trip or rad monitor trip.
#
# Warning: Make sure the RFGun Interlock Monitor is no longer running

proc RFGUN_StartupInfo {} {
    return "RFGun Startup from Standby or Fault Mode.\
This procedure is to be used when restoring the RFGun to\
operation from standby mode or in a case when you would\
experience a vacuum, BL or TW trip or rad monitor trip."
}

proc RFGUN_Startup {args} {
    set SCRFile ""
    set CathodeHeaterSetpoint ""
    set CathodeHeaterWatts ""
    set KLYFwdPwr ""
    set KickerSetpoint ""
    set resumeRFPowerControlLaws 1
    set bypassMode 0
    set resumeSmartMonitor 1
    set high 0
    set sectors "L1 L2 L3 L4 L5"
    set changeBypassState 0
    set changeMode 0
    set skipLLRFChecks 0
    foreach sector $sectors {
	set $sector 1
    }

    global execMode

    if {$execMode == "Manual"} {
	set prefix "Press Continue to "
    } else {
	set prefix ""
    }

    if {[catch {APSLinkToLINACPVs} result]} {
	APSMpReturn error "RFGUN_Startup: $result"
    }

    if {[catch {APSRFGUN_DetectSelectedGun} gun]} {
	APSMpReturn error "RFGUN_Startup: $gun"
    }
    APSParseArguments {gun SCRFile high L1 L2 L3 L4 L5 skipLLRFChecks}
    if {$gun == "RG2"} {
	set deviceList 2
	set KLYFwdPwrMin 8.0e6
#	if {[catch {exec cavget -list=L:RFG:smartMonitorParm7C -pend=20} SectFwdPwrMin]} {
#	    set SectFwdPwrMin 8e+06
#	}
    } elseif {$gun == "RG1"} {
	set deviceList 1
	set KLYFwdPwrMin 8.0e6
	#if {[catch {exec cavget -list=L:RFG:smartMonitorParm6C -pend=20} SectFwdPwrMin]} {
	#    set SectFwdPwrMin 5e+06
	#}
#        set SectFwdPwrMin 6.8e+06
    } else {
	APSMpReturn error "RFGUN_Startup: Unable to detect which Gun is selected. Check the LINAC RF Switching System State to ensure it is set to RUNNING and not LOCAL SWITCH or REMOTE SWITCH."
    }
#    set SectFwdPwrMin [expr {$SectFwdPwrMin + 100000}]
    global APSMpLControlPS
    
    set arrayName APSMpLControlPS
    
    APSLGenericPSArraySetup -arrayName $arrayName -SCRFile $SCRFile

    if {(![file exists $SCRFile]) && ($execMode == "Automatic")} {
	set execMode "Semi-Automatic"
    }

    APSMpStep init -promptDialog \
      "APSLGenericPSDialog -chooseSCR 1 -arrayName $arrayName -frame "
    
    set SCRFile $APSMpLControlPS(Filename)
    if {![file exists $SCRFile]} {
	APSMpReturn error "RFGUN_Startup: No SCR file selected"
    }
    if {[catch {exec sdds2stream $SCRFile -parameter=SnapshotDescription -page=1 -noquotes} results]} {
	APSMpReturn error "RFGUN_Startup: Invalid SCR file"
    }    
    if {[string first "Limited save:" $results] != -1} {
        if {[catch {exec sddssort $SCRFile -pipe=out -column=Beamline -unique | sdds2stream -pipe=in -col=Beamline} results]} {
            APSMpReturn error "RFGUN_Startup: Problem reading beamlines categories from the SCR file"
        } else {
            if {([lsearch -exact $results L1] == -1) || \
                  ([lsearch -exact $results L2] == -1) || \
                  ([lsearch -exact $results L3] == -1) || \
                  ([lsearch -exact $results L4] == -1) || \
                  ([lsearch -exact $results L5] == -1) || \
                  ([lsearch -exact $results rfGun] == -1) || \
                  ([lsearch -exact $results rfGun1] == -1) || \
                  ([lsearch -exact $results rfGun2] == -1)} {
                APSMpReturn error "RFGUN_Startup: LIMITED SAVE SCR file does not include all the Linac and rfGun beamline categories"
            }
        }
    }

    if {[catch {exec sddsselect $SCRFile /home/helios/oagData/pem/linac/filterFile -match=ControlName -pipe=out | sdds2stream -column=ControlName,ValueString -pipe=in} results]} {
	APSMpReturn error "RFGUN_Startup: Invalid SCR file"
    }

    set gateStartTime 0
    set gateWidth 0
    set rfgForwardPowerSCR 0
    foreach {controlName valueString} $results {
        if {[string range $controlName end-3 end] == ".VAL"} {
            set controlName [string range $controlName 0 end-4]
        }
	if {$controlName == "L1:${gun}:HTR:CurrentAO"} {
	    set CathodeHeaterSetpoint [expr 2.75 * $valueString]
	} elseif {$controlName == "L1:${gun}:HTR:VoltageAO"} {
	    set CathodeHeaterSetpoint $valueString
	} elseif {$controlName == "L1:${gun}:HTR:PowerCC"} {
	    set CathodeHeaterWatts $valueString
	} elseif {$controlName == "L1:KY:DC2ARF"} {
	    set KLYFwdPwr $valueString
	} elseif {$controlName == "L3:KY:DC2ARF"} {
	    set KLYFwdPwr3 $valueString
	} elseif {$controlName == "L1:${gun}:KIK:VoltSetSendAO"} {
	    set KickerSetpoint $valueString
	} elseif {"L1:TM:RFG:RF:startTimeAO" == $controlName} {
	    set gateStartTime $valueString
	} elseif {"L1:TM:RFG:RF:widthAO" == $controlName} {
	    set gateWidth $valueString
	} elseif {"L:BPD:L1P1:RFGUN1:SetpointAO" == $controlName} {
	    set rg1phasesetpoint $valueString
	} elseif {"L:BPD:L1P1:RFGUN2:SetpointAO" == $controlName} {
	    set rg2phasesetpoint $valueString
	} elseif {"L:BPD:L1P1:L2AS1:SetpointAO" == $controlName} {
	    set l2phasesetpoint $valueString
	} elseif {"L:BPD:L3P3:L4AS1:SetpointAO" == $controlName} {
	    set l4phasesetpoint $valueString
	} elseif {"L:BPD:L3P3:L4AS1:SetpointAO" == $controlName} {
	    set l5phasesetpoint $valueString
	} elseif {"L1:RFG:DC2ARF" == $controlName} {
	    set rfgForwardPowerSCR $valueString
	} elseif {"L1:RFG:DC2ARF.VAL" == $controlName} {
	    set rfgForwardPowerSCR $valueString
	}
    }

    APSStrictParseArguments {CathodeHeaterSetpoint CathodeHeaterWatts KLYFwdPwr KickerSetpoint resumeRFPowerControlLaws bypassMode resumeSmartMonitor changeBypassState changeMode}

    if {[catch {APSDevSend \
                  -group LinacRFGun \
                  -operation ReadRFCurrentMode \
                  -deviceList 0} linacMode]} {
	APSMpReturn error "RFGUN_Startup: $linacMode"
    }
    if {[join $linacMode] == "K1 Down"} {
        set L1SectorSource 3
        set L2SectorSource 2
        set KLYFwdPwr $KLYFwdPwr3
    } elseif {[join $linacMode] == "K2 Down"} {
        set L1SectorSource 1
        set L2SectorSource 3
    } else {
        set L1SectorSource 1
        set L2SectorSource 2
    }

    if {($CathodeHeaterSetpoint == "")} {
	APSMpReturn error "RFGUN_Startup: No Cathode Heater Voltage setpoint found. Invalid SCR file"
    }
    if {($CathodeHeaterWatts == "")} {
	APSMpReturn error "RFGUN_Startup: No Cathode Heater Power setpoint found. Invalid SCR file"
    }
    if {($KLYFwdPwr == "")} {
	APSMpReturn error "RFGUN_Startup: No Klystron Forward Power setpoint found. Invalid SCR file"
    }
    if {$CathodeHeaterWatts < 9} {
        APSMpStep "Cathode Heater Power Warning" -desc "Cathode Heater Power Warning"
        if {[APSPEMTKDialog .warningDialog -text "The heater power level in the SCR file you chose is only $CathodeHeaterWatts, which may indicate an inappropriate SCR file was chosen."] == 1} {
            APSMpReturn error "RFGUN_Startup: Invalid SCR file (heater power too low)"
        }
    }

    set CathodeHeaterWattsMin [expr {$CathodeHeaterWatts * .5}]
    set CathodeHeaterWattsMax [expr {$CathodeHeaterWatts * 1.042}]


    set klystronList ""
    foreach sector "1 2 3" {
        if {[set L$sector]} {
            append klystronList "$sector "
        }
    }
    if {[llength $klystronList]} {
        APSMpStep "${prefix}Suspend $klystronList Power Control Stabilizer" -desc "Suspend L1, L2 and L3 power control stabilizers"
        if {[catch {APSLSuspendResumeRFPowerControllaws -klystronList $klystronList -mode suspend} result]} {
            if {[APSPEMTKDialog .warningDialog -text "Error while suspending $klystronList power controllaws \n$result"] == 1} {
                APSMpReturn error "RFGUN_Startup: $result"
            }
        }
    }
    
    APSMpStep "${prefix}Suspend linac RF phase controllaw" -desc "Suspend linac RF phase controllaw"
    if {[catch {exec cavput -list=L:BPD:RFGUN1:L1L2L4L5RC.SUSP=1,L:BPD:RFGUN1:L1L2L4L5RC.SUSP=1} result]} {
        if {[APSPEMTKDialog .warningDialog -text "Problem suspending the linac RF phase controllaw\nRFGUN_Startup: $result"] == 1} {
            APSMpReturn error "Problem suspending the linac RF phase controllaw: RFGUN_Startup: $result"
        }
    }
    if {[catch {exec cavput -list=L:BPD:RFGUN1RC.ABRT=1,L:BPD:RFGUN2RC.ABRT=1,L:BPD:RFGUN2K1DnRC.ABRT=1,L:BPD:RFGUN2K2DnRC.ABRT=1,L:BPD:L3P3:L4AS1RC.ABRT=1,L:BPD:L4P1:L5AS1RC.ABRT=1} result]} {
        if {[APSPEMTKDialog .warningDialog -text "Problem suspending the linac RF phase controllaw\nRFGUN_Startup: $result"] == 1} {
            APSMpReturn error "Problem suspending the linac RF phase controllaw: RFGUN_Startup: $result"
        }
    }

    APSMpStep "${prefix}Set script control button Available" -desc "Set script control button to Available"
    if {[catch {APSDevSend \
                  -group LinacRFGun \
                  -operation SetActiveScript \
                  -deviceList 0 \
                  -value "Available"} result]} {
        if {[APSPEMTKDialog .warningDialog -text "Error while setting the script control button to Available \n$result"] == 1} {
            APSMpReturn error "RFGUN_Startup: $result"
        }
    }
    
    #modify here
    if {[join $linacMode] != "K1 Down"} {
        if {[catch {APScavget -list=L1:MO:LowPFNvSP} PFNLowTripValue]} {
            set PFNLowTripValue 22
        }
	APSMpStep "${prefix}Check PFN voltage for L1" -desc "Check that PFN voltage for L1 is above $PFNLowTripValue volts"
	if {[catch {APSLINAC_CheckModulatorCurrents -deviceList "1" -minValueList "$PFNLowTripValue"} result]} {
	    if {$execMode != "Automatic"} {
		bell
                if {[APSPEMTKDialog .warningDialog -text "L1 PFN voltage is less then $PFNLowTripValue Volts."] == 1} {
		    APSMpReturn error "RFGUN_Startup: Operation aborted by user"
		}
	    }
	}
    }
    #L1SectorSource can only be 1 or 3
    if {[catch {APSDevSend \
                  -group LinacKlystron \
                  -operation QueryACISPermit400WAmp \
                  -deviceList $L1SectorSource} permit]} {
        APSMpReturn error "RFGUN_Startup: $permit"
    }
    if {[catch {APSDevSend \
                  -group LinacKlystron \
                  -operation QueryACISLatch400WAmp \
                  -deviceList $L1SectorSource} latch]} {
        APSMpReturn error "RFGUN_Startup: $latch"
    }
    if {(!$permit) || (!$latch)} {
        if {[catch {APSDevSend \
                      -group LinacKlystron \
                      -operation ResetACIS400WAmp \
                      -deviceList $L1SectorSource -releaseLinks 1} result]} {
            APSMpReturn error "RFGUN_Startup: $result"
        }
        APSWaitWithUpdate -waitSeconds 2
    }
    if {[catch {APSDevSend \
                  -group LinacKlystron \
                  -operation Query400WAmpOn \
                  -deviceList $L1SectorSource} result]} {
        APSMpReturn error "RFGUN_Startup: $result"
    }
    if {!$result} {
        APSMpStep "${prefix}Turn 400W RF Amp On" -desc "Reset 400 watt amp if needed"
        if {[catch {APSDevSend \
                      -group LinacKlystron \
                      -operation Reset400WAmp \
                      -deviceList $L1SectorSource -releaseLinks 1} result]} {
            APSMpReturn error "RFGUN_Startup: $result"
        }
    }
    #L1SectorSource can only be 1 or 3
    APSMpStep "${prefix}Set Klystron Forward Power" -desc "Set Klystron forward power"
    if {[catch {APSLINAC_SetKlystronForwardPower \
                  -deviceList $L1SectorSource \
                  -KLYFwdPwr $KLYFwdPwr} result]} {
        if {[APSPEMTKDialog .warningDialog -text "Error while setting klystron forward power \n$result"] == 1} {
            APSMpReturn error "RFGUN_Startup: $result"
        }
    }

    APSMpStep "${prefix}Turn on heater power" -desc "Turn the heater on"
    if {[catch {APSDevSend \
                  -group LinacRFGun \
                  -operation QueryHeaterPowerOn \
                  -deviceList $deviceList} result]} {
	APSMpReturn error "RFGUN_Startup: $result"
    }
    #This pv is backwards
    if {$result} {
	if {[catch {APSDevSend \
                      -group LinacRFGun \
                      -operation TurnOnHeaterPower \
                      -deviceList $deviceList} result]} {
	    APSMpReturn error "RFGUN_Startup: $result"
	}
        if {[catch {APSRFGUN_SetCathodeHeaterCurrent \
                      -value 0.0 \
                      -deviceList $deviceList} result]} {
            APSMpReturn error "RFGUN_Startup: $result"
        }
        set changeMode 0
        if {[catch {APSDevSend \
                      -group LinacRFGun \
                      -operation QueryHeaterPowerOn \
                      -deviceList $deviceList} result]} {
            APSMpReturn error "RFGUN_Startup: $result"
        }
        set steps 0
        while {$result} {
            incr steps
            if {$steps > 20} {
                if {[APSPEMTKDialog .warningDialog -text "Please turn on the cathode heater power supply."] == 1} {
                    APSMpReturn error "RFGUN_Startup: Operation aborted by user"
                }
                break
            }
            after 1000
            if {[catch {APSDevSend \
                          -group LinacRFGun \
                          -operation QueryHeaterPowerOn \
                          -deviceList $deviceList} result]} {
                APSMpReturn error "RFGUN_Startup: $result"
            }
        }
    }

    if {!$skipLLRFChecks} {
        APSMpStep "${prefix}Check if L$L1SectorSource LLRF is on" -desc "Check if L1 is on"
        if [catch {APSLINAC_CheckLLRFStatus -desiredState on -deviceList $L1SectorSource} result] {
            if {[APSPEMTKDialog .warningDialog -text "L$L1SectorSource LLRF is not on"] == 1} {
                APSMpReturn error "RFGUN_Startup: $result"
            }
        }
    }

    if {[catch {APScavput -list=L1:RG${deviceList}:HTR:ACISResetSEQ.PROC=1} result]} {
        if {[APSPEMTKDialog .warningDialog -text "Problem occured while resetting the ACIS Interlock: $result"] == 1} {
            APSMpReturn error "RFGUN_Startup: $result"
        }
    }

    if {!$changeMode} {
        if {[catch {exec cavget -list=L1:RG${deviceList}:HTR:VoltageAO} cathodeHeaterVoltageValue]} {
            set cathodeHeaterVoltageValue 0
        }
        if {$cathodeHeaterVoltageValue < [expr {$CathodeHeaterSetpoint / 8.0}]} {
            APSMpStep "${prefix}Set RFGun cathode heater current to 1/8 value" \
              -desc "Set RFGun cathode heater current to 1/8 value"
            if {[catch {APSRFGUN_SetCathodeHeaterCurrent \
                          -value [expr {$CathodeHeaterSetpoint / 8.0}] \
                          -deviceList $deviceList} result]} {
                if {[APSPEMTKDialog .warningDialog -text "Error while setting RFGun cathode heater current to 1/8 value \n$result"] == 1} {
                    APSMpReturn error "RFGUN_Startup: $result"
                }
            }
            
            if {$bypassMode} {
                for {set i 0} {$i < 3} {incr i} {
                    APSMpStep "${prefix}Wait 30 seconds for cathode heater current to stabilize" \
                      -desc "Wait 30 seconds for cathode heater current to stabilize"
                    APSWaitWithUpdate -waitSeconds 10
                    if {!$skipLLRFChecks} {
                        if [catch {APSLINAC_CheckLLRFStatus -desiredState on -deviceList $L1SectorSource} result] {
                            if {[APSPEMTKDialog .warningDialog -text "L$L1SectorSource LLRF is not on"] == 1} {
                                APSMpReturn error "RFGUN_Startup: $result"
                            }
                        }
                    }
                }
            } else {
                for {set i 0} {$i < 3} {incr i} {
                    APSMpStep "${prefix}Wait 15 seconds for cathode heater current to stabilize" \
                      -desc "Wait 15 seconds for cathode heater current to stabilize"
                    APSWaitWithUpdate -waitSeconds 5
                    if {!$skipLLRFChecks} {
                        if [catch {APSLINAC_CheckLLRFStatus -desiredState on -deviceList $L1SectorSource} result] {
                            if {[APSPEMTKDialog .warningDialog -text "L$L1SectorSource LLRF is not on"] == 1} {
                                APSMpReturn error "RFGUN_Startup: $result"
                            }
                        }
                    }
                }
            }
        }
        if {$cathodeHeaterVoltageValue < [expr {$CathodeHeaterSetpoint / 4.0}]} {
            APSMpStep "${prefix}Set RFGun cathode heater current to 1/4 value" \
              -desc "Set RFGun cathode heater current to 1/4 value"
            if {[catch {APSRFGUN_SetCathodeHeaterCurrent \
                          -value [expr {$CathodeHeaterSetpoint / 4.0}] \
                          -deviceList $deviceList} result]} {
                if {[APSPEMTKDialog .warningDialog -text "Error while setting RFGun cathode heater current to 1/4 value \n$result"] == 1} {
                    APSMpReturn error "RFGUN_Startup: $result"
                }
            }
            
            if {$bypassMode} {
                for {set i 0} {$i < 3} {incr i} {
                    APSMpStep "${prefix}Wait 30 seconds for cathode heater current to stabilize" \
                      -desc "Wait 30 seconds for cathode heater current to stabilize"
                    APSWaitWithUpdate -waitSeconds 10
                    if {!$skipLLRFChecks} {
                        if [catch {APSLINAC_CheckLLRFStatus -desiredState on -deviceList $L1SectorSource} result] {
                            if {[APSPEMTKDialog .warningDialog -text "L$L1SectorSource LLRF is not on"] == 1} {
                                APSMpReturn error "RFGUN_Startup: $result"
                            }
                        }
                    }
                }
            } else {
                for {set i 0} {$i < 3} {incr i} {
                    APSMpStep "${prefix}Wait 15 seconds for cathode heater current to stabilize" \
                      -desc "Wait 15 seconds for cathode heater current to stabilize"
                    APSWaitWithUpdate -waitSeconds 5
                    if {!$skipLLRFChecks} {
                        if [catch {APSLINAC_CheckLLRFStatus -desiredState on -deviceList $L1SectorSource} result] {
                            if {[APSPEMTKDialog .warningDialog -text "L$L1SectorSource LLRF is not on"] == 1} {
                                APSMpReturn error "RFGUN_Startup: $result"
                            }
                        }
                    }
                }
            }
        }


        if {$cathodeHeaterVoltageValue < [expr {3.0 * $CathodeHeaterSetpoint / 8.0}]} {
            APSMpStep "${prefix}Set RFGun cathode heater current to 3/8 value" \
              -desc "Set RFGun cathode heater current to 3/8 value"
            if {[catch {APSRFGUN_SetCathodeHeaterCurrent \
                          -value [expr {3.0 * $CathodeHeaterSetpoint / 8.0}] \
                          -deviceList $deviceList} result]} {
                if {[APSPEMTKDialog .warningDialog -text "Error while setting RFGun cathode heater current to 3/8 value \n$result"] == 1} {
                    APSMpReturn error "RFGUN_Startup: $result"
                }
            }
            
            if {$bypassMode} {
                for {set i 0} {$i < 3} {incr i} {
                    APSMpStep "${prefix}Wait 30 seconds for cathode heater current to stabilize" \
                      -desc "Wait 30 seconds for cathode heater current to stabilize"
                    APSWaitWithUpdate -waitSeconds 10
                    if {!$skipLLRFChecks} {
                        if [catch {APSLINAC_CheckLLRFStatus -desiredState on -deviceList $L1SectorSource} result] {
                            if {[APSPEMTKDialog .warningDialog -text "L$L1SectorSource LLRF is not on"] == 1} {
                                APSMpReturn error "RFGUN_Startup: $result"
                            }
                        }
                    }
                }
            } else {
                for {set i 0} {$i < 3} {incr i} {
                    APSMpStep "${prefix}Wait 15 seconds for cathode heater current to stabilize" \
                      -desc "Wait 15 seconds for cathode heater current to stabilize"
                    APSWaitWithUpdate -waitSeconds 5
                    if {!$skipLLRFChecks} {
                        if [catch {APSLINAC_CheckLLRFStatus -desiredState on -deviceList $L1SectorSource} result] {
                            if {[APSPEMTKDialog .warningDialog -text "L$L1SectorSource LLRF is not on"] == 1} {
                                APSMpReturn error "RFGUN_Startup: $result"
                            }
                        }
                    }
                }
            }
        }



        if {$cathodeHeaterVoltageValue < [expr {$CathodeHeaterSetpoint / 2.0}]} {
            APSMpStep "${prefix}Set RFGun cathode heater current to half value" \
              -desc "Set RFGun cathode heater current to half value"
            if {[catch {APSRFGUN_SetCathodeHeaterCurrent \
                          -value [expr {$CathodeHeaterSetpoint / 2.0}] \
                          -deviceList $deviceList} result]} {
                if {[APSPEMTKDialog .warningDialog -text "Error while setting RFGun cathode heater current to half value \n$result"] == 1} {
                    APSMpReturn error "RFGUN_Startup: $result"
                }
            }
            if {$bypassMode} {
                for {set i 0} {$i < 3} {incr i} {
                    APSMpStep "${prefix}Wait 30 seconds for cathode heater current to stabilize" \
                      -desc "Wait 30 seconds for cathode heater current to stabilize"
                    APSWaitWithUpdate -waitSeconds 10
                    if {!$skipLLRFChecks} {
                        if [catch {APSLINAC_CheckLLRFStatus -desiredState on -deviceList $L1SectorSource} result] {
                            if {[APSPEMTKDialog .warningDialog -text "L$L1SectorSource LLRF is not on"] == 1} {
                                APSMpReturn error "RFGUN_Startup: $result"
                            }
                        }
                    }
                }
            } else {
                for {set i 0} {$i < 3} {incr i} {
                    APSMpStep "${prefix}Wait 15 seconds for cathode heater current to stabilize" \
                      -desc "Wait 15 seconds for cathode heater current to stabilize"
                    APSWaitWithUpdate -waitSeconds 5
                    if {!$skipLLRFChecks} {
                        if [catch {APSLINAC_CheckLLRFStatus -desiredState on -deviceList $L1SectorSource} result] {
                            if {[APSPEMTKDialog .warningDialog -text "L$L1SectorSource LLRF is not on"] == 1} {
                                APSMpReturn error "RFGUN_Startup: $result"
                            }
                        }
                    }
                }
            }
        }


        if {$cathodeHeaterVoltageValue < [expr {5.0 * $CathodeHeaterSetpoint / 8.0}]} {
            APSMpStep "${prefix}Set RFGun cathode heater current to 5/8 value" \
              -desc "Set RFGun cathode heater current to 5/8 value"
            if {[catch {APSRFGUN_SetCathodeHeaterCurrent \
                          -value [expr {5.0 * $CathodeHeaterSetpoint / 8.0}] \
                          -deviceList $deviceList} result]} {
                if {[APSPEMTKDialog .warningDialog -text "Error while setting RFGun cathode heater current to 5/8 value \n$result"] == 1} {
                    APSMpReturn error "RFGUN_Startup: $result"
                }
            }
            
            if {$bypassMode} {
                for {set i 0} {$i < 3} {incr i} {
                    APSMpStep "${prefix}Wait 30 seconds for cathode heater current to stabilize" \
                      -desc "Wait 30 seconds for cathode heater current to stabilize"
                    APSWaitWithUpdate -waitSeconds 10
                    if {!$skipLLRFChecks} {
                        if [catch {APSLINAC_CheckLLRFStatus -desiredState on -deviceList $L1SectorSource} result] {
                            if {[APSPEMTKDialog .warningDialog -text "L$L1SectorSource LLRF is not on"] == 1} {
                                APSMpReturn error "RFGUN_Startup: $result"
                            }
                        }
                    }
                }
            } else {
                for {set i 0} {$i < 3} {incr i} {
                    APSMpStep "${prefix}Wait 15 seconds for cathode heater current to stabilize" \
                      -desc "Wait 15 seconds for cathode heater current to stabilize"
                    APSWaitWithUpdate -waitSeconds 5
                    if {!$skipLLRFChecks} {
                        if [catch {APSLINAC_CheckLLRFStatus -desiredState on -deviceList $L1SectorSource} result] {
                            if {[APSPEMTKDialog .warningDialog -text "L$L1SectorSource LLRF is not on"] == 1} {
                                APSMpReturn error "RFGUN_Startup: $result"
                            }
                        }
                    }
                }
            }
        }



        if {$cathodeHeaterVoltageValue < [expr {$CathodeHeaterSetpoint / 1.33}]} {
            APSMpStep "${prefix}Set RFGun cathode heater current to 3/4 value" \
              -desc "Set RFGun cathode heater current to 3/4 value"
            if {[catch {APSRFGUN_SetCathodeHeaterCurrent \
                          -value [expr {$CathodeHeaterSetpoint / 1.33}] \
                          -deviceList $deviceList} result]} {
                if {[APSPEMTKDialog .warningDialog -text "Error while setting RFGun cathode heater current to 3/4 value \n$result"] == 1} {
                    APSMpReturn error "RFGUN_Startup: $result"
                }
            }
            for {set i 0} {$i < 3} {incr i} {
                APSMpStep "${prefix}Wait 15 seconds for cathode heater current to stabilize" -desc "Wait 30 seconds for cathode heater current to stabilize"
                APSWaitWithUpdate -waitSeconds 5
                if {!$skipLLRFChecks} {
                    if [catch {APSLINAC_CheckLLRFStatus -desiredState on -deviceList $L1SectorSource} result] {
                        if {[APSPEMTKDialog .warningDialog -text "L$L1SectorSource LLRF is not on"] == 1} {
                            APSMpReturn error "RFGUN_Startup: $result"
                        }
                    }
                }
            }
        }


        if {$cathodeHeaterVoltageValue < [expr {7.0 * $CathodeHeaterSetpoint / 8.0}]} {
            APSMpStep "${prefix}Set RFGun cathode heater current to 7/8 value" \
              -desc "Set RFGun cathode heater current to 3/4 value"
            if {[catch {APSRFGUN_SetCathodeHeaterCurrent \
                          -value [expr {7.0 * $CathodeHeaterSetpoint / 8.0}] \
                          -deviceList $deviceList} result]} {
                if {[APSPEMTKDialog .warningDialog -text "Error while setting RFGun cathode heater current to 7/8 value \n$result"] == 1} {
                    APSMpReturn error "RFGUN_Startup: $result"
                }
            }
            for {set i 0} {$i < 3} {incr i} {
                APSMpStep "${prefix}Wait 15 seconds for cathode heater current to stabilize" -desc "Wait 30 seconds for cathode heater current to stabilize"
                APSWaitWithUpdate -waitSeconds 5
                if {!$skipLLRFChecks} {
                    if [catch {APSLINAC_CheckLLRFStatus -desiredState on -deviceList $L1SectorSource} result] {
                        if {[APSPEMTKDialog .warningDialog -text "L$L1SectorSource LLRF is not on"] == 1} {
                            APSMpReturn error "RFGUN_Startup: $result"
                        }
                    }
                }
            }
        }
    }



    APSMpStep "${prefix}Set RFGun cathode heater current" -desc "Set RFGun cathode heater current"
    if {[catch {APSRFGUN_SetCathodeHeaterCurrent \
                  -value $CathodeHeaterSetpoint \
                  -deviceList $deviceList} result]} {
        if {[APSPEMTKDialog .warningDialog -text "Error while setting RFGun cathode heater current \n$result"] == 1} {
            APSMpReturn error "RFGUN_Startup: $result"
        }
    }

    APSWaitWithUpdate -waitSeconds 5
    if {[catch {APSDevSend \
                  -group LinacRFGun \
                  -operation ReadCathodeHeaterCurrent \
                  -deviceList $deviceList} result]} {
	APSMpReturn error "RFGUN_Startup: $result"
    }
    if {$result < 0.1} {
        if {[APSPEMTKDialog .warningDialog -text "Please reset RFgun Smart Monitor and turn on the cathode heater power supply."] == 1} {
	    APSMpReturn error "RFGUN_Startup: Operation aborted by user"
	}
    }
        
    APSMpStep "${prefix}Select LLRF ON" -desc "Turn L${L1SectorSource} LLRF on"
    if {[catch {APSLINAC_LLRFOnOff -button "On" -deviceList $L1SectorSource} result]} {
	if {[APSPEMTKDialog .warningDialog -text "Error while turning on LLRF for sector $L1SectorSource \n$result"] == 1} {
	    APSMpReturn error "RFGUN_Startup: $result"
	}
    }

    #APSMpStep "Resume power control scripts for sector $L1SectorSource" -desc "Resume power control script"
    #if {[catch {LMOCS_PowerControlLawSetup -mode high -deviceList L$L1SectorSource} result]} {
#	if {[APSPEMTKDialog .warningDialog -text "Error while setting L$L1SectorSource power controllaw into high mode \n$result"] == 1} {
#	    APSMpReturn error "RFGUN_Startup: $result"
#	}
#    }
    #if {[catch {APSLSuspendResumeRFPowerControllaws -klystronList $L1SectorSource -mode resume} result]} {
#	if {[APSPEMTKDialog .warningDialog -text "Error while resuming power control scripts for sector $L1SectorSource \n$result"] == 1} {
#	    APSMpReturn error "RFGUN_Startup: $result"
#	}
#    }	

#    set KLYFwdPwrMin [expr {$KLYFwdPwr - 100000}]
#    APSMpStep "${prefix}Wait for Klystron forward power to reach $KLYFwdPwrMin" -desc "Wait for Klystron forward power to reach a minimum value"
    if {0} {
        if {[catch {APSMpLWaitForMinimumValue \
                      -deviceList $L1SectorSource \
                      -group LinacKlystron \
                      -operation ReadForwardPower \
                      -minValue $KLYFwdPwrMin \
                      -intervalLimit 900 \
                      -intervalTime 1} result]} {
            if {[APSPEMTKDialog .warningDialog -text "Error while waiting for Klystron forward power to reach $KLYFwdPwrMin \n$result"] == 1} {
                APSMpReturn error "RFGUN_Startup: $result"
            }
        }
    }
#    if {[catch {LMOCS_PowerControlLawSetup -mode low -deviceList L$L1SectorSource} result]} {
#	if {[APSPEMTKDialog .warningDialog -text "Error while setting L$L1SectorSource power controllaw into high mode \n$result"] == 1} {
#	    APSMpReturn error "RFGUN_Startup: $result"
#	}
#    }
    #done



    APSMpStep "${prefix}Set gate valves" -desc "Set gate valves"
    if {[catch {
	APSRFGUN_OpenCloseGateValve -deviceList L1GV1 -value Close -group LinacValves
	APSRFGUN_OpenCloseGateValve -deviceList L1GV2 -value Open -group LinacValves
	if {$gun == "RG2"} {
	    APSRFGUN_OpenCloseGateValve -deviceList 2 -value Open -group LinacRFGun
	    APSRFGUN_OpenCloseGateValve -deviceList 1 -value Close -group LinacRFGun
	} else {
	    APSRFGUN_OpenCloseGateValve -deviceList 2 -value Close -group LinacRFGun
	    APSRFGUN_OpenCloseGateValve -deviceList 1 -value Open -group LinacRFGun
	}
    } result]} {
        if {[APSPEMTKDialog .warningDialog -text "Error while setting up the gate valves \n$result"] == 1} {
            APSMpReturn error "RFGUN_Startup: $result"
        }
    }

    APSMpStep "${prefix}Turn the kicker charge pulse off" -desc "Turn the kicker charge pulse off"
    if {[catch {exec cavput -list=L1:RFG:KIK:chargeTimePD.GATE=0 -num -pend=20} results]} {
        if {[APSPEMTKDialog .warningDialog -text "Error while turning the kicker charge pulse off \n$results"] == 1} {
            APSMpReturn error "RFGUN_Startup: $results"
        }
    }

    APSMpStep "${prefix}Check if L$L1SectorSource LLRF is on" -desc "Check if L1 is on"
    if [catch {APSLINAC_CheckLLRFStatus -desiredState on -deviceList $L1SectorSource} result] {
	if {[APSPEMTKDialog .warningDialog -text "L$L1SectorSource LLRF is not on"] == 1} {
	    APSMpReturn error "RFGUN_Startup: $result"
	}
    }

    APSMpStep "${prefix}Check that the RFGun cathode heater power reaches $CathodeHeaterWattsMin but is below $CathodeHeaterWattsMax" -desc "Check that the RFGun cathode heater power reaches a minimum value"
    if {[catch {APSRFGUN_CathodeHeaterPower -deviceList $deviceList -wattsMin $CathodeHeaterWattsMin -watts $CathodeHeaterWatts -wattsMax $CathodeHeaterWattsMax} result]} {
        if {[APSPEMTKDialog .warningDialog -text "Error while checking that the RFGun cathode heater power reaches a minimum value \n$result"] == 1} {
            APSMpReturn error "RFGUN_Startup: $result"
        }
    }
    
    APSMpStep "${prefix}Reset Timing System" -desc "Reset Timing System"
    if {[catch {APSRFGUN_ResetTimingSystem} result]} {
        if {[APSPEMTKDialog .warningDialog -text "Error while reseting the timing system \n$result"] == 1} {
            APSMpReturn error "RFGUN_Startup: $result"
        }
    }

    APSMpStep "${prefix}Check if L$L1SectorSource LLRF is on" -desc "Check if L1 is on"
    if [catch {APSLINAC_CheckLLRFStatus -desiredState on -deviceList $L1SectorSource} result] {
	if {[APSPEMTKDialog .warningDialog -text "L$L1SectorSource LLRF is not on"] == 1} {
	    APSMpReturn error "RFGUN_Startup: $result"
	}
    }

    if {0} {
        APSMpStep "${prefix}Wait for power controllaw to increase the power in L$L1SectorSource" -desc "Wait for power controllaw to increase the power"
        while 1 {
            if {[catch {exec cavget -list=L${L1SectorSource}:KY:DC2ARF:ErrorM -num -pend=20} errorValue]} {
                APSMpReturn error "RFGUN_Startup: $errorValue"
            }
            if {[expr abs($errorValue)] < 1000e3} {
                break
            } else {
                APSMpStep "${prefix}Wait for L$L1SectorSource power control to increase power ([clock format [clock seconds] -format %T])" -desc "Wait for L$L1SectorSource power control to increase power"
                if {[catch {APSDevSend \
                              -group LinacKlystron \
                              -operation QueryLLRFOn \
                              -deviceList $L1SectorSource} result]} {
                    APSMpReturn error "RFGUN_Startup: $result"
                }
                if {!$result} {
                    if {[APSPEMTKDialog .warningDialog -text "RFGUN_Startup: L$L1SectorSource LLRF tripped. Please fix."] == 1} {
                        APSMpReturn error "RFGUN_Startup: L$L1SectorSource LLRF tripped."
                    }
                }
                if [catch {APSDevSend \
                             -group LinacKlystron \
                             -operation QueryPowerControlRunning \
                             -deviceList $L1SectorSource} results] {
                    APSMpReturn error "RFGUN_Startup: $results"
                }
                if {$results != 1} {
                    set answer [APSPEMTKDialog .warningDialog -text "RFGUN_Startup: sector $L1SectorSource power control is not running. Please fix." -strings {Continue Recheck "Abort Procedure"}]
                    if {$answer == 2} {
                        APSMpReturn error "RFGUN_Startup:  sector $L1SectorSource power control is not running."
                    }
                    if {$answer == 0} {
                        break
                    }
                }
            }
            APSWaitWithUpdate -waitSeconds 5
        }
    }

    set rfgForwardPowerSCR [expr $rfgForwardPowerSCR * .9]
    set step 0
    APSMpStep "${prefix}Wait for L1:RFG:DC2ARF to reach $rfgForwardPowerSCR" -desc "Wait for L1:RFG:DC2ARF to reach 90% full power"
    while 1 {
        if {[catch {exec cavget -list=L1:RFG:DC2ARF -num -pend=20} rfgForwardPower]} {
            APSMpReturn error "RFGUN_Startup: $rfgForwardPower"
        }
        if {$rfgForwardPower > $rfgForwardPowerSCR} {
            break
        }
        incr step
        if {$step > 84} {
            set answer [APSPEMTKDialog .warningDialog -text "L1:RFG:DC2ARF has not reached $rfgForwardPowerSCR" -strings {Continue "Wait Longer" "Abort Procedure"}]
            if {$answer == 2} {
                APSMpReturn error "RFGUN_Startup: L1:RFG:DC2ARF has not reached $rfgForwardPowerSCR"
            }
            if {$answer == 1} {
                set step 0
            }
            if {$answer == 0} {
                break
            }
        }
        APSWaitWithUpdate -waitSeconds 5
        APSMpStep "${prefix}Wait for L1:RFG:DC2ARF to reach $rfgForwardPowerSCR"
    }

    APSMpStep "${prefix}Resume fixedCurrent" -desc "Resume fixedCurrent"
    if {[catch {APSMpLSuspendResumeRunControl \
                  -value "Resume" \
                  -name "FixedCurrent" \
                  -group "LinacRFGun" \
                  -deviceList 0 \
                  -port 4579 \
                  -executable "fixedCurrent -start 1"} result]} {
        if {[APSPEMTKDialog .warningDialog -text "Error while resuming the hold current script \n$result"] == 1} {
            APSMpReturn error "RFGUN_Startup: $result"
        }
    }	        


    set CathodeHeaterWattsMin [expr {$CathodeHeaterWatts * .9}]
    APSMpStep "${prefix}Check that the RFGun cathode heater power reaches $CathodeHeaterWattsMin but is below $CathodeHeaterWattsMax" -desc "Check that the RFGun cathode heater reaches a minimum value"
    if {[catch {APSRFGUN_CathodeHeaterPower -deviceList $deviceList -wattsMin $CathodeHeaterWattsMin -watts $CathodeHeaterWatts -wattsMax $CathodeHeaterWattsMax} result]} {
        if {[APSPEMTKDialog .warningDialog -text "Error while checking that the RFGun cathode heater power reaches a minimum value \n$result"] == 1} {
            APSMpReturn error "RFGUN_Startup: $result"
        }
    }

    if {$resumeSmartMonitor} {
	if 0 {
	APSMpStep "${prefix}Resume Smart Monitor" -desc "Resume Smart Monitor"
	catch {exec cavput -list=L:RFG:smartMonitorOnC.PROC=1 -num -pend=20}
        APSWaitWithUpdate -waitSeconds 3
	catch {exec cavput -list=L:RFG:smartMonitorResetC=1 -num -pend=20}
        APSWaitWithUpdate -waitSeconds 3
        if {[catch {exec cavget -list=L:RFG:smartMonitorAlarmM -num -pend=20} result]} {
            if {$result != 0} {
                if {[APSPEMTKDialog .warningDialog -text "Problem resuming the smart monitor"] == 1} {
                    APSMpReturn error "RFGUN_Startup: $result"
                }
            }
        }
	}
    }

    APSMpStep "${prefix}Check if L$L1SectorSource LLRF is on" -desc "Check if L1 is on"
    if [catch {APSLINAC_CheckLLRFStatus -desiredState on -deviceList $L1SectorSource} result] {
	if {[APSPEMTKDialog .warningDialog -text "L$L1SectorSource LLRF is not on. Check smart monitor reset, check vacuum or water faults, check 400W amp trip"] == 1} {
	    APSMpReturn error "RFGUN_Startup: $result"
	}
    }



    APSMpStep "${prefix}Set Linac Phase Control Points from SCR File" -desc "${prefix}Set Linac Phase Control Points from SCR File"
    if {[info exists rg1phasesetpoint]} {
        if {[catch {APScavput -list=L:BPD:L1P1:RFGUN1:SetpointAO=$rg1phasesetpoint} result]} {
            if {[APSPEMTKDialog .warningDialog -text "Problem occured while setting linac phase control points: $result"] == 1} {
                APSMpReturn error "RFGUN_Startup: $result"
            }
        }
    }
    if {[info exists rg2phasesetpoint]} {
        if {[catch {APScavput -list=L:BPD:L1P1:RFGUN2:SetpointAO=$rg2phasesetpoint} result]} {
            if {[APSPEMTKDialog .warningDialog -text "Problem occured while setting linac phase control points: $result"] == 1} {
                APSMpReturn error "RFGUN_Startup: $result"
            }
        }
    }
    if {[info exists l2phasesetpoint]} {
        if {[catch {APScavput -list=L:BPD:L1P1:L2AS1:SetpointAO=$l2phasesetpoint} result]} {
            if {[APSPEMTKDialog .warningDialog -text "Problem occured while setting linac phase control points: $result"] == 1} {
                APSMpReturn error "RFGUN_Startup: $result"
            }
        }
    }
    if {[info exists l4phasesetpoint]} {
        if {[catch {APScavput -list=L:BPD:L3P3:L4AS1:SetpointAO=$l4phasesetpoint} result]} {
            if {[APSPEMTKDialog .warningDialog -text "Problem occured while setting linac phase control points: $result"] == 1} {
                APSMpReturn error "RFGUN_Startup: $result"
            }
        }
    }
    if {[info exists l5phasesetpoint]} {
        if {[catch {APScavput -list=L:BPD:L4P1:L5AS1:SetpointAO=$l5phasesetpoint} result]} {
            if {[APSPEMTKDialog .warningDialog -text "Problem occured while setting linac phase control points: $result"] == 1} {
                APSMpReturn error "RFGUN_Startup: $result"
            }
        }
    }

    if {!$changeBypassState} {
        APSMpStep "${prefix}Resume linac RF phase controllaw" -desc "Resume linac RF phase controllaw"
        if {$gun == "RG1"} {
            if {[catch {exec cavput -list=L:BPD:RFGUN1:L1L2L4L5RC.SUSP=0} result]} {
                if {[APSPEMTKDialog .warningDialog -text "Problem resuming the linac RF phase controllaw\nRFGUN_Startup: $result"] == 1} {
                    APSMpReturn error "Problem resuming the linac RF phase controllaw: RFGUN_Startup: $result"
                }
            }
            after 3000
            if {[catch [APScavget -noquote -list=L:BPD:RFGUN1:L1L2L4L5RC.PID] result]} {
            } else {
                if {[llength $result] != 1} {
                    exec cavput -list=L:BPD:RFGUN1:L1L2L4L5RC.CLR=1
                    set dir [pwd]
                    cd /home/helios/oagData/controllaw/linacBeamRfPhase/L1RFGun1_L2_L4_L5
                    if {[catch {exec ./linacBeamRfPhase-L1RFGun1-L2-L4-L5Options -rcPV L:BPD:RFGUN1:L1L2L4L5RC} options]} {
                        APSMpReturn error "Unable to start linac RF phase controllaw: $options"
                    }
                    if {[catch {eval exec sddscontrollaw $options >& /dev/null &} result]} {
                        if {[APSPEMTKDialog .warningDialog -text "Unable to start linac RF phase controllaw: $result"] == 1} {
                            APSMpReturn error "Unable to start linac RF phase controllaw: $result"
                        }
                    }
                    cd $dir
                }
            }
        } else {
            if {[catch {exec cavput -list=L:BPD:RFGUN2:L1L2L4L5RC.SUSP=0} result]} {
                if {[APSPEMTKDialog .warningDialog -text "Problem resuming the linac RF phase controllaw\nRFGUN_Startup: $result"] == 1} {
                    APSMpReturn error "Problem resuming the linac RF phase controllaw: RFGUN_Startup: $result"
                }
            }
            after 3000
            if {[catch [APScavget -noquote -list=L:BPD:RFGUN2:L1L2L4L5RC.PID] result]} {
            } else {
                if {[llength $result] != 1} {
                    exec cavput -list=L:BPD:RFGUN2:L1L2L4L5RC.CLR=1
                    set dir [pwd]
                    cd /home/helios/oagData/controllaw/linacBeamRfPhase/L1RFGun2_L2_L4_L5
                    if {[catch {exec ./linacBeamRfPhase-L1RFGun2-L2-L4-L5Options -rcPV L:BPD:RFGUN2:L1L2L4L5RC} options]} {
                        APSMpReturn error "Unable to start linac RF phase controllaw: $options"
                    }
                    if {[catch {eval exec sddscontrollaw $options >& /dev/null &} result]} {
                        if {[APSPEMTKDialog .warningDialog -text "Unable to start linac RF phase controllaw: $result"] == 1} {
                            APSMpReturn error "Unable to start linac RF phase controllaw: $result"
                        }
                    }
                    cd $dir
                }
            }
        }
    }
    
    #    catch {exec cavput -list=L:RFG:smartMonitorEnableC=1 -num -pend=20}

    APSMpStep "${prefix}Set Kicker Setpoint" -desc "Set Kicker setpoint"
    if {[catch {APSDevSend \
                  -group LinacRFGun \
                  -operation SetKickerVoltage \
                  -deviceList $deviceList \
                  -value $KickerSetpoint} result]} {
        if {[APSPEMTKDialog .warningDialog -text "Error while setting the kicker setpoint \n$result"] == 1} {
            APSMpReturn error "RFGUN_Startup: $result"
        }
    }

    APSMpStep "${prefix}Restore values from SCR file" -desc "Restore SCR file"
    if {$bypassMode} {
	set options "-match=column,ControlName=*MO:PFN*,!"
    } else {
	#set options "-match=column,ControlName=L1:MO:PFN*,ControlName=*MO:PFN*,!,|"
	set options "-match=column,ControlName=*MO:PFN*,!"
    }
    set options2 "-match=column,Beamline=L1,Beamline=L2,|,Beamline=L3,|,Beamline=LINAC,|,Beamline=LI,|,Beamline=rfGun,|,Beamline=rfGun1,|,Beamline=rfGun2,|"
    if {$L1SectorSource == 3} {
        append options " -match=column,ControlName=L1:MO:PFN*,!"
        set options2 "-match=column,Beamline=L2,Beamline=L3,|,Beamline=LINAC,|,Beamline=LI,|,Beamline=rfGun,|,Beamline=rfGun1,|,Beamline=rfGun2,|"
    }
    if {$L2SectorSource == 3} {
        append options " -match=column,ControlName=L2:MO:PFN*,!"
        set options2 "-match=column,Beamline=L1,Beamline=L3,|,Beamline=LINAC,|,Beamline=LI,|,Beamline=rfGun,|,Beamline=rfGun1,|,Beamline=rfGun2,|"
    }
    if {$L4} {
	append options2 ",Beamline=L4,|"
    }
    if {$L5} {
	append options2 ",Beamline=L5,|"
    }

    if {[string length $SCRFile]} {
	APSMpStep RestoringSCRValues 
	set tmpFile /tmp/[APSTmpString]

	APSMpStep RestoringNonTimingSCRValues 
	if {[catch {eval exec sddsprocess $SCRFile $tmpFile \
                      -match=column,IsReadOnly=+n \
                      -match=column,IsProtected=+n \
                      -match=column,Category=Diag,Category=Gun,|,Category=InTiming,|,Category=PulsedPS,|,Category=RF,|,Category=RFConstants,|,Category=Scrapers,|,Category=SteeringSetpoint,|,Category=Timing,| \
                      $options \
                      $options2 \
                      -match=col,ControlType=,! -match=col,ControlName=,! \
                      -match=column,ControlName=L:L?Stabilizer:SDDS*,!
	    exec sddscasr -restore $tmpFile} result]} {
            APSPEMTKDialog .warningDialog -strings Continue -default 0 -text "Problem restoring SCR file: RFGUN_Startup: $result"
            catch {APSSendEMail -address soliday \
                     -subject "Problem restoring SCR file."  \
                     -message "Problem restore SCR file\nRFGUN_Startup\n$SCRFile\n$result"} 
	}
	file delete $tmpFile

    }
    
    APSMpStep "${prefix}Turn the kicker on" -desc "Turn the kicker on"
    if {$gun == "RG1"} {
	if {[catch {APSRFGUN_TurnKickerOnOff -deviceList 1 -value 0 -state "on"} result]} {
            if {[APSPEMTKDialog .warningDialog -text "Unable to turn off rfgun 1 kicker\n$result"] == 1} {
                APSMpReturn error "RFGUN_SwitchWith: $result"
            }
	}
    } elseif {$gun == "RG2"} {
	if {[catch {APSRFGUN_TurnKickerOnOff -deviceList 2 -value 0 -state "on"} result]} {
            if {[APSPEMTKDialog .warningDialog -text "Unable to turn off rfgun 2 kicker\n$result"] == 1} {
                APSMpReturn error "RFGUN_SwitchWith: $result"
            }
	}
    }


    APSMpReturn ok
}

proc APSLINACFindMostRecentSCRFile {} {
    set machine LPL
    set oldDir [pwd]
    set snapDir /home/helios/oagData/SCR/snapshots/$machine
    cd $snapDir
    set time [clock seconds]
    if {[file exists collapsed]} {
        set collapsedDirectoryFound 1
        cd collapsed
    } else {
        set collapsedDirectoryFound 0
    }
    set fileList [APSFindFilesBetweenDates -directory . -rootname c$machine -tailsOnly 1 \
                    -startDateList [clock format [expr {$time - 5184000}] -format "%Y %j %m %d"] \
                    -endDateList [clock format $time -format "%Y %j %m %d"] \
                    -extensionList {""}]
    if {[llength $fileList]==0} {
        cd $oldDir
        return -code ok ""
    }
    set fileList [lindex [lsort $fileList] end]
    if {[catch {exec sdds2stream $fileList -column=SnapshotFilename} result]} {
        cd $oldDir
        return -code ok ""
    }
    set result [file join $snapDir $result]
    if {![file exists $result]} {
	set result ${result}.gz
	if {![file exists $result]} {
	    cd $oldDir
	    return -code ok ""
	}	
    }
    cd $oldDir
    return -code ok $result
}

proc APSRFGUN_ResetTimingSystem {} {
    if {[catch {exec cavput -list=L1:KY:DC2ARF.SPD=On,L1:KY:DC2ARF.AQWF=On} result]} {
	return -code error "APSRFGUN_ResetTimingSystem: $result"
    }
    APSWaitWithUpdate -waitSeconds 2
}

#Suspend or Resume Run Controls
proc APSMpLSuspendResumeRunControl {args} {
    set value ""
    set name ""
    set group ""
    set deviceList ""
    set waitSeconds 4
    set port ""
    set executable ""
    APSStrictParseArguments {value name group deviceList waitSeconds port executable}
    if {($value != "Resume") && ($value != "Suspend")} {
	return -code error "APSMpLSuspendResumeRunControl: value must be Resume or Suspend"
    }
    if {(![llength $name]) || (![llength $group])} {
	return -code error "APSMpLSuspendResumeRunControl: invalid arguments passed"
    }
    if {[catch {APSDevSend \
                  -group $group \
                  -operation ${value}${name} \
                  -deviceList $deviceList} result]} {
	return -code error "APSMpLSuspendResumeRunControl: $result"
    }

    if {$value == "Resume"} {
	APSWaitWithUpdate -waitSeconds $waitSeconds

	if {[catch {APSDevSend \
                      -group $group \
                      -operation Query${name}Running \
                      -deviceList $deviceList} result]} {
	    return -code error "APSMpLSuspendResumeRunControl: $result"
	}
	if {$result != 1} {
	    if {[llength $port] == 1} {		
		if {![catch {socket localhost $port} sid]} {
		    APSWaitWithUpdate -waitSeconds [expr {3 + $waitSeconds}]
		    if {[catch {APSDevSend \
                                  -group $group \
                                  -operation Query${name}Running \
                                  -deviceList $deviceList} result]} {
			return -code error "APSMpLSuspendResumeRunControl: $result"
		    }
		    if {$result == 1} {
			return -code ok
		    }
		}
	    }
	    if {[llength $executable]} {
		if {![catch {eval exec $executable &} result]} {		    
		    APSWaitWithUpdate -waitSeconds [expr {3 + $waitSeconds}]
		    if {[catch {APSDevSend \
                                  -group $group \
                                  -operation Query${name}Running \
                                  -deviceList $deviceList} result]} {
			return -code error "APSMpLSuspendResumeRunControl: $result"
		    }
		    if {$result == 1} {
			return -code ok
		    }
		}
	    }
	    bell
            if {[APSPEMTKDialog .warningDialog -text "Unable to automatically resume runcontrol ${name}."] == 1} {
		return -code error "APSMpLSuspendResumeRunControl: Operation aborted by user"
	    }
	}
    }
}

#Verify that the macropulse repetition rate is RFG 10 Hz or less 
proc APSRFGUN_Macropulse {args} {
    set i 1
    while {$i} {
	set i 0
	if {[catch {APSDevSend \
                      -group LinacTiming \
                      -operation ReadBeamTriggerRate \
                      -deviceList 0} result]} {
	    return -code error "APSRFGUN_Macropulse: $result"
	}
	if {($result != {{RFG 10Hz}}) && ($result != "--10Hz--") && ($result != "--6Hz--") && ($result != "--4Hz--") && ($result != "--2Hz--")} {
	    set i 1
            if {[APSPEMTKDialog .warningDialog -text "Macropulse repetition rate is not <= 10Hz. Lower rate and click Continue when finished"] == 1} {
		return -code error "APSRFGUN_Macropulse: Macropulse repetition rate is not <= 10Hz"
	    }
	}
    }
    update
}

#Verify that the Gun's cathode heater power is 12W +- .2W
#Do not proceed until this condition is met.
proc APSRFGUN_CathodeHeaterPower {args} {
    set deviceList ""
    set wattsMin 11.8
    set watts 12.0
    set wattsMax 12.5
    APSStrictParseArguments {deviceList wattsMin watts wattsMax}

    if {[catch {APSDevSend \
                  -group LinacRFGun \
                  -operation ReadCathodeHeaterPower \
                  -deviceList $deviceList} result]} {
	return -code error "APSRFGUN_CathodeHeaterPower: $result"
    }
    #    set previous $result

    if {($result < $wattsMin) || ($result > $wattsMax)} {
	set i 1
	while {$i} {
	    set i 0
	    set timeLimit 380
	    set time 0
	    while {$time < $timeLimit} {
		after 1000
		update
		if {[catch {APSDevSend \
                              -group LinacRFGun \
                              -operation ReadCathodeHeaterPower \
                              -deviceList $deviceList} result]} {
		    return -code error "APSRFGUN_CathodeHeaterPower: $result"
		}
		if {($result > $wattsMin) && ($result < $wattsMax)} {
		    break
		}
		#		set change [expr $result - $previous]
		#		if {$change == 0} { set change .001 }
		#		set estimatedTime [expr ($wattsMin - $result) / $change]
		#		set previous $result
		#		APSSetVarAndUpdate status "$estimatedTime seconds left"
		incr time
	    }
	    if {$time >= $timeLimit} {
		set i 1
                set answer [APSPEMTKDialog .warningDialog -strings {Continue Ignore "Abort Procedure"} -text "Timed out waiting to reach $wattsMin Watts in the cathode heater"]
                if {$answer == 2} {
		    return -code error "APSRFGUN_CathodeHeaterPower: Timed out waiting to reach $wattsMin Watts"
		}
                if {$answer == 1} {
                    break
                }
	    }
	}
    }
    update
}

proc APSMpLWaitForMinimumValue {args} {
    set deviceList ""
    set group ""
    set operation ""
    set minValue ""
    set intervalLimit 60
    set intervalTime 1
    APSStrictParseArguments {deviceList group operation minValue intervalLimit intervalTime}
    if {[llength deviceList] != 1} {return -code error "APSMpLWaitForMinimumValue: Only one device allowed"}
    if {[catch {APSDevSend \
                  -group $group \
                  -operation $operation \
                  -deviceList $deviceList} result]} {
	return -code error "APSMpLWaitForMinimumValue: $result"
    }

    if {$result < $minValue} {
	set i 1
	while {$i} {
	    set i 0
	    set time 0
	    while {$time < $intervalLimit} {
		APSWaitWithUpdate -waitSeconds $intervalTime
		if {[catch {APSDevSend \
                              -group $group \
                              -operation $operation \
                              -deviceList $deviceList} result]} {
		    return -code error "APSMpLWaitForMinimumValue: $result"
		}
		if {$result > $minValue} {
		    break
		}
		incr time
	    }
	    if {$time >= $intervalLimit} {
		set i 1
		set answer [APSPEMTKDialog .warningDialog -strings {Continue Ignore "Abort Procedure"} -text "Timed out waiting for L${deviceList} $operation to reach $minValue"]
                if {$answer == 2} {
		    return -code error "APSMpLWaitForMinimumValue: Timed out waiting for $deviceList $operation to reach $minValue"
		}
                if {$answer == 1} {
                    break
                }
	    }
	}
    }
    update
}

proc APSLINAC_LLRFOnOff {args} {
    set button ""
    set deviceList ""
    APSStrictParseArguments {button deviceList}
    if {($button != "On") && ($button != "Off")} {
	return -code error "APSLINAC_LLRFOnOff: Invalid button"
    }
    if {[catch {APSDevSend \
                  -group LinacKlystron \
                  -operation QueryLLRFOn \
                  -deviceList $deviceList} results]} {
	return -code error "APSLINAC_LLRFOnOff: $results"
    }
    foreach result $results sector $deviceList {
	if {($button == "On") && (!$result)} {
            

	    if {[catch {APSDevSend \
                          -group LinacKlystron \
                          -operation TurnOnLLRF \
                          -deviceList $sector} result]} {
		return -code error "APSLINAC_LLRFOnOff: $result"
	    }
	}
	
	if {($button == "Off") && ($result)} {
	    if {[catch {APSDevSend \
                          -group LinacKlystron \
                          -operation TurnOffLLRF \
                          -deviceList $sector} result]} {
		return -code error "APSLINAC_LLRFOnOff: $result"
	    }
	}
    }
    APSWaitWithUpdate -waitSeconds 4
    if {[catch {APSDevSend \
                  -group LinacKlystron \
                  -operation QueryLLRFOn \
                  -deviceList $deviceList} results]} {
	return -code error "APSLINAC_LLRFOnOff: $results"
    }
    foreach result $results sector $deviceList {
	if {($button == "On") && (!$result)} {
            if {[APSPEMTKDialog .warningDialog -text "LLRF in sector $sector is not on."] == 1} {
		return -code error "APSLINAC_LLRFOnOff: RF is not on"
	    }
	}
	if {($button == "Off") && ($result)} {
            if {[APSPEMTKDialog .warningDialog -text "LLRF in sector $sector is not off."] == 1} {
		return -code error "APSLINAC_LLRFOnOff: RF is not off"
	    }
	}
    }
    update
}

proc APSLINAC_CheckModulatorCurrents {args} {
    set deviceList ""
    set minValueList ""
    APSStrictParseArguments {deviceList minValueList}
    if {[catch {APSDevSend \
                  -group LinacKlystron \
                  -operation SetPFNVoltage \
                  -deviceList $deviceList \
                  -forceRead 1} results]} {
	return -code error "APSLINAC_CheckModulatorCurrents: $results"
    }
    foreach result $results sector $deviceList minValue $minValueList {
	if {$result < $minValue} {
	    bell
            if {[APSPEMTKDialog .warningDialog -text "PFN voltage set point in modulator $sector is below $minValue KV."] == 1} {
		return -code error "APSLINAC_CheckModulatorCurrents: PFN voltage set point in modulator $sector is below $minValue KV"
	    }
	}
    }
}

proc APSLINAC_SetKlystronForwardPower {args} {
    set deviceList ""
    set KLYFwdPwr 13000000
    APSStrictParseArguments {deviceList KLYFwdPwr}
    
    #This is only called for devices 1 and 3

    set oldDeviceList ""
    set newDeviceList ""
    foreach device $deviceList {
        if {$device == 2} {
            lappend newDeviceList $device
        } else {
            lappend oldDeviceList $device
        }
    }
    foreach device $oldDeviceList {
        if {[catch {APScavget -list=L${device}:MO:LowPFNvSP} PFNLowTripValue]} {
            set PFNLowTripValue 22
        }
        if {[catch {APSLINAC_CheckModulatorCurrents -deviceList $device -minValueList $PFNLowTripValue} result]} {
            return -code error "APSLINAC_SetKlystronForwardPower: $result"
        }
    }
    if {[llength $oldDeviceList]} {
        if {[catch {APSDevSend \
                      -group LinacKlystron \
                      -operation QueryReadyToPulseOn \
                      -deviceList $deviceList} results]} {
            return -code error "APSLINAC_SetKlystronForwardPower: $result"
        }
        foreach result $results device $deviceList {
            if {$result != 1} {
                if {[APSPEMTKDialog .warningDialog -text "Not ready to pulse in sector L$device."] == 1} {
                    return -code error "APSLINAC_SetKlystronForwardPower: Not ready to pulse in sector L$device."
                }
            }
        }
    }
    if {[llength $newDeviceList]} {
        foreach device $newDeviceList {
            if {[catch {APSPEMCheckPVs -pvList L-K${device}:Mod:State-RB \
                          -desiredList 13 \
                          -greaterThanOrEqualTo 1 \
                          -allowContinue 0 \
                          -allowRecheck 0 \
                          -message "K${device} K400 State found to be less than 13, unable to continue."} k400state]} {
                return -code error $k400state
            }
        }
    }
    if {[catch {APSDevSend \
                  -group LinacKlystron \
                  -operation SetForwardPowerSetpoint \
                  -deviceList $deviceList \
                  -value $KLYFwdPwr} result]} {
	return -code error "APSLINAC_SetKlystronForwardPower: $result"
    }
    update
}


proc RestartLinacRFPhaseControllaw {args} {
    set SCRFile ""
    APSStrictParseArguments {SCRFile}

    if {[catch {APSLinkToLINACPVs} result]} {
	APSMpReturn error "RestartLinacRFPhaseControllaw: $result"
    }
    if {[catch {APSRFGUN_DetectSelectedGun} gun]} {
	APSMpReturn error "Unable to detect selected gun: $gun"
    }
    if {[catch {APSDevSend \
                  -group LinacRFGun \
                  -operation ReadRFCurrentMode \
                  -deviceList 0} linacMode]} {
	APSMpReturn error "Unable to detect linac mode: $linacMode"
    }
    set linacMode [join $linacMode]

    if {[catch {exec sddsselect $SCRFile /home/helios/oagData/pem/linac/filterFile -match=ControlName -pipe=out | sdds2stream -column=ControlName,ValueString -pipe=in} results]} {
	APSMpReturn error "RestartLinacRFPhaseControllaw: Invalid SCR file"
    }
    foreach {controlName valueString} $results {
	if {"L:BPD:L1P1:RFGUN1:SetpointAO" == $controlName} {
	    set rg1phasesetpoint $valueString
	} elseif {"L:BPD:L1P1:RFGUN2:SetpointAO" == $controlName} {
	    set rg2phasesetpoint $valueString
	} elseif {"L:BPD:L1P1:L2AS1:SetpointAO" == $controlName} {
	    set l2phasesetpoint $valueString
	} elseif {"L:BPD:L3P3:L4AS1:SetpointAO" == $controlName} {
	    set l4phasesetpoint $valueString
	} elseif {"L:BPD:L3P3:L4AS1:SetpointAO" == $controlName} {
	    set l5phasesetpoint $valueString
	}
    }


    APSMpStep "Abort linac RF phase controllaw"
    if {[catch {exec cavput -list=L:BPD:RFGUN1:L1L2L4L5RC.ABRT=1,L:BPD:RFGUN2:L1L2L4L5RC.ABRT=1,L:BPD:RFGUN1RC.ABRT=1,L:BPD:RFGUN2RC.ABRT=1,L:BPD:RFGUN2K1DnRC.ABRT=1,L:BPD:RFGUN2K2DnRC.ABRT=1,L:BPD:L3P3:L4AS1RC.ABRT=1,L:BPD:L4P1:L5AS1RC.ABRT=1} result]} {
        if {[APSPEMTKDialog .warningDialog -text "Problem aborting the linac RF phase controllaw: $result"] == 1} {
            APSMpReturn error "Problem aborting the linac RF phase controllaw: $result"
        }
    }
    APSMpStep "Relink RF phase controllaw input files"
    if {$linacMode == "K1 Down"} {
        if {[catch {
            exec ln -fs /home/helios/oagData/controllaw/linacBeamRfPhase/L1RFGun1_L2_L4_L5/linacBeamRfPhase-L1RFGun1-L2-L4-L5_K1Dn.def \
              /home/helios/oagData/controllaw/linacBeamRfPhase/L1RFGun1_L2_L4_L5/linacBeamRfPhase-L1RFGun1-L2-L4-L5.def
            exec ln -fs /home/helios/oagData/controllaw/linacBeamRfPhase/L1RFGun2_L2_L4_L5/linacBeamRfPhase-L1RFGun2-L2-L4-L5_K1Dn.def \
              /home/helios/oagData/controllaw/linacBeamRfPhase/L1RFGun2_L2_L4_L5/linacBeamRfPhase-L1RFGun2-L2-L4-L5.def
            exec ln -fs /home/helios/oagData/controllaw/linacBeamRfPhase/L1RFGun1_L2_L4_L5/linacBeamRfPhase-L1RFGun1-L2-L4-L5_K1Dn.tests \
              /home/helios/oagData/controllaw/linacBeamRfPhase/L1RFGun1_L2_L4_L5/linacBeamRfPhase-L1RFGun1-L2-L4-L5.tests
            exec ln -fs /home/helios/oagData/controllaw/linacBeamRfPhase/L1RFGun2_L2_L4_L5/linacBeamRfPhase-L1RFGun2-L2-L4-L5_K1Dn.tests \
              /home/helios/oagData/controllaw/linacBeamRfPhase/L1RFGun2_L2_L4_L5/linacBeamRfPhase-L1RFGun2-L2-L4-L5.tests
        } result]} {
            if {[APSPEMTKDialog .warningDialog -text "Unable to relink RF phase controllaw input file for K1 down mode: $result"] == 1} {
                APSMpReturn error "Unable to relink RF phase controllaw input file for K1 down mode: $result"
            }
        }
    } elseif {$linacMode == "K2 Down"} {
        if {[catch {
            exec ln -fs /home/helios/oagData/controllaw/linacBeamRfPhase/L1RFGun1_L2_L4_L5/linacBeamRfPhase-L1RFGun1-L2-L4-L5_K2Dn.def \
              /home/helios/oagData/controllaw/linacBeamRfPhase/L1RFGun1_L2_L4_L5/linacBeamRfPhase-L1RFGun1-L2-L4-L5.def
            exec ln -fs /home/helios/oagData/controllaw/linacBeamRfPhase/L1RFGun2_L2_L4_L5/linacBeamRfPhase-L1RFGun2-L2-L4-L5_K2Dn.def \
              /home/helios/oagData/controllaw/linacBeamRfPhase/L1RFGun2_L2_L4_L5/linacBeamRfPhase-L1RFGun2-L2-L4-L5.def
            exec ln -fs /home/helios/oagData/controllaw/linacBeamRfPhase/L1RFGun1_L2_L4_L5/linacBeamRfPhase-L1RFGun1-L2-L4-L5_K2Dn.tests \
              /home/helios/oagData/controllaw/linacBeamRfPhase/L1RFGun1_L2_L4_L5/linacBeamRfPhase-L1RFGun1-L2-L4-L5.tests
            exec ln -fs /home/helios/oagData/controllaw/linacBeamRfPhase/L1RFGun2_L2_L4_L5/linacBeamRfPhase-L1RFGun2-L2-L4-L5_K2Dn.tests \
              /home/helios/oagData/controllaw/linacBeamRfPhase/L1RFGun2_L2_L4_L5/linacBeamRfPhase-L1RFGun2-L2-L4-L5.tests
        } result]} {
            if {[APSPEMTKDialog .warningDialog -text "Unable to relink RF phase controllaw input file for K2 down mode: $result"] == 1} {
                APSMpReturn error "Unable to relink RF phase controllaw input file for K2 down mode: $result"
            }
        }
    } else {
        if {[catch {
            exec ln -fs /home/helios/oagData/controllaw/linacBeamRfPhase/L1RFGun1_L2_L4_L5/linacBeamRfPhase-L1RFGun1-L2-L4-L5_K3Dn.def \
              /home/helios/oagData/controllaw/linacBeamRfPhase/L1RFGun1_L2_L4_L5/linacBeamRfPhase-L1RFGun1-L2-L4-L5.def
            exec ln -fs /home/helios/oagData/controllaw/linacBeamRfPhase/L1RFGun2_L2_L4_L5/linacBeamRfPhase-L1RFGun2-L2-L4-L5_K3Dn.def \
              /home/helios/oagData/controllaw/linacBeamRfPhase/L1RFGun2_L2_L4_L5/linacBeamRfPhase-L1RFGun2-L2-L4-L5.def
            exec ln -fs /home/helios/oagData/controllaw/linacBeamRfPhase/L1RFGun1_L2_L4_L5/linacBeamRfPhase-L1RFGun1-L2-L4-L5_K3Dn.tests \
              /home/helios/oagData/controllaw/linacBeamRfPhase/L1RFGun1_L2_L4_L5/linacBeamRfPhase-L1RFGun1-L2-L4-L5.tests
            exec ln -fs /home/helios/oagData/controllaw/linacBeamRfPhase/L1RFGun2_L2_L4_L5/linacBeamRfPhase-L1RFGun2-L2-L4-L5_K3Dn.tests \
              /home/helios/oagData/controllaw/linacBeamRfPhase/L1RFGun2_L2_L4_L5/linacBeamRfPhase-L1RFGun2-L2-L4-L5.tests
        } result]} {
            if {[APSPEMTKDialog .warningDialog -text "Unable to relink RF phase controllaw input file for K3 down mode: $result"] == 1} {
                APSMpReturn error "Unable to relink RF phase controllaw input file for K3 down mode: $result"
            }
        }
    }

    APSMpStep "Set Linac Phase Control Points from SCR File" -desc "Set Linac Phase Control Points from SCR File"
    if {[info exists rg1phasesetpoint]} {
        if {[catch {APScavput -list=L:BPD:L1P1:RFGUN1:SetpointAO=$rg1phasesetpoint} result]} {
            if {[APSPEMTKDialog .warningDialog -text "Problem occured while setting linac phase control points: $result"] == 1} {
                APSMpReturn error "RestartLinacRFPhaseControllaw: $result"
            }
        }
    }
    if {[info exists rg2phasesetpoint]} {
        if {[catch {APScavput -list=L:BPD:L1P1:RFGUN2:SetpointAO=$rg2phasesetpoint} result]} {
            if {[APSPEMTKDialog .warningDialog -text "Problem occured while setting linac phase control points: $result"] == 1} {
                APSMpReturn error "RestartLinacRFPhaseControllaw: $result"
            }
        }
    }
    if {[info exists l2phasesetpoint]} {
        if {[catch {APScavput -list=L:BPD:L1P1:L2AS1:SetpointAO=$l2phasesetpoint} result]} {
            if {[APSPEMTKDialog .warningDialog -text "Problem occured while setting linac phase control points: $result"] == 1} {
                APSMpReturn error "RestartLinacRFPhaseControllaw: $result"
            }
        }
    }
    if {[info exists l4phasesetpoint]} {
        if {[catch {APScavput -list=L:BPD:L3P3:L4AS1:SetpointAO=$l4phasesetpoint} result]} {
            if {[APSPEMTKDialog .warningDialog -text "Problem occured while setting linac phase control points: $result"] == 1} {
                APSMpReturn error "RestartLinacRFPhaseControllaw: $result"
            }
        }
    }
    if {[info exists l5phasesetpoint]} {
        if {[catch {APScavput -list=L:BPD:L4P1:L5AS1:SetpointAO=$l5phasesetpoint} result]} {
            if {[APSPEMTKDialog .warningDialog -text "Problem occured while setting linac phase control points: $result"] == 1} {
                APSMpReturn error "RestartLinacRFPhaseControllaw: $result"
            }
        }
    }

    if {$gun == "RG1"} {
        if {[catch {exec cavput -list=L:BPD:RFGUN1:L1L2L4L5RC.SUSP=0} result]} {
            if {[APSPEMTKDialog .warningDialog -text "Problem connecting to RF phase controllaw run control: $result"] == 1} {
                APSMpReturn error "Problem connecting to RF phase controllaw run control: $result"
            }
        }
        after 3000
        exec cavput -list=L:BPD:RFGUN1:L1L2L4L5RC.CLR=1
        set dir [pwd]
        cd /home/helios/oagData/controllaw/linacBeamRfPhase/L1RFGun1_L2_L4_L5
        if {[catch {exec ./linacBeamRfPhase-L1RFGun1-L2-L4-L5Options -rcPV L:BPD:RFGUN1:L1L2L4L5RC} options]} {
            APSMpReturn error "Unable to start linac RF phase controllaw: $options"
        }
        if {[catch {eval exec sddscontrollaw $options >& /dev/null &} result]} {
            if {[APSPEMTKDialog .warningDialog -text "Unable to start linac RF phase controllaw: $result"] == 1} {
                APSMpReturn error "Unable to start linac RF phase controllaw: $result"
            }
        }
        cd $dir
    } else {
        if {[catch {exec cavput -list=L:BPD:RFGUN2:L1L2L4L5RC.SUSP=0} result]} {
            if {[APSPEMTKDialog .warningDialog -text "Problem connecting to RF phase controllaw run control: $result"] == 1} {
                APSMpReturn error "Problem connecting to RF phase controllaw run control: $result"
            }
        }
        after 3000
        exec cavput -list=L:BPD:RFGUN2:L1L2L4L5RC.CLR=1
        set dir [pwd]
        cd /home/helios/oagData/controllaw/linacBeamRfPhase/L1RFGun2_L2_L4_L5
        if {[catch {exec ./linacBeamRfPhase-L1RFGun2-L2-L4-L5Options -rcPV L:BPD:RFGUN2:L1L2L4L5RC} options]} {
            APSMpReturn error "Unable to start linac RF phase controllaw: $options"
        }
        if {[catch {eval exec sddscontrollaw $options >& /dev/null &} result]} {
            if {[APSPEMTKDialog .warningDialog -text "Unable to start linac RF phase controllaw: $result"] == 1} {
                APSMpReturn error "Unable to start linac RF phase controllaw: $result"
            }
        }
        cd $dir
    }
    APSMpReturn ok
}
