# $Log: not supported by cvs2svn $
# Revision ??? 2013/09/20 H. Shang 
# Per Karen's request, added QS4, S1-6A:H4, S40A:H4; removed S39A:V4, S13A:[HV]4, S15A:[HV]4, S17A:[HV]4.
#
# Revision 1.116  2012/04/17 15:24:14  soliday
# The canted dipoles can not be selected without also selecting the main dipole.
#
# Revision 1.115  2012/03/13 17:11:30  soliday
# Updated after testing changes for the canted undulator correctors. They
# are now treated the same as the other correctors.
#
# Revision 1.114  2011/03/16 15:42:53  shang
# fixed a typo
#
# Revision 1.113  2011/03/16 14:38:19  shang
# added second "file readlink" because SR UNOP has 2 layers of link.
#
# Revision 1.112  2010/09/28 16:59:49  soliday
# No longer stops or starts BControl when conditioning per CY's request.
#
# Revision 1.111  2010/02/02 17:26:25  soliday
# Improved the code when doing the 100Hz reset procedure so that it can retry
# to the reset if cavget does not connect to all the PVs the first time.
#
# Revision 1.110  2009/11/06 16:05:50  soliday
# Added a check before dumping beam to verify there is at least .5mA in the SR.
# If there isn't, then it does not dump the beam.
#
# Revision 1.109  2009/06/03 15:04:30  soliday
# Added a check for open shutters prior to dumping the beam.
#
# Revision 1.108  2009/02/24 19:01:03  soliday
# Updated sot that forced conditioning is no longer the default.
#
# Revision 1.107  2007/09/28 18:54:10  soliday
# Fixed syntax error.
#
# Revision 1.106  2007/09/28 16:03:24  soliday
# Removed code for the EMW power supply in sector 11 because it no longer exists.
#
# Revision 1.105  2007/06/06 00:35:51  emery
# Commented out reseting the rf agc in APSMpSRResetMPS
# (normally run after a beam dump).
#
# Revision 1.104  2007/05/30 18:03:32  soliday
# Added a stabilization time to the SR Dipole before conditioning.
# Also added code to restore the kicker and septum setpoints.
#
# Revision 1.103  2007/03/27 13:36:54  soliday
# Added the ability to start the kickers and septums when zone F is selected.
#
# Revision 1.102  2007/03/27 13:14:30  soliday
# The kickers and septums are turned off when sector F is selected.
#
# Revision 1.101  2007/01/18 16:08:45  soliday
# Updated to work with the new domain names.
#
# Revision 1.100  2006/10/10 14:00:20  soliday
# Force conditioning is now set by default.
#
# Revision 1.99  2006/02/21 17:27:13  soliday
# Added code to change the background of the cycles entry box in the initial
# dialog if the number of cycles is excessive.
#
# Revision 1.98  2006/02/17 22:05:17  soliday
# Changed the wording of the option to force conditioning of power supplies
# even if it is not required. Also added context help for this option
# to explain why conditioning is not required all the time.
#
# Revision 1.97  2006/02/07 16:13:16  soliday
# Installed after testing usage of the conditioning validity PVs.
#
# Revision 1.96  2005/12/13 16:03:07  shang
# modified WaitForTaskToFinish procedure to printout all error messages.
#
# Revision 1.95  2005/07/18 21:21:29  soliday
# Added code to produce an error message if no sectors are selected.
#
# Revision 1.94  2005/07/11 21:25:21  soliday
# Added additional checks to test if the dipole has tripped off while conditioning.
#
# Revision 1.93  2005/04/07 16:31:13  soliday
# Updated to identify limited save files and not allow the operator to use them.
#
# Revision 1.92  2004/12/08 13:37:58  soliday
# Backed out of the last two changes.
#
# Revision 1.91  2004/12/08 12:53:14  soliday
# Similar to last change.
#
# Revision 1.90  2004/12/08 12:49:25  soliday
# Increased the APSSrDcpsAction -action on number of tries and pause time.
#
# Revision 1.89  2004/11/08 22:58:45  soliday
# Added the option of not turning off the raw power supplies.
#
# Revision 1.88  2004/10/19 08:55:44  emery
# Added arguments -checkStatus 0 to calls to APSSetRTFBCorrectorMode.
# Otherwise the stauts of the PS will be tested and give an error
# when some of the PS's are turned off by the pem itself.
#
# Revision 1.87  2004/07/01 19:02:12  soliday
# The correctors are now always put into scalar mode when conditioning even
# if they were not selected. It also aborts the DC orbit controllaws before
# doing conditioning.
#
# Revision 1.86  2004/05/25 02:35:44  emery
# Added call to APSSetRTFBCorrectorMode following calls to APSSetCorrMode
# so that RTFB correctors are changed as well as regular correctors.
#
# Revision 1.85  2004/05/19 23:21:18  emery
# Fixed problem with variable used for turning off and setting
# MT family.
#
# Revision 1.84  2004/05/19 21:39:14  emery
# Fixed several erroroneous assignments of MTList and unusedMTList lists.
#
# Revision 1.83  2004/05/19 15:15:54  emery
# Swicthed the variables MTsectors and unusedMTsectors because
# the XAXSTF sectors no longer will use MT supplies as the BM
# magnets will be made weaker by 1 mrad, while the non-XAXASTF
# sectors will need the MT to get the extra 1 mrad.
#
# Revision 1.82  2004/03/30 19:04:13  emery
# Added manual mps dump and reset mps before standardizing. (Soliday).
# Fixed a typo in a standardization template file name.
#
# Revision 1.81  2004/03/23 15:58:25  soliday
# Fixed zone buttons in the condition double sector initial dialog.
#
# Revision 1.80  2004/03/23 14:15:40  soliday
# Updated the dialog messages displayed to the user when operator intervention
# is needed. Also converted channel access calls to use APScavget and APScavput.
#
# Revision 1.79  2004/03/16 17:46:10  soliday
# Reindented code.
#
# Revision 1.78  2004/03/10 10:49:39  emery
# Added -noWarning to sddsxref command in standardization configure section
# where the new file /home/helios/oagData/sr/magnetConditioning/template.std
# is used. (A warning would occur where there are more magnet in template.std
# than in the SCR file.)
#
# Revision 1.77  2004/03/10 10:19:29  emery
# Added 4 second wait time after setting scalar mode
# and before corrector setpoints are sent 0 A values.
#
# Revision 1.76  2004/03/10 10:07:01  emery
# Forgot some parenthesis in last version.
#
# Revision 1.75  2004/03/10 10:01:49  emery
# Make use of APSSetCorrMode in parts where correctors are brought
# down and degaussed. Removed unnecessary cavput to corrector's
# ControlModeBO, which is taken care of internally by APSSetCorrMode.
#
# Revision 1.74  2004/03/03 20:27:39  emery
# Replace /home/helios/oagData/sr/magnetConditioning/lattices/default/
# default.std file with /home/helios/oagData/sr/magnetConditioning/template.std
# Added 1 second of wait time between setting correctors to vector
# mode and sending scaler setpoint values to correctors.
#
# Revision 1.73  2004/03/01 22:33:00  soliday
# Added support for the canted undulator power supplies.
#
# Revision 1.72  2004/02/17 15:22:32  soliday
# non XAXSTF MT supplies are turned off prior to conditioning the dipole.
#
# Revision 1.71  2004/01/28 18:03:56  soliday
# Updated misleading descriptions related to setting the power supplies to zero.
#
# Revision 1.70  2003/11/20 17:24:56  soliday
# The 100Hz reset is also done with the UpDoubleSector pem because this also
# conditions the power supplies.
#
# Revision 1.69  2003/11/18 16:22:24  soliday
# Added 100Hz glitch reset procedure after conditioning. I also changed it
# so that it conditions the V1 correctors.
#
# Revision 1.68  2003/09/24 15:12:30  soliday
# It does not stop anymore if logDaemon is not running.
#
# Revision 1.67  2003/08/12 21:36:46  shang
# setting the corrector mode to scalar for conditioning correctors
#
# Revision 1.66  2003/01/23 19:44:33  soliday
# The accelerator subnet changed to accel.ntw0rk
#
# Revision 1.65  2002/10/04 03:01:23  emery
# Forgot to convert argument of "after" command into an integer.
#
# Revision 1.64  2002/10/04 02:56:48  emery
# Added prefix and suffix arguments to APSRampPVs to
# allow simpler device name-oriented PV list as argument.
#
# Revision 1.63  2002/10/04 02:46:38  emery
# Added test to length of nameAndValue list before running cavput command
# for non-steering PS.
#
# Revision 1.62  2002/10/04 02:40:54  emery
# After conditioning, the correctors are ramped to the target values
# with a new procedure APSRampPVs.
#
# Revision 1.61  2002/10/04 01:14:38  emery
# Removed an extraneous command to set the corrector to operation
# mode after degaussing. Replaced many arguments of configFile to NULL
# where a file wasn't necessary. Indented code.
#
# Revision 1.60  2002/10/03 20:45:41  soliday
# Added an extra wait time after setting correctors in operation mode
#
# Revision 1.59  2002/10/02 20:39:46  shang
# removed set PSCU mode and set selected correctors to Operation before degaussing.
#
# Revision 1.58  2002/10/02 20:21:05  shang
# replaced the deagussing config file
#
# Revision 1.57  2002/10/02 19:38:02  shang
# set the default file to be UBOP and added set corrector mode to Operation before degaussing
#
# Revision 1.56  2002/09/23 15:06:48  emery
# Indented with spaces and changed one instance of OffBO=1 to OnBO=0.
#
# Revision 1.55  2002/07/23 01:55:44  borland
# Conditioning configuration changed so that the "To" value is always greater
# than or equal to the "Finish" value.
#
# Revision 1.54  2002/06/06 14:46:09  soliday
# Fixed another dialog message.
#
# Revision 1.53  2002/06/04 14:04:14  soliday
# Changed some of the dialogs.
#
# Revision 1.52  2002/06/04 07:01:03  borland
# Try [file tail $ShortFilename] to solve a problem with new code that
# standardizes to setpoint.
#
# Revision 1.51  2002/05/30 18:46:24  soliday
# The standardization routines finish at the SCR setpoint now.
#
# Revision 1.50  2002/03/05 15:37:09  soliday
# Changed the pend from 10 seconds to 30.
#
# Revision 1.49  2002/03/04 18:35:49  soliday
# Added pend to the cavput command for the PSCU/Diag mode
#
# Revision 1.48  2002/03/04 16:01:02  soliday
# Added support for PSCU and diag mode in the correctors.
#
# Revision 1.47  2002/01/28 17:16:05  soliday
# Added reset PVs for startup.
#
# Revision 1.46  2001/10/26 15:19:04  soliday
# Changed dialog screen.
#
# Revision 1.45  2001/06/27 15:25:27  soliday
# It now skips steps if not on aps4.anl.gov. This is for training purposes.
# It also calibrates BM at the end of the script.
#
# Revision 1.44  2001/05/29 16:23:01  soliday
# Changed action variable to action1 because of a conflict.
#
# Revision 1.43  2001/04/30 17:52:01  soliday
# Updated the UpDoubleSector description so that it is apparent that the
# power supplies are conditioned.
#
# Revision 1.42  2001/02/06 21:21:49  soliday
# Added the EMW power supply in sector 11 to the shutdown pem.
#
# Revision 1.41  2000/12/08 16:28:25  soliday
# Changed variables passed to APSMpSRConditionDoubleSectorDialog.
#
# Revision 1.40  2000/11/13 22:35:59  soliday
# Added Zone buttons.
#
# Revision 1.39  2000/10/27 05:12:04  emery
# Forgot a $ in front of XAXSTFsectors which prevented the H1s from being set.
#
# Revision 1.38  2000/10/27 04:52:32  emery
# Fixed case error on XAXSTFsectors.
#
# Revision 1.37  2000/10/26 17:23:20  emery
# Get the list of operating dipole trims and H1 magnets frmo
# the XAXSTF/sectors.sdds file.
#
# Revision 1.36  2000/09/07 21:49:32  emery
# Added sector 32 to a list of tests for setting or not
# setting H1s. Indented code.
#
# Revision 1.35  2000/07/29 19:23:17  emery
# Added S32 MT.
#
# Revision 1.34  2000/05/30 23:41:49  soliday
# Fixed APSMpStep init line so that there are no dialog screens after it.
#
# Revision 1.33  2000/05/30 21:51:01  soliday
# Added init steps so that logDaemon logs the start of the pem execution.
#
# Revision 1.32  2000/04/26 19:14:03  soliday
# Added more detail to the APSMpStep descriptions.
#
# Revision 1.31  2000/04/26 15:47:42  soliday
# Added APSMpStep descriptions.
#
# Revision 1.30  2000/04/10 17:19:09  soliday
# Improved speed and added changes made durring shutdown period.
#
# Revision 1.29  2000/01/19 06:24:01  emery
# Replace old default dipole and quad/sext standardization file
# with /home/helios/oagData/sr/magnetConditioning/lattices/default/default.std
#
# Revision 1.28  1999/12/15 19:35:09  soliday
# Added another dialog box to let the operator continue if a PS failed to start
#
# Revision 1.27  1999/12/13 17:04:36  soliday
# Added an option to continue if a power supply failed to turn on
#
# Revision 1.26  1999/11/15 20:01:46  soliday
# It now sets the current to 0 on all power supplies before it turns
# them on.
#
# Revision 1.25  1999/11/15 19:36:11  borland
# The default is now to configure the power supplies.
#
# Revision 1.24  1999/08/30 17:16:40  soliday
# Now stops and starts BControl when conditioning
#
# Revision 1.23  1999/08/24 19:02:17  soliday
# Only sectors 34 and 35 trims are now used
#
# Revision 1.20  1999/08/17 20:52:43  soliday
# QS is now turned on as set when Q is conditioned.
#
# Revision 1.19  1999/07/27 01:13:02  soliday
# Increased range that is allowed before power supplies are shut off
#
#

proc APSMpSRUpDoubleSectorInfo {} {
    return "Brings two consecutive SR sectors up by turning the converters and corresponding raw supplies on and loading the appropriate set points after conditioning the power supplies."
}

proc APSMpSRUpDoubleSectorDialog {frame} {
    global status APSMpSRUpDoubleSector

    set status "Select SR Sectors"
    pack [frame $frame.f2]
    pack [frame $frame.f2.f1] -side left -fill y
    pack [frame $frame.f2.f2] -side left
#    pack [frame $frame.f2.f3] -side left
    pack [label $frame.f2.f1.sectorSelectLabel -text "Storage Ring Sector Selection"]
    APSCheckButtonFrame .sectorSelectButtons \
        -parent $frame.f2.f1 \
        -label "" \
        -variableList {APSMpSRUpDoubleSector(S40:01) APSMpSRUpDoubleSector(S02:03) \
                           APSMpSRUpDoubleSector(S04:05) APSMpSRUpDoubleSector(S06:07) \
                           APSMpSRUpDoubleSector(S08:09) APSMpSRUpDoubleSector(S10:11) \
                           APSMpSRUpDoubleSector(S12:13) APSMpSRUpDoubleSector(S14:15) \
                           APSMpSRUpDoubleSector(S16:17) APSMpSRUpDoubleSector(S18:19) \
                           APSMpSRUpDoubleSector(S20:21) APSMpSRUpDoubleSector(S22:23) \
                           APSMpSRUpDoubleSector(S24:25) APSMpSRUpDoubleSector(S26:27) \
                           APSMpSRUpDoubleSector(S28:29) APSMpSRUpDoubleSector(S30:31) \
                           APSMpSRUpDoubleSector(S32:33) APSMpSRUpDoubleSector(S34:35) \
                           APSMpSRUpDoubleSector(S36:37) APSMpSRUpDoubleSector(S38:39)} \
        -buttonList {S40:01 S02:03 S04:05 S06:07 S08:09 S10:11 S12:13 \
                         S14:15 S16:17 S18:19 S20:21 S22:23 S24:25 S26:27 \
                         S28:29 S30:31 S32:33 S34:35 S36:37 S38:39} \
        -allNone 1 \
        -orientation horizontal \
        -limitPerRow 4
    APSButton .a -parent $frame.f2.f1.sectorSelectButtons.frame -text A -size small -command "$frame.f2.f1.sectorSelectButtons.frame.none.button invoke; for {set i 2} {\$i <= 8} {incr i} {set APSMpSRUpDoubleSector(S0\$i:0\[expr \$i + 1\]) 1} ; set APSMpSRUpDoubleSector(kickers) 0"
    APSButton .b -parent $frame.f2.f1.sectorSelectButtons.frame -text B -size small -command "$frame.f2.f1.sectorSelectButtons.frame.none.button invoke; for {set i 10} {\$i <= 16} {incr i} {set APSMpSRUpDoubleSector(S\$i:\[expr \$i + 1\]) 1} ; set APSMpSRUpDoubleSector(kickers) 0"
    APSButton .c -parent $frame.f2.f1.sectorSelectButtons.frame -text C -size small -command "$frame.f2.f1.sectorSelectButtons.frame.none.button invoke; for {set i 18} {\$i <= 22} {incr i} {set APSMpSRUpDoubleSector(S\$i:\[expr \$i + 1\]) 1} ; set APSMpSRUpDoubleSector(kickers) 0"
    APSButton .d -parent $frame.f2.f1.sectorSelectButtons.frame -text D -size small -command "$frame.f2.f1.sectorSelectButtons.frame.none.button invoke; for {set i 24} {\$i <= 28} {incr i} {set APSMpSRUpDoubleSector(S\$i:\[expr \$i + 1\]) 1} ; set APSMpSRUpDoubleSector(kickers) 0"
    APSButton .e -parent $frame.f2.f1.sectorSelectButtons.frame -text E -size small -command "$frame.f2.f1.sectorSelectButtons.frame.none.button invoke; for {set i 30} {\$i <= 35} {incr i} {set APSMpSRUpDoubleSector(S\$i:\[expr \$i + 1\]) 1} ; set APSMpSRUpDoubleSector(kickers) 0"
    APSButton .f -parent $frame.f2.f1.sectorSelectButtons.frame -text F -size small -command "$frame.f2.f1.sectorSelectButtons.frame.none.button invoke; for {set i 36} {\$i <= 38} {incr i} {set APSMpSRUpDoubleSector(S\$i:\[expr \$i + 1\]) 1}; set APSMpSRUpDoubleSector(S40:01) 1 ; set APSMpSRUpDoubleSector(kickers) 1"
    $frame.f2.f1.sectorSelectButtons.frame.all.button configure -command "[$frame.f2.f1.sectorSelectButtons.frame.all.button cget -command] ; set APSMpSRUpDoubleSector(kickers) 1"
    $frame.f2.f1.sectorSelectButtons.frame.none.button configure -command "[$frame.f2.f1.sectorSelectButtons.frame.none.button cget -command] ; set APSMpSRUpDoubleSector(kickers) 0"
    APSCheckButtonFrame .kicker \
      -parent $frame.f2.f1 \
      -label "" \
      -variableList APSMpSRUpDoubleSector(kickers) \
      -buttonList {"Turn on kickers and septums"} \
      -orientation horizontal

    pack [label $frame.f2.f2.groupSelectLabel -text "Power Supply Selection"]
    APSCheckButtonFrame .groupSelectButtons \
      -parent $frame.f2.f2 \
      -label "" \
      -variableList {APSMpSRUpDoubleSector(BM) APSMpSRUpDoubleSector(C:BM) \
                       APSMpSRUpDoubleSector(Q) APSMpSRUpDoubleSector(QS)  \
                       APSMpSRUpDoubleSector(S) APSMpSRUpDoubleSector(H)} \
        -buttonList {"DIPOLE (BM) + TRIMs (MT)" "Canted Undulator DIPOLEs (C:BM)" "QUADs  (Q)" "SKEWs  (QS)" "SEXTs  (S)" "HCORRs (H) + VCORRs (V) | SKEWs (QS4)"} \
        -allNone 1 \
        -orientation horizontal \
        -limitPerRow 1
#    pack [label $frame.f2.f3.groupSelectLabel -text "Kicker Selection"]
#    APSCheckButtonFrame .kickerSelectButtons -parent $frame.f2.f3 -label "" -variableList {APSMpSRUpDoubleSector(IK1) APSMpSRUpDoubleSector(IK2) APSMpSRUpDoubleSector(IK3) APSMpSRUpDoubleSector(IK4) APSMpSRUpDoubleSector(IK5)} -buttonList {"S:IK1" "S:IK2" "S:IK3" "S:IK4" "S:IK5"} -allNone 1 -orientation horizontal -limitPerRow 1
    pack [frame $frame.f1]
    APSLabeledEntry .cycles -parent $frame.f1 -label "Number of Cycles:" \
        -textVariable APSMpSRUpDoubleSector(cycles) -width 3 -packOption "-side left"

    bindtags $frame.f1.cycles.entry "Entry .promptDialog all $frame.f1.cycles.entry"
    bind $frame.f1.cycles.entry <KeyPress> "
    if \"(\[llength \[set APSMpSRUpDoubleSector(cycles)\]\]) && (\[string is integer \[set APSMpSRUpDoubleSector(cycles)\]\])\" \{
        if \"(\[llength \[set APSMpSRUpDoubleSector(cycles)\]\]) && (\[set APSMpSRUpDoubleSector(cycles)\] <= 3) && (\[set APSMpSRUpDoubleSector(cycles)\] >= 1)\" \{ $frame.f1.cycles.entry configure -bg white \} else \{ $frame.f1.cycles.entry configure -bg yellow \}
        \} else \{ $frame.f1.cycles.entry configure -bg yellow \}
    "
    bind $frame.f1.cycles.entry <ButtonRelease-2> "
    if \"(\[llength \[set APSMpSRUpDoubleSector(cycles)\]\]) && (\[string is integer \[set APSMpSRUpDoubleSector(cycles)\]\])\" \{
        if \"(\[llength \[set APSMpSRUpDoubleSector(cycles)\]\]) && (\[set APSMpSRUpDoubleSector(cycles)\] <= 3) && (\[set APSMpSRUpDoubleSector(cycles)\] >= 1)\" \{ $frame.f1.cycles.entry configure -bg white \} else \{ $frame.f1.cycles.entry configure -bg yellow \}
        \} else \{ $frame.f1.cycles.entry configure -bg yellow \}
    "


    pack [checkbutton $frame.configure -text "Configure Power Supplies" \
              -variable APSMpSRUpDoubleSector(configure) -padx 4 -pady 4 -relief ridge] -padx 5
    pack [checkbutton $frame.condition -text "Force conditioning" \
              -variable APSMpSRUpDoubleSector(forcecondition) -padx 4 -pady 4 -relief ridge] -padx 5
    pack [checkbutton $frame.launch -text "Launch APSCalibrateDCPS" \
              -variable APSMpSRUpDoubleSector(apsCalibrate) -padx 4 -pady 4 -relief ridge] -padx 5
    global apsContextHelp
    set apsContextHelp($frame.condition) "Normally, magnets will not be conditioned if the IOC indicates that they are already properly conditioned.  If you have a good reason to override this, select Force Conditioning.  Don't use this unnecessarily since the purpose is to avoid excessive conditioning."

    update
    APSAddSCRDialog .scr -parent $frame -system SR -arrayName APSMpSRUpDoubleSector \
        -defaultFile $APSMpSRUpDoubleSector(ShortFilename) -description $APSMpSRUpDoubleSector(Description)
    
    return
}

proc APSMpSRUpDoubleSector {args} {
    set sectorGroups {S40:01 S02:03 S04:05 S06:07 S08:09 S10:11 S12:13 \
                          S14:15 S16:17 S18:19 S20:21 S22:23 S24:25 S26:27 \
                          S28:29 S30:31 S32:33 S34:35 S36:37 S38:39}
    set QS4Sectors {1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40}
    #2013/09/19 QS4 replace A:V4 in above sectors, which means there will no AV4 correctors in QS4Sectors
    set QS4sectorList ""
    set VsectorList ""
    foreach sectorGroup $sectorGroups {
        set $sectorGroup 0
    }
    set configure 1
    set forcecondition 0
    set cycles 3
    set ShortFilename [APSFindSCROperationFile -machine SR -arrayName APSMpSRUpDoubleSector]
    set apsCalibrate 0
    set families "BM C:BM Q QS S H V QS4 MT"
    foreach family $families {
        set $family 1
    }
    set kickers 0
    APSStrictParseArguments "$sectorGroups ShortFilename cycles configure BM C:BM Q QS S H V QS4 MT apsCalibrate forcecondition kickers"
    APSMpStep init \
        -promptDialog APSMpSRUpDoubleSectorDialog \
        -vars "$sectorGroups ShortFilename cycles configure BM C:BM Q QS S H V QS4 MT apsCalibrate forcecondition kickers" -proc "APSMpSRUpDownDoubleSector forcecondition"
    if {![string length $ShortFilename]} {
        APSMpReturn error "APSMpSRUpDoubleSector: SCR file not choosen!"
    }
    if {![file exists /home/helios/oagData/SCR/snapshots/SR/$ShortFilename]} {
        APSMpReturn error "SCR file $ShortFilename not found"
    }
    if {[catch {exec sdds2stream /home/helios/oagData/SCR/snapshots/SR/$ShortFilename -parameter=SnapshotDescription -page=1 -noquotes} results]} {
        APSMpReturn error "APSMpSRUpDoubleSector: Invalid SCR file"
    }    
    if {[string first "Limited save:" $results] != -1} {
        APSMpReturn error "APSMpSRUpDoubleSector: LIMITED SAVE SCR files cannot be used for this procedure."
    }

    set MT $BM
    set V $H
    set QS4 $H
    set standardizeQ $Q
    set standardizeS $S
    set conditionBM $BM
    set conditionC:BM ${C:BM}
    set conditionQS $QS
    set degauss $H
    set sectorArgs ""
    foreach sectorGroup $sectorGroups {
        append sectorArgs "-$sectorGroup [set $sectorGroup] "
    }

    if {$kickers} {
	APSMpStep "Turning on the kickers and septums"
	set id1 [APSMpParallel -mode "Automatic" -procedure "APSMpSRTurnOnKickers -IS2 0"]
    }
    set id2 [eval APSMpParallel -mode "Automatic" -procedure "\"APSMpSRUpDownDoubleSector $sectorArgs -state on -standardizeQ $standardizeQ -standardizeS $standardizeS -conditionBM $conditionBM -conditionC:BM ${conditionC:BM} -degauss $degauss -conditionQS $conditionQS -ShortFilename $ShortFilename -cycles $cycles -configure $configure -BM $BM -C:BM ${C:BM} -Q $Q -QS $QS -S $S -H $H -V $V -QS4 $QS4 -MT $MT -apsCalibrate $apsCalibrate -forcecondition $forcecondition\""]

    if {$kickers} {
	if {[catch {APSMpJoin $id1} results]} {
	    if {[APSPEMTKDialog .warningDialog -text "Problem occured turning the kickers and septums on: $results"] == 1} {
		APSMpReturn error "APSMpSRUpDoubleSector: $results"
	    }
	} else {
            set tmpFile /tmp/[APSTmpString]
            if [catch {exec sddsprocess /home/helios/oagData/SCR/snapshots/SR/$ShortFilename $tmpFile \
                         "-match=column,ControlName=S:IS1:VoltageSetSendAO,ControlName=S:IK\[1-4\]:VoltageSetSendAO,|"} result] {
                APSPEMTKDialog .warningDialog -strings Continue -default 0 -text "Problem restoring kicker and septum setpoints: $result"
                catch {APSSendEMail -address soliday \
                         -subject "Problem restoring  kicker and septum setpoints."  \
                         -message "Problem restoring  kicker and septum setpoints.\n$result"} 
                
            }
            if [catch {exec sddscasr -restore=verify $tmpFile} result] {
                APSPEMTKDialog .warningDialog -strings Continue -default 0 -text "Problem restoring kicker and septum setpoints: $result"
                catch {APSSendEMail -address soliday \
                         -subject "Problem restoring  kicker and septum setpoints."  \
                         -message "Problem restoring  kicker and septum setpoints.\n$result"} 
                
            }
           catch {file delete $tmpFile}
            
            #TEMP Changes until we can resolve the caused of the over voltage or over current condition. (Al Hillman)
            if {[catch {APSSRPulsedPSDischargeEnable -PSList IS1} results]} {
                APSMpReturn error "APSMpSRUpDoubleSector: $results"
            }
	    APSMpStep "SR: Wait for IS:1 septum to reach operating voltage"
            set retries 180
            while {$retries} {
                after 1000
                if {[catch {APScavget -list=S:IS1:VoltageSetSendAO,S:IS1:DacAI} results]} {
                    APSMpReturn error "APSMpSRUpDoubleSector: $results"
                }
                set percentage [expr [lindex $results 1] / [lindex $results 0]]
                if {$percentage > .95} {
                    break
                }
                incr retries -1
            }
            if {[catch {APSSRPulsedPSDischargeEnable -PSList IS1 -disable 1} results]} {
                APSMpReturn error "APSMpSRUpDoubleSector: $results"
            }

            #TEMP changes for IS2
            APSMpSRTurnOnKickers -IK1 0 -IK2 0 -IK3 0 -IK4 0 -IK5 0 -IS1 0 -IS2 1

            set tmpFile /tmp/[APSTmpString]
            if [catch {exec sddsprocess /home/helios/oagData/SCR/snapshots/SR/$ShortFilename $tmpFile \
                         "-match=column,ControlName=S:IS2:VoltageSetSendAO"} result] {
                APSPEMTKDialog .warningDialog -strings Continue -default 0 -text "Problem restoring kicker and septum setpoints: $result"
                catch {APSSendEMail -address soliday \
                         -subject "Problem restoring  kicker and septum setpoints."  \
                         -message "Problem restoring  kicker and septum setpoints.\n$result"} 

            }
            if [catch {exec sddscasr -restore=verify $tmpFile} result] {
                APSPEMTKDialog .warningDialog -strings Continue -default 0 -text "Problem restoring kicker and septum setpoints: $result"
                catch {APSSendEMail -address soliday \
                         -subject "Problem restoring  kicker and septum setpoints."  \
                         -message "Problem restoring  kicker and septum setpoints.\n$result"} 

            }
            catch {file delete $tmpFile}

            if {[catch {APSSRPulsedPSDischargeEnable -PSList IS2} results]} {
                APSMpReturn error "APSMpSRUpDoubleSector: $results"
            }
	    APSMpStep "SR: Wait for IS:2 septum to reach operating voltage"
            set retries 180
            while {$retries} {
                after 1000
                if {[catch {APScavget -list=S:IS2:VoltageSetSendAO,S:IS2:DacAI} results]} {
                    APSMpReturn error "APSMpSRUpDoubleSector: $results"
                }
                set percentage [expr [lindex $results 1] / [lindex $results 0]]
                if {$percentage > .95} {
                    break
                }
                incr retries -1
            }
            if {[catch {APSSRPulsedPSDischargeEnable -PSList IS2 -disable 1} results]} {
                APSMpReturn error "APSMpSRUpDoubleSector: $results"
            }
            #END temp changes
        }
    }
    if {[catch {APSMpJoin $id2} results]} {
	if {[APSPEMTKDialog .warningDialog -text "Problem occured starting double sectors: $results"] == 1} {
	    APSMpReturn error "APSMpSRUpDoubleSector: $results"
	}
    }

    

    APSMpReturn ok
}

proc APSMpSRConditionDoubleSectorInfo {} {
    return "Conditions the selected power supplies\
if their power supplies are already on."
}

proc APSMpSRConditionDoubleSectorDialog {frame} {
    global status
    global APSMpSRConditionDoubleSector
    set status "Select SR Sectors"
    pack [frame $frame.f2]
    pack [frame $frame.f2.f3] -side left -fill y
    pack [frame $frame.f2.f2] -side left -fill y
    pack [frame $frame.f2.f1] -side left -fill y

    APSRadioButtonFrame .stateRadioButtons \
        -parent $frame.f2.f3 \
        -label "Choose Operation" \
        -variable APSMpSRConditionDoubleSector(stateVal) \
        -buttonList {"Start Conditioning" "Abort Conditioning"} \
        -valueList {"condition" "abortCondition"}

    pack [label $frame.f2.f2.sectorSelectLabel -text "Storage Ring Sector Selection"]
    APSCheckButtonFrame .sectorSelectButtons \
        -parent $frame.f2.f2 \
        -label "" \
        -variableList {APSMpSRConditionDoubleSector(S40:01) APSMpSRConditionDoubleSector(S02:03) \
                           APSMpSRConditionDoubleSector(S04:05) APSMpSRConditionDoubleSector(S06:07) \
                           APSMpSRConditionDoubleSector(S08:09) APSMpSRConditionDoubleSector(S10:11) \
                           APSMpSRConditionDoubleSector(S12:13) APSMpSRConditionDoubleSector(S14:15) \
                           APSMpSRConditionDoubleSector(S16:17) APSMpSRConditionDoubleSector(S18:19) \
                           APSMpSRConditionDoubleSector(S20:21) APSMpSRConditionDoubleSector(S22:23) \
                           APSMpSRConditionDoubleSector(S24:25) APSMpSRConditionDoubleSector(S26:27) \
                           APSMpSRConditionDoubleSector(S28:29) APSMpSRConditionDoubleSector(S30:31) \
                           APSMpSRConditionDoubleSector(S32:33) APSMpSRConditionDoubleSector(S34:35) \
                           APSMpSRConditionDoubleSector(S36:37) APSMpSRConditionDoubleSector(S38:39)} \
        -buttonList {S40:01 S02:03 S04:05 S06:07 S08:09 S10:11 S12:13 \
                         S14:15 S16:17 S18:19 S20:21 S22:23 S24:25 S26:27 \
                         S28:29 S30:31 S32:33 S34:35 S36:37 S38:39} \
        -allNone 1 \
        -orientation horizontal \
        -limitPerRow 4
    APSButton .a -parent $frame.f2.f2.sectorSelectButtons.frame -text A -size small -command "$frame.f2.f2.sectorSelectButtons.frame.none.button invoke; for {set i 2} {\$i <= 8} {incr i} {set APSMpSRConditionDoubleSector(S0\$i:0\[expr \$i + 1\]) 1}"
    APSButton .b -parent $frame.f2.f2.sectorSelectButtons.frame -text B -size small -command "$frame.f2.f2.sectorSelectButtons.frame.none.button invoke; for {set i 10} {\$i <= 16} {incr i} {set APSMpSRConditionDoubleSector(S\$i:\[expr \$i + 1\]) 1}"
    APSButton .c -parent $frame.f2.f2.sectorSelectButtons.frame -text C -size small -command "$frame.f2.f2.sectorSelectButtons.frame.none.button invoke; for {set i 18} {\$i <= 22} {incr i} {set APSMpSRConditionDoubleSector(S\$i:\[expr \$i + 1\]) 1}"
    APSButton .d -parent $frame.f2.f2.sectorSelectButtons.frame -text D -size small -command "$frame.f2.f2.sectorSelectButtons.frame.none.button invoke; for {set i 24} {\$i <= 28} {incr i} {set APSMpSRConditionDoubleSector(S\$i:\[expr \$i + 1\]) 1}"
    APSButton .e -parent $frame.f2.f2.sectorSelectButtons.frame -text E -size small -command "$frame.f2.f2.sectorSelectButtons.frame.none.button invoke; for {set i 30} {\$i <= 35} {incr i} {set APSMpSRConditionDoubleSector(S\$i:\[expr \$i + 1\]) 1}"
    APSButton .f -parent $frame.f2.f2.sectorSelectButtons.frame -text F -size small -command "$frame.f2.f2.sectorSelectButtons.frame.none.button invoke; for {set i 36} {\$i <= 38} {incr i} {set APSMpSRConditionDoubleSector(S\$i:\[expr \$i + 1\]) 1}; set APSMpSRConditionDoubleSector(S40:01) 1"

    APSCheckButtonFrame .configurations \
      -parent $frame.f2.f1 \
      -label "Conditioning Configuration" \
      -variableList {APSMpSRConditionDoubleSector(conditionBM) \
                       APSMpSRConditionDoubleSector(conditionC:BM) \
                       APSMpSRConditionDoubleSector(standardizeQ) \
                       APSMpSRConditionDoubleSector(conditionQS) \
                       APSMpSRConditionDoubleSector(standardizeS) \
                       APSMpSRConditionDoubleSector(degauss)} \
      -buttonList {"DIPOLE (BM)" "Canted Undulator DIPOLEs (C:BM)" "QUADs (Q)" "SKEWs (QS)" "SEXTs (S)" "HCORRs (H) + QS4"} \
      -allNone 1

    pack [frame $frame.f1]
    APSLabeledEntry .cycles -parent $frame.f1 -label "Number of Cycles:" \
        -textVariable APSMpSRConditionDoubleSector(cycles) -width 3

    bindtags $frame.f1.cycles.entry "Entry .promptDialog all $frame.f1.cycles.entry"
    bind $frame.f1.cycles.entry <KeyPress> "
    if \"(\[llength \[set APSMpSRConditionDoubleSector(cycles)\]\]) && (\[string is integer \[set APSMpSRConditionDoubleSector(cycles)\]\])\" \{
        if \"(\[llength \[set APSMpSRConditionDoubleSector(cycles)\]\]) && (\[set APSMpSRConditionDoubleSector(cycles)\] <= 3) && (\[set APSMpSRConditionDoubleSector(cycles)\] >= 1)\" \{ $frame.f1.cycles.entry configure -bg white \} else \{ $frame.f1.cycles.entry configure -bg yellow \}
        \} else \{ $frame.f1.cycles.entry configure -bg yellow \}
    "
    bind $frame.f1.cycles.entry <ButtonRelease-2> "
    if \"(\[llength \[set APSMpSRConditionDoubleSector(cycles)\]\]) && (\[string is integer \[set APSMpSRConditionDoubleSector(cycles)\]\])\" \{
        if \"(\[llength \[set APSMpSRConditionDoubleSector(cycles)\]\]) && (\[set APSMpSRConditionDoubleSector(cycles)\] <= 3) && (\[set APSMpSRConditionDoubleSector(cycles)\] >= 1)\" \{ $frame.f1.cycles.entry configure -bg white \} else \{ $frame.f1.cycles.entry configure -bg yellow \}
        \} else \{ $frame.f1.cycles.entry configure -bg yellow \}
    "

    pack [checkbutton $frame.configure -text "Configure Power Supplies" \
              -variable APSMpSRConditionDoubleSector(configure) \
              -padx 4 -pady 4 -relief ridge] -pady 5
    pack [checkbutton $frame.condition -text "Force conditioning" \
              -variable APSMpSRConditionDoubleSector(forcecondition) \
              -padx 4 -pady 4 -relief ridge] -pady 5
    pack [checkbutton $frame.launch -text "Launch APSCalibrateDCPS" \
              -variable APSMpSRConditionDoubleSector(apsCalibrate) \
              -padx 4 -pady 4 -relief ridge]
    global apsContextHelp
    set apsContextHelp($frame.condition) "Normally, magnets will not be conditioned if the IOC indicates that they are already properly conditioned.  If you have a good reason to override this, select Force Conditioning.  Don't use this unnecessarily since the purpose is to avoid excessive conditioning."

    update
    APSAddSCRDialog .scr -parent $frame -system SR -arrayName APSMpSRConditionDoubleSector \
        -defaultFile $APSMpSRConditionDoubleSector(ShortFilename) -description $APSMpSRConditionDoubleSector(Description)
    
    return
}

proc APSMpSRConditionDoubleSector {args} {
    set sectorGroups {S40:01 S02:03 S04:05 S06:07 S08:09 S10:11 S12:13 \
                          S14:15 S16:17 S18:19 S20:21 S22:23 S24:25 S26:27 \
                          S28:29 S30:31 S32:33 S34:35 S36:37 S38:39}
    foreach sectorGroup $sectorGroups {
        set $sectorGroup 0
    }
    set standardizeQ 1
    set standardizeS 1
    set conditionBM 1
    set conditionQS 1
    set conditionC:BM 1
    set degauss 1
    set cycles 3
    set configure 1
    set apsCalibrate 0
    set forcecondition 0
    set ShortFilename [APSFindSCROperationFile -machine SR -arrayName APSMpSRConditionDoubleSector]
    set state condition
    APSStrictParseArguments "$sectorGroups standardizeQ standardizeS conditionBM conditionC:BM conditionQS degauss ShortFilename cycles configure apsCalibrate state forcecondition"
    set stateVal $state
    APSMpStep init \
        -promptDialog APSMpSRConditionDoubleSectorDialog \
        -vars "$sectorGroups standardizeQ standardizeS conditionBM conditionC:BM conditionQS degauss ShortFilename cycles configure apsCalibrate stateVal forcecondition" -proc "APSMpSRUpDownDoubleSector"

    if {![string length $ShortFilename]} {
        APSMpReturn error "APSMpSRConditionDoubleSector: SCR file not choosen!"
    }
    if {![file exists /home/helios/oagData/SCR/snapshots/SR/$ShortFilename]} {
        APSMpReturn error "SCR file $ShortFilename not found"
    }
    if {[catch {exec sdds2stream /home/helios/oagData/SCR/snapshots/SR/$ShortFilename -parameter=SnapshotDescription -page=1 -noquotes} results]} {
        APSMpReturn error "APSMpSRConditionDoubleSector: Invalid SCR file"
    }    
    if {[string first "Limited save:" $results] != -1} {
        APSMpReturn error "APSMpSRConditionDoubleSector: LIMITED SAVE SCR files cannot be used for this procedure."
    }

    set Q $standardizeQ
    set QS $conditionQS
    set S $standardizeS
    set MT [set BM $conditionBM]
    set V [set H $degauss]
    set QS4 $V
    set C:BM ${conditionC:BM}
    set sectorArgs ""
    foreach sectorGroup $sectorGroups {
        append sectorArgs "-$sectorGroup [set $sectorGroup] "
    }

    if {[catch {eval APSMpSRUpDownDoubleSector $sectorArgs -state $stateVal -standardizeQ $standardizeQ -standardizeS $standardizeS -conditionBM $conditionBM -conditionC:BM ${conditionC:BM} -conditionQS $conditionQS -degauss $degauss -ShortFilename $ShortFilename -cycles $cycles -configure $configure -BM $BM -C:BM ${C:BM} -Q $Q -QS $QS -S $S -H $H -V $V -QS4 $QS4 -MT $MT -apsCalibrate $apsCalibrate -forcecondition $forcecondition} results]} {
        APSMpReturn error "$results"
    }
    APSMpReturn ok
}

proc APSMpSRDownDoubleSectorInfo {} {
    return "Brings two consecutive SR sectors down\
by turning the converters and corresponding raw\
  supplies off."
}

proc APSMpSRDownDoubleSectorDialog {frame} {

    global status APSMpSRDownDoubleSector

    set status "Select SR Sectors"
    pack [frame $frame.f2]
    pack [frame $frame.f2.f1] -side left -fill y
    pack [frame $frame.f2.f2] -side left
    pack [label $frame.f2.f1.sectorSelectLabel -text "Storage Ring Sector Selection"]
    APSCheckButtonFrame .sectorSelectButtons \
        -parent $frame.f2.f1 \
        -label "" \
        -variableList {APSMpSRDownDoubleSector(S40:01) APSMpSRDownDoubleSector(S02:03) \
                           APSMpSRDownDoubleSector(S04:05) APSMpSRDownDoubleSector(S06:07) \
                           APSMpSRDownDoubleSector(S08:09) APSMpSRDownDoubleSector(S10:11) \
                           APSMpSRDownDoubleSector(S12:13) APSMpSRDownDoubleSector(S14:15) \
                           APSMpSRDownDoubleSector(S16:17) APSMpSRDownDoubleSector(S18:19) \
                           APSMpSRDownDoubleSector(S20:21) APSMpSRDownDoubleSector(S22:23) \
                           APSMpSRDownDoubleSector(S24:25) APSMpSRDownDoubleSector(S26:27) \
                           APSMpSRDownDoubleSector(S28:29) APSMpSRDownDoubleSector(S30:31) \
                           APSMpSRDownDoubleSector(S32:33) APSMpSRDownDoubleSector(S34:35) \
                           APSMpSRDownDoubleSector(S36:37) APSMpSRDownDoubleSector(S38:39)} \
        -buttonList {S40:01 S02:03 S04:05 S06:07 S08:09 S10:11 S12:13 \
                         S14:15 S16:17 S18:19 S20:21 S22:23 S24:25 S26:27 \
                         S28:29 S30:31 S32:33 S34:35 S36:37 S38:39} \
        -allNone 1 \
        -orientation horizontal \
        -limitPerRow 4
    APSButton .a -parent $frame.f2.f1.sectorSelectButtons.frame -text A -size small -command "$frame.f2.f1.sectorSelectButtons.frame.none.button invoke; for {set i 2} {\$i <= 8} {incr i} {set APSMpSRDownDoubleSector(S0\$i:0\[expr \$i + 1\]) 1} ; set APSMpSRDownDoubleSector(kickers) 0; set APSMpSRDownDoubleSector(btsoff) 0"
    APSButton .b -parent $frame.f2.f1.sectorSelectButtons.frame -text B -size small -command "$frame.f2.f1.sectorSelectButtons.frame.none.button invoke; for {set i 10} {\$i <= 16} {incr i} {set APSMpSRDownDoubleSector(S\$i:\[expr \$i + 1\]) 1} ; set APSMpSRDownDoubleSector(kickers) 0; set APSMpSRDownDoubleSector(btsoff) 0"
    APSButton .c -parent $frame.f2.f1.sectorSelectButtons.frame -text C -size small -command "$frame.f2.f1.sectorSelectButtons.frame.none.button invoke; for {set i 18} {\$i <= 22} {incr i} {set APSMpSRDownDoubleSector(S\$i:\[expr \$i + 1\]) 1} ; set APSMpSRDownDoubleSector(kickers) 0; set APSMpSRDownDoubleSector(btsoff) 0"
    APSButton .d -parent $frame.f2.f1.sectorSelectButtons.frame -text D -size small -command "$frame.f2.f1.sectorSelectButtons.frame.none.button invoke; for {set i 24} {\$i <= 28} {incr i} {set APSMpSRDownDoubleSector(S\$i:\[expr \$i + 1\]) 1} ; set APSMpSRDownDoubleSector(kickers) 0; set APSMpSRDownDoubleSector(btsoff) 0"
    APSButton .e -parent $frame.f2.f1.sectorSelectButtons.frame -text E -size small -command "$frame.f2.f1.sectorSelectButtons.frame.none.button invoke; for {set i 30} {\$i <= 35} {incr i} {set APSMpSRDownDoubleSector(S\$i:\[expr \$i + 1\]) 1} ; set APSMpSRDownDoubleSector(kickers) 0; set APSMpSRDownDoubleSector(btsoff) 0"
    APSButton .f -parent $frame.f2.f1.sectorSelectButtons.frame -text F -size small -command "$frame.f2.f1.sectorSelectButtons.frame.none.button invoke; for {set i 36} {\$i <= 38} {incr i} {set APSMpSRDownDoubleSector(S\$i:\[expr \$i + 1\]) 1}; set APSMpSRDownDoubleSector(S40:01) 1 ; set APSMpSRDownDoubleSector(kickers) 1; set APSMpSRDownDoubleSector(btsoff) 1"
    $frame.f2.f1.sectorSelectButtons.frame.all.button configure -command "[$frame.f2.f1.sectorSelectButtons.frame.all.button cget -command] ; set APSMpSRDownDoubleSector(kickers) 1; set APSMpSRDownDoubleSector(btsoff) 1"
    $frame.f2.f1.sectorSelectButtons.frame.none.button configure -command "[$frame.f2.f1.sectorSelectButtons.frame.none.button cget -command] ; set APSMpSRDownDoubleSector(kickers) 0; set APSMpSRDownDoubleSector(btsoff) 0"
    APSCheckButtonFrame .kicker \
      -parent $frame.f2.f1 \
      -label "" \
      -variableList APSMpSRDownDoubleSector(kickers) \
      -buttonList {"Turn off kickers and septums"} \
      -orientation horizontal
    APSCheckButtonFrame .bts \
      -parent $frame.f2.f1 \
      -label "" \
      -variableList APSMpSRDownDoubleSector(btsoff) \
      -buttonList {"Turn off SR side of the BTS"} \
      -orientation horizontal


    pack [label $frame.f2.f2.groupSelectLabel -text "Power Supply Selection"]
    APSCheckButtonFrame .groupSelectButtons \
        -parent $frame.f2.f2 \
        -label "" \
        -variableList {APSMpSRDownDoubleSector(BM) APSMpSRDownDoubleSector(C:BM) \
                         APSMpSRDownDoubleSector(Q) APSMpSRDownDoubleSector(QS)  \
                         APSMpSRDownDoubleSector(S) APSMpSRDownDoubleSector(H) \
                         APSMpSRDownDoubleSector(excludeRaws)} \
        -buttonList {"DIPOLE (BM) + TRIMs (MT)" "Canted Undulator DIPOLEs (C:BM)" "QUADs  (Q)" "SKEWs  (QS)"  "SEXTs  (S)" "HCORRs (H) + VCORRs (V) | SKEWS (QS4)" "Exclude Raw Supplies"} \
        -allNone 1 \
        -orientation horizontal \
        -limitPerRow 1
    update
    APSAddSCRDialog .scr -parent $frame -system SR -arrayName APSMpSRDownDoubleSector \
        -defaultFile $APSMpSRDownDoubleSector(ShortFilename) -description $APSMpSRDownDoubleSector(Description)

    return
}

proc APSMpSRDownDoubleSector {args} {
    set sectorGroups {S40:01 S02:03 S04:05 S06:07 S08:09 S10:11 S12:13 \
                          S14:15 S16:17 S18:19 S20:21 S22:23 S24:25 S26:27 \
                          S28:29 S30:31 S32:33 S34:35 S36:37 S38:39}
    foreach sectorGroup $sectorGroups {
        set $sectorGroup 0
    }
    set ShortFilename [APSFindSCROperationFile -machine SR -arrayName APSMpSRDownDoubleSector]
    set families "BM C:BM Q QS S H V MT"
    foreach family $families {
        set $family 1
    }
    set excludeRaws 0
    set kickers 0
    set btsoff 0
    APSStrictParseArguments "$sectorGroups ShortFilename BM C:BM Q QS S H V MT excludeRaws kickers btsoff"
    APSMpStep init \
        -promptDialog APSMpSRDownDoubleSectorDialog \
        -vars "$sectorGroups ShortFilename BM C:BM Q QS S H V MT excludeRaws kickers btsoff" -proc "APSMpSRUpDownDoubleSector"
    if {![string length $ShortFilename]} {
        APSMpReturn error "APSMpSRDownDoubleSector: SCR file not choosen!"
    }
    if {![file exists /home/helios/oagData/SCR/snapshots/SR/$ShortFilename]} {
        APSMpReturn error "SCR file $ShortFilename not found"
    }
    if {[catch {exec sdds2stream /home/helios/oagData/SCR/snapshots/SR/$ShortFilename -parameter=SnapshotDescription -page=1 -noquotes} results]} {
        APSMpReturn error "APSMpSRDownDoubleSector: Invalid SCR file"
    }    
    if {[string first "Limited save:" $results] != -1} {
        APSMpReturn error "APSMpSRDownDoubleSector: LIMITED SAVE SCR files cannot be used for this procedure."
    }

    if {$BM} {set MT 1} else {set MT 0}
    if {$H} {
        set V 1
        set QS4 1
    } else {
        set QS4 0
        set V 0
    }
    set sectorArgs ""
    foreach sectorGroup $sectorGroups {
        append sectorArgs "-$sectorGroup [set $sectorGroup] "
    }
    if {$kickers} {
	APSMpStep "Turning off the kickers and septums"
        if {[catch {APSMpSRTurnOffKickers} results]} {
            APSMpReturn error "$results"
        }
    }
    if {$btsoff} {
	APSMpStep "Turning off the SR side of the BTS"
        if {[catch {APSMpBTSTurnOffPS -turnOffList "BTS:BH BTS:BV BTS:BQ BTS:CH BTS:CV BTS:CQ BTS:BB" -turnOffListDetailed "BTS:BH1 BTS:BH2 BTS:BV1 BTS:BQ1 BTS:BQ2 BTS:BQ3 BTS:BQ4 BTS:CH1 BTS:CV1 BTS:CQ1 BTS:CQ2 BTS:CQ3 BTS:BB"} results]} {
            APSMpReturn error "$results"
        }
    }
    APSMpStep "APSMpSRUpDownDoubleSector $sectorArgs $QS4"
    if {[catch {eval APSMpSRUpDownDoubleSector $sectorArgs -state off -standardizeQ 0 -standardizeS 0 -conditionBM 0 -conditionC:BM 0 -conditionQS 0 -degauss 0 -ShortFilename $ShortFilename -cycles 0 -configure 0 -BM $BM -C:BM ${C:BM} -Q $Q -QS $QS -S $S -H $H -V $V -QS4 $QS4 -MT $MT -apsCalibrate 0 -excludeRaws $excludeRaws} results]} {
        APSMpReturn error "$results"
    }
    APSMpReturn ok
}

proc APSMpSRUpDownDoubleSector {args} {
    global apsNetworkDomain
    set sectorGroups {S40:01 S02:03 S04:05 S06:07 S08:09 S10:11 S12:13 \
                          S14:15 S16:17 S18:19 S20:21 S22:23 S24:25 S26:27 \
                          S28:29 S30:31 S32:33 S34:35 S36:37 S38:39}
    foreach sectorGroup $sectorGroups {
        set $sectorGroup 0
    }
    set QS4Sectors {1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40}
    #2013/09/19 QS4 replace A:V4 in above sectors, which means there will no AV4 correctors in QS4Sectors
    set QS4sectorList ""
    set VsectorList ""
    
    set state on
    set standardizeQ 1
    set standardizeS 1
    set conditionBM 1
    set conditionC:BM 1
    set conditionQS 1
    set degauss 1
    set cycles 3
    set configure 1
    set apsCalibrate 0
    set forcecondition 0
    set ShortFilename [APSFindSCROperationFile -machine SR -arrayName APSMpSRUpDownDoubleSector]
    set XAXSTFsectors [exec sddsprocess /home/helios/oagData/sr/XAXSTF/sectors.sdds -pipe=out "-filter=column,Sector,6.9,7.1,!" | \
                         sdds2stream -pipe=in -col=Sector]
    set XAXSTFcuflag [exec sddsprocess /home/helios/oagData/sr/XAXSTF/sectors.sdds -pipe=out "-filter=column,Sector,6.9,7.1,!" | \
                        sdds2stream -pipe=in -col=CUflag]
    set XAXSTFcubmflag [exec sddsprocess /home/helios/oagData/sr/XAXSTF/sectors.sdds -pipe=out "-filter=column,Sector,6.9,7.1,!" | \
                          sdds2stream -pipe=in -col=CUdipoleExists]

    # Example value for MTList:
    # {S02:03 {2 A} {2 B}} {S02:03 {3 A} {3 B}} {S04:05 {4 A} {4 B}} {S32:33 {32 A} {32 B}} {S32:33 {33 A} {33 B}} {S34:35 {34 A} {34 B}} {S34:35 {35 A} {35 B}}
    set families "BM Q QS S H V QS4 MT C:BM C:H C:V"
    foreach family $families {
        set $family 1
    }
    set excludeRaws 0
    
    APSStrictParseArguments "$sectorGroups state standardizeQ standardizeS conditionBM conditionC:BM conditionQS degauss ShortFilename cycles configure BM C:BM Q QS S H V QS4 MT apsCalibrate excludeRaws forcecondition"

    foreach param "state standardizeQ standardizeS conditionBM conditionC:BM conditionQS degauss ShortFilename cycles configure BM C:BM Q QS S H V QS4 MT apsCalibrate forcecondition" {
        append paramString "$param=\"[set $param]\" "
    }
    
    if {$forcecondition} {
        set condition "forcecondition"
    } else {
        set condition "condition"
    }
    
    
    set sectorPairs {"40 1 " "2 3 " "4 5 " "6 7 " "8 9 " "10 11 " "12 13 " \
                       "14 15 " "16 17 " "18 19 " "20 21 " "22 23 " "24 25 " \
                         "26 27 " "28 29 " "30 31 " "32 33 " "34 35 " "36 37 " "38 39 "}
    foreach sectorGroup $sectorGroups sectorPair $sectorPairs {
        if {[set $sectorGroup]} {
            append sectors $sectorPair
            lappend sectorsOdd [lindex $sectorPair 1]
            lappend sectorPairWithColon "[lindex $sectorPair 0]:[lindex $sectorPair 1]"
        }
    }
    
    if {![info exists sectors]} {
        APSMpReturn error "APSMpSRUpDownDoubleSector: No sectors chosen"
    }
    foreach sector $sectors {
        if [lsearch -exact $QS4Sectors $sector]>=0 {
            lappend QS4sectorList $sector
        } else {
            lappend VsectorList $sector
        }
    }
    APSMpStep "QS4 sectors $QS4sectorList"
    set MTList ""
    set CUDipoleList ""
    set CUCorrList ""
    set unusedMTList ""
    set nonXAXSTFsectors ""
    for {set sector 1} {$sector <= 40} {incr sector} {
        set evenSector [format %02ld [expr $sector / 2 * 2]]
        set oddSector [format %02ld [expr $sector / 2 * 2 + 1]]
        if {$evenSector == 0 } {
            set evenSector 40
        }
        if {$oddSector == 41 } {
            set oddSector 01
        }
        set j [lsearch -exact $XAXSTFsectors $sector]
        set j2 [lsearch -exact $sectors $sector]
        # now the XAXSTF sectors no longer need the MT powered on.
        # It is the non-XAXSTF sectors that need them.
        if {($j != -1)} {
            # Basically the variable unusedMTList is now the same as 
            # variable XAXSTFsectors now.
            append unusedMTList "$sector "
            if {[lindex $XAXSTFcuflag $j] == 1} {
                lappend CUCorrList $sector
            }
            if {[lindex $XAXSTFcubmflag $j] == 1} {
                lappend CUDipoleList $sector
            }
        } elseif {($j == -1)} {
            lappend nonXAXSTFsectors $sector
            lappend MTList [list "S$evenSector:$oddSector" "$sector A" "$sector B"]
        }
    }

    set MTcavgetOptions ""
    foreach MTItem $MTList {
        foreach MTSubItem [lrange $MTItem 1 end] {
            lappend MTcavgetOptions S[join $MTSubItem ""]:MT
        }
    }
    
    set cycles2 $cycles
    if {$cycles == 0} {
        set cycles2 1
    }
    APSMpStep "log script action..."
    if {[catch {APSLogScriptAction \
                  -procedure APSMpSRUpDownDoubleSector \
                  -action initialize \
                  -state ok \
                  -parameters $paramString} results]} {
        #APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
    }
    
    if {![string length $ShortFilename]} {
        APSMpReturn error "APSMpSRUpDownDoubleSector: SCR file was not choosed!"
    }
    if {$ShortFilename == [file tail $ShortFilename]} {
        set ShortFilename [file join /home/helios/oagData/SCR/snapshots/SR/ $ShortFilename]
    }
    if {[catch {exec sdds2stream $ShortFilename -parameter=SnapshotDescription -page=1 -noquotes} results]} {
        APSMpReturn error "APSMpSRDownDoubleSector: Invalid SCR file"
    }    
    if {[string first "Limited save:" $results] != -1} {
        APSMpReturn error "APSMpSRDownDoubleSector: LIMITED SAVE SCR files cannot be used for this procedure."
    }
    
    APSMpStep "check state..."
    #If the user wants to condition the the power supplies make sure the corresponding 
    #power supplies were choosen
    if {(($state == "on") || ($state == "condition") || ($state == "abortCondition"))} {
        if {$conditionBM} {
            set MT [set BM 1]
        }
        if {${conditionC:BM}} {
            set C:BM 1
        }
        if {$conditionQS} {
            set QS 1
        }
        if {$standardizeQ} {
            set Q 1
        }
        if {$standardizeS} {
            set S 1
        }
        if {$degauss} {
            set H 1
            set V 1
            set QS4 1
        }
    }
    APSMpStep "H $H, V $V"
    if {!$H} {
        set C:H 0
    }
    if {!$V} {
        set C:V 0
    }
    APSMpStep "QS4 $QS4"
    #Ensure that at least one sectorpair was chosen
    if {![info exists sectors]} {
        if {(!${C:BM}) && (!$Q) && (!$QS) && (!$S) && (!$H) && (!$V) && (!$QS4) && $BM} {
            set sectors ""
        } else {
            APSMpReturn error "APSMpSRUpDownDoubleSector: No sectors chosen"
        }
    }

    set tmp ""
    foreach s $CUCorrList {
        if {[lsearch -exact $sectors $s] != -1} {
            lappend tmp $s
        }
    }
    set CUCorrList $tmp
    set tmp ""
    foreach s $CUDipoleList {
        if {[lsearch -exact $sectors $s] != -1} {
            lappend tmp $s
        }
    }
    set CUDipoleList $tmp

    set corrMode scalar
    if {$BM} {
        APSMpStep "Stopping Conditioning of S:BM Dipole" -desc "Stop conditioning of S:BM dipole"
        if {[catch {APSSrDcpsAction \
                      -family BM \
                      -action stopcond \
                      -value 0 \
                      -sectors $sectors \
                      -pause 10 \
                      -configFile "/home/helios/oagData/sr/magnetConditioning/template.std"} results]} {
            if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: Problem stopping conditioning of S:BM Dipole \n$results"] == 1} {
                APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
            }
        }
    }
    if {${C:BM}} {
        if {[llength $CUDipoleList]} {
            APSMpStep "Stopping Conditioning of Canted Undulator Dipoles" -desc "Stop conditioning of Canted Undulator Dipoles"
            if {[catch {APSSrDcpsAction \
                          -family C:BM \
                          -action stopcond \
                          -value 0 \
                          -sectors $CUDipoleList \
                          -pause 10 \
                          -configFile "/home/helios/oagData/sr/magnetConditioning/template.std"} results]} {
                if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: Problem stopping conditioning of canted undulator dipoles \n$results"] == 1} {
                    APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
                }
            }
        }
    }
    foreach family "Q S" {
        if {[set $family]} {
            APSMpStep "Stopping Conditioning of $family power supplies" -desc "Stop conditioning of Q and S power supplies"
            if {[catch {APSSrDcpsAction \
                          -family $family \
                          -action stopcond \
                          -value 0 \
                          -sectors $sectors \
                          -pause 10 \
                            -configFile "/home/helios/oagData/sr/magnetConditioning/template.std"} results]} {
                if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: Problem stopping conditioning of $family power supplies \n$results"] == 1} {
                    APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
                }
            }
        }
    }
    if {$QS} {
        APSMpStep "Stopping Degaussing of QS power supplies" -desc "Stop conditioning of QS power supplies"
        if {[catch {APSSrDcpsAction \
                      -family QS \
                      -action stopcond \
                      -value 0 \
                      -sectors $sectors \
                      -pause 10 \
                      -configFile "/home/helios/oagData/sr/magnetConditioning/SQS_degauss.sdds"} results]} {
            if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: Problem stopping degaussing of QS power supplies \n$results"] == 1} {
                APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
            }
        }
    }
    if {$H} {
        APSMpStep "Stopping Degaussing of H power supplies" -desc "Stop conditioning of H power supplies"
        if {[catch {APSSrDcpsAction \
                      -family H \
                      -action stopcond \
                      -value 0 \
                      -sectors $sectors \
                      -pause 10 \
                      -configFile "/home/helios/oagData/sr/magnetConditioning/S_degauss.sdds"} results]} {
            if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: Problem stopping degaussing of H power supplies \n$results"] == 1} {
                APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
            }
        }
        if {[llength $CUCorrList]} {
            APSMpStep "Stopping Degaussing of Canted Undulator H power supplies" -desc "Stop conditioning of Canted Undulator H power supplies"
            if {[catch {APSSrDcpsAction \
                          -family C:H \
                          -action stopcond \
                          -value 0 \
                          -sectors $CUCorrList \
                          -pause 10 \
                          -configFile "/home/helios/oagData/sr/magnetConditioning/S_degauss.sdds"} results]} {
                if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: Problem stopping degaussing of canted undulator H power supplies \n$results"] == 1} {
                    APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
                }
            }
        }
    }
    if {$state == "abortCondition"} {
        APSMpReturn ok
    }
    #Make sure the the SCR File exists
    if {![file isfile $ShortFilename]} {
        APSMpReturn error "APSMpSRUpDownDoubleSector: Invalid SCR File $ShortFilename"
    }
    #Finding active power supplies by searching the the ShortFilename
    #for :CurrentAOs
    APSMpStep "Finding active power supplies" -desc "Find active power supplies"
    set match -match=column,ControlName=S:BM:CurrentAO
    
    foreach sector $sectors {
        append match ",ControlName=S${sector}\[AB\]:Q\[1-5\]:CurrentAO,|,ControlName=S${sector}\[AB\]:S\[1-4\]:CurrentAO,|,ControlName=S${sector}\[AB\]:\[HV\]\[1-4\]:CurrentAO,|,ControlName=S${sector}\[AB\]:MT:CurrentAO,|,ControlName=S${sector}C:BM:CurrentAO,|,ControlName=S${sector}C:\[HV\]1:CurrentAO,|,ControlName=S${sector}A:QS4:CurrentAO,|"
        #louie says this is not required    
        #        if {-1 < [lsearch $XAXSTFsectors $sector] } {
        #            append match ",ControlName=S${sector}\[AB\]:Q\[1-5\]:CurrentAO,|,ControlName=S${sector}\[AB\]:S\[1-4\]:CurrentAO,|,ControlName=S${sector}\[AB\]:\[HV\]\[1-4\]:CurrentAO,|,ControlName=S${sector}\[AB\]:MT:CurrentAO,|"  
        #        } else {
        #            append match ",ControlName=S${sector}\[AB\]:Q\[1-5\]:CurrentAO,|,ControlName=S${sector}\[AB\]:S\[1-4\]:CurrentAO,|,ControlName=S${sector}\[AB\]:\[HV\]\[2-4\]:CurrentAO,|,ControlName=S${sector}\[AB\]:MT:CurrentAO,|"
        #        }
    }
    
    if {$BM} {
        foreach MTItem $MTList {
            if {![set [lindex $MTItem 0]]} {
                foreach MTSubItem [lrange $MTItem 1 end] {
                    append match ",ControlName=S[join $MTSubItem ""]:MT:CurrentAO,|"
                }
            }
        }
    }
    if {[info exists sectorsOdd]} {
        foreach sectorOdd $sectorsOdd {
            append match ",ControlName=S${sectorOdd}\[AB\]:QS:CurrentAO,|"
        }
    }
    
    if {[catch {exec sddsprocess $ShortFilename -pipe=out \
                  -match=column,ControlName=*:CurrentAO $match \
                  | sdds2stream -pipe=in -columns=ControlName,ValueString} results]} {
        APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
    }
    foreach {name value} $results {
        lappend activeConverters $name
        lappend activeConverterValues $value
    }
    regsub -all {:CurrentAO} $activeConverters {} activeConverters
    
    foreach family $families {
        set activeSelected${family}Converters ""
        set activeSelected${family}Values ""
    }
    foreach activeConverter $activeConverters value $activeConverterValues {
        if {([regexp -- {S[1-9][0-9]?[AB]:Q[1-5]} $activeConverter]) && ($Q)} {
            lappend activeSelectedQConverters $activeConverter
            lappend activeSelectedQValues $value
        } elseif {([regexp -- {S[1-9][0-9]?[ABC]:H[1-4]} $activeConverter]) && ($H)} {
            if {($activeConverter != "S1A:H3") && 
                ($activeConverter != "S40A:H3") &&
                ($activeConverter != "S39B:H1")} {
                lappend activeSelectedHConverters $activeConverter
                lappend activeSelectedHValues $value
            }
            #[2-4] Changed because louie said it is not required
        } elseif {([regexp -- {S[1-9][0-9]?[ABC]:V[1-4]} $activeConverter]) && ($V)} {
            if {($activeConverter != "S1A:V3") && 
                ($activeConverter != "S40A:V3") && 
                ($activeConverter != "S39B:V1") &&
                ($activeConverter != "S1A:V4") &&
                ($activeConverter != "S2A:V4") &&
                ($activeConverter != "S3A:V4") &&
                ($activeConverter != "S4A:V4") &&
                ($activeConverter != "S5A:V4") &&
                ($activeConverter != "S6A:V4") &&
                ($activeConverter != "S7A:V4") &&
                ($activeConverter != "S8A:V4") &&
                ($activeConverter != "S9A:V4") &&
                ($activeConverter != "S10A:V4") &&
                ($activeConverter != "S11A:V4") &&
                ($activeConverter != "S12A:V4") &&
                ($activeConverter != "S13A:V4") &&
                ($activeConverter != "S14A:V4") &&
                ($activeConverter != "S15A:V4") &&
                ($activeConverter != "S16A:V4") &&
                ($activeConverter != "S17A:V4") &&
                ($activeConverter != "S18A:V4") &&
                ($activeConverter != "S19A:V4") &&
                ($activeConverter != "S20A:V4") &&
                ($activeConverter != "S21A:V4") &&
                ($activeConverter != "S22A:V4") &&
                ($activeConverter != "S23A:V4") &&
                ($activeConverter != "S24A:V4") &&
                ($activeConverter != "S25A:V4") &&
                ($activeConverter != "S26A:V4") &&
                ($activeConverter != "S27A:V4") &&
                ($activeConverter != "S28A:V4") &&
                ($activeConverter != "S29A:V4") &&
                ($activeConverter != "S30A:V4") &&
                ($activeConverter != "S31A:V4") &&
                ($activeConverter != "S32A:V4") &&
                ($activeConverter != "S33A:V4") &&
                ($activeConverter != "S34A:V4") &&
                ($activeConverter != "S35A:V4") &&
                ($activeConverter != "S36A:V4") &&
                ($activeConverter != "S37A:V4") &&
                ($activeConverter != "S38A:V4") &&
                ($activeConverter != "S39A:V4") &&
                ($activeConverter != "S40A:V4")} { 
                lappend activeSelectedVConverters $activeConverter
                lappend activeSelectedVValues $value
            }
        } elseif {([regexp -- {S[1-9][0-9]?[AB]:S[1-4]} $activeConverter]) && ($S)} {
            lappend activeSelectedSConverters $activeConverter
            lappend activeSelectedSValues $value
        }  elseif {([regexp -- {S[1-9][0-9]?A:QS4} $activeConverter]) && ($QS4)} {
            if {($activeConverter == "S1A:QS4") || 
                ($activeConverter == "S2A:QS4") || 
                ($activeConverter == "S3A:QS4") || 
                ($activeConverter == "S4A:QS4") || 
                ($activeConverter == "S5A:QS4") || 
                ($activeConverter == "S6A:QS4") || 
                ($activeConverter == "S7A:QS4") || 
                ($activeConverter == "S8A:QS4") || 
                ($activeConverter == "S9A:QS4") || 
                ($activeConverter == "S10A:QS4") || 
                ($activeConverter == "S11A:QS4") || 
                ($activeConverter == "S12A:QS4") || 
                ($activeConverter == "S13A:QS4") || 
                ($activeConverter == "S14A:QS4") || 
                ($activeConverter == "S15A:QS4") || 
                ($activeConverter == "S16A:QS4") || 
                ($activeConverter == "S17A:QS4") || 
                ($activeConverter == "S18A:QS4") || 
                ($activeConverter == "S19A:QS4") || 
                ($activeConverter == "S20A:QS4") || 
                ($activeConverter == "S21A:QS4") || 
                ($activeConverter == "S22A:QS4") || 
                ($activeConverter == "S23A:QS4") || 
                ($activeConverter == "S24A:QS4") || 
                ($activeConverter == "S25A:QS4") || 
                ($activeConverter == "S26A:QS4") || 
                ($activeConverter == "S27A:QS4") || 
                ($activeConverter == "S28A:QS4") || 
                ($activeConverter == "S29A:QS4") || 
                ($activeConverter == "S30A:QS4") || 
                ($activeConverter == "S31A:QS4") || 
                ($activeConverter == "S32A:QS4") || 
                ($activeConverter == "S33A:QS4") || 
                ($activeConverter == "S34A:QS4") || 
                ($activeConverter == "S35A:QS4") || 
                ($activeConverter == "S36A:QS4") || 
                ($activeConverter == "S37A:QS4") || 
                ($activeConverter == "S38A:QS4") || 
                ($activeConverter == "S39A:QS4") || 
                ($activeConverter == "S40A:QS4")} {
                lappend activeSelectedQS4Converters $activeConverter
                lappend activeSelectedQS4Values $value
            }
        }  elseif {([regexp -- {S[1-9][0-9]?[AB]:QS$} $activeConverter]) && ($QS)} {
            if {$activeConverter != "S1B:QS"} {
                lappend activeSelectedQSConverters $activeConverter
                lappend activeSelectedQSValues $value
            }
        } elseif {("S:BM" == $activeConverter) && ($BM)} {
            lappend activeSelectedBMConverters $activeConverter
            lappend activeSelectedBMValues $value
        } elseif {([regexp -- {S[1-9][0-9]?C:H1} $activeConverter]) && (${C:H})} {
            lappend activeSelectedC:HConverters $activeConverter
            lappend activeSelectedC:HValues $value
        } elseif {([regexp -- {S[1-9][0-9]?C:V1} $activeConverter]) && (${C:V})} {
            lappend activeSelectedC:VConverters $activeConverter
            lappend activeSelectedC:VValues $value
        } elseif {([regexp -- {S[1-9][0-9]?C:BM} $activeConverter]) && (${C:BM})} {
            lappend activeSelectedC:BMConverters $activeConverter
            lappend activeSelectedC:BMValues $value
        } elseif {$MT} {
            foreach MTItem $MTList {
                foreach MTSubItem [lrange $MTItem 1 end] {
                    if {$activeConverter == "S[join $MTSubItem ""]:MT"} {
                        lappend activeSelectedMTConverters $activeConverter
                        lappend activeSelectedMTValues $value
                    }
                }
            }
        }
    }

    set activeSelectedConverters "$activeSelectedBMConverters $activeSelectedQConverters $activeSelectedQSConverters $activeSelectedQS4Converters  $activeSelectedSConverters $activeSelectedHConverters $activeSelectedVConverters $activeSelectedQS4Converters $activeSelectedMTConverters ${activeSelectedC:HConverters} ${activeSelectedC:VConverters} ${activeSelectedC:BMConverters} "
    set activeSelectedConverterValues "$activeSelectedBMValues $activeSelectedQValues $activeSelectedQSValues $activeSelectedQS4Values $activeSelectedSValues $activeSelectedHValues $activeSelectedVValues $activeSelectedQS4Values $activeSelectedMTValues ${activeSelectedC:HValues} ${activeSelectedC:VValues} ${activeSelectedC:BMValues}"
    
    if {$state == "off"} {
        #Set all corresponding power supplies to zero.
        #NULL configFile is needed for QS family
        foreach family $families {
            if {[set $family]} {
                APSMpStep "Set $family family power supplies to 0" -desc "(Off) Set power supplies to 0"
                if { ($family == "H") || ($family == "V") } {
                    # If family is corrector, then set corrector to scalar mode 
                    # and operation mode
                    set plane $family
                    APSMpStep "Setting $plane correctors to operation mode and Scalar mode if not already" \
                      -desc "Setting $plane power supplies to Operation and Scalar mode"
                    if {[catch {APSSetCorrMode -plane $plane -sourceMode Operation -corrMode Scalar} results]} {
                        if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: Problem occurred while setting $plane correctors into operation and scalar mode \n$results"] == 1} {
                            APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
                        }
                    }
                    # Use checkStatus = 0 because we don't care whether the PS is on or not.
                    # During the pem, some PS's will be turned off.
                    if {[catch {APSSetRTFBCorrectorMode -all 1 -mode Operation -checkStatus 0 } results]} {
                        if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: Problem occurred while setting RTFB correctors into operations and scalar mode \n$results"] == 1} {
                            APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
                        }
                    }
                    # allow several seconds for the changes to take place.
                    APSWaitWithUpdate -waitSeconds 4
                }
                set sectors1 $sectors
                if {$family=="QS4"} {
                    set sectors1 $QS4sectorList
                }
                if {($family != "BM") && ([llength $sectors1] == 0)} {
                    continue
                }
                if {[catch {APSSrDcpsAction \
                              -family $family \
                              -action set \
                              -value 0 \
                              -sectors $sectors1 \
                              -pause 10 \
                              -configFile NULL} results]} {
                    if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: Problem occurred while setting $family correctors to 0 \n$result"] == 1} {
                        APSMpReturn error "APSMpSRUpDownDoubleSector: $result"
                    }
                }
            }
        }
        
        #Wait for corresponding current readbacks to reach zero.
        #Check status every few seconds until completed or timed out.
        if {[llength $activeSelectedBMConverters]} {
            APSMpStep "Waiting for BM power supplies to reach <10 Amps" -desc "(Off) Wait for BM power supplies to reach less then 10 Amps"
            if {[catch {WaitForTaskToFinish \
                          -cavget "APScavget -list=[join $activeSelectedBMConverters ,] -list=:CurrentAI -pend=30 -label" \
                          -waitLimit 12 \
                          -updateInterval 5 \
                          -checkExpression {($value < -10.00) || ($value > 10.00)}} results]} {
                if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: Problem occurred while waiting for BM to reach less than 10 amps \n$results"] == 1} {
                    APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
                }
            }
        }
        
        foreach family "C:BM Q QS QS4 S H V MT" {
            if {[llength [set activeSelected${family}Converters]]} {
                APSMpStep "Waiting for $family power supplies to reach <10 Amps" -desc "(Off) Wait for all power supplies to reach less then 10 Amps"
                if {[catch {WaitForTaskToFinish \
                              -cavget "APScavget -list=[join [set activeSelected${family}Converters] ,] -list=:CurrentAI -pend=30 -label" \
                              -waitLimit 6 \
                              -updateInterval 5 \
                              -checkExpression {($value < -10.0) || ($value > 10.0)}} results]} {
                    if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: Problem occurred while waiting for $family power supplies to reach less than 10 amps \n$results"] == 1} {
                        APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
                    }
                }
            }
        }
        
        #Turn off corresponding power supplies for all families.
        foreach family $families {
            if {[set $family]} {
                set sectors1 $sectors
                if {$family=="QS4"} {
                    set sectors1 $QS4sectorList
                }
                if {($family != "BM") && ([llength $sectors1] == 0)} {
                    continue
                }
                APSMpStep "Turning $family family power supplies off" -desc "(Off) Turn off power supplies"
                if {[catch {APSSrDcpsAction \
                              -family $family \
                              -action off \
                              -value 0 \
                              -sectors $sectors1 \
                              -pause 10 \
                              -configFile NULL} results]} {
                    if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: Problem occurred while turning $family power supplies off \n$results"] == 1} {
                        APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
                    }
                }
            }
        }
        
        #Wait for corresponding power supplies to turn off by checking the status of the LED.
        #Check status every few seconds until completed or timed out.
        APSMpStep "Waiting for power supplies to turn off" -desc "(Off) Wait for power supplies to turn off"
        foreach family "BM C:BM Q QS QS4 S MT" {
            if {[llength [set activeSelected${family}Converters]]} {
                if {[catch {WaitForTaskToFinish \
                              -cavget "APScavget -list=[join [set activeSelected${family}Converters] ,] -list=:StatusCALC -pend=30 -label" \
                              -waitLimit 3 \
                              -updateInterval 5 \
                              -checkExpression {($value != 2) && ($value != 3)}} results]} {
                    if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: Problem occurred while waiting for $family power supplies to turn off \n$results"] == 1} {
                        APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
                    }
                }
            }
        }
        foreach family "H V" {
            if {[llength [set activeSelected${family}Converters]]} {
                if {[catch {WaitForTaskToFinish \
                              -cavget "APScavget -list=[join [set activeSelected${family}Converters] ,] -list=:StatusCALC -pend=30 -label" \
                              -waitLimit 3 \
                              -updateInterval 5 \
                              -checkExpression {($value != 2) && ($value != 3)}} results]} {
                    if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: Problem occurred while waiting for $family power supplies to turn off \n$results"] == 1} {
                        APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
                    }
                }
            }
        }
        
        #If all families except BM and MT are not chosen, exit here
        foreach family $families {
            if {($family == "BM") || ($family == "MT")} {
                continue
            }
            if {![set $family]} {
                APSMpReturn ok
            }
        }
        if {$excludeRaws == 0} {
            #Turn off corresponding raw power supplies.
            APSMpStep "Turning off corresponding raw power supplies" -desc "(Off) Turn off corresponding raw power supplies"
            if {[catch {APSSrRpsAction \
                          -action off \
                          -group R \
                          -sectors $sectorsOdd} results]} {
                if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: Problem occurred while turning raw power supplies off \n$results"] == 1} {
                    APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
                }
            }
            
            #Wait for corresponding raw power supplies to turn off by checking the status of the LED.
            #Check status every few seconds until completed or timed out.
            APSMpStep "Waiting for raw power supplies to turn off" -desc "(Off) Waiting for raw power supplies to turn off"
            if {[catch {WaitForTaskToFinish \
                          -cavget "APScavget -list=S -list=[join $sectorPairWithColon ,] -list=:R -list=1,2,3,4 -list=:StatusCALC -pend=30 -label" \
                          -waitLimit 3 \
                          -updateInterval 5 \
                          -checkExpression {($value != 1)}} results]} {
                if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: Problem occurred while waiting for raw power supplies to turn off \n$results"] == 1} {
                    APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
                }
            }
        }
    } elseif {($state == "on") || ($state == "condition")} {
        APSMpStep "Checking for open shutters" -desc "(On or Condition) Checking for open shutters"
        if {[catch {APScavget -list=NoOfShuttersOpenA -pend=30} results]} {
            APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
        }
        if {$results != "0"} {
            if {[APSPEMTKDialog .warningDialog -text "$results open shutters found. Please rectify the problem."] == 1} {
                APSMpReturn error "APSMpSRUpDownDoubleSector: $results open shutters found. User aborted."
            }
        }
        
        
        APSMpStep "Checking for SR Current" -desc "(On or Condition) Checking for SR Current"
        if {[catch {APScavget -list=S35DCCT:currentCC -pend=30} results]} {
            APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
        }
        if { $results > .5 } {
            APSMpStep "Dumping beam" -desc "(On or Condition) Dump beam"
            if {[catch {APSMpSRDumpBeam} results]} {
                if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: \n$results"] == 1} {
                    APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
                }
            }
            if {[catch {APSMpSRResetMPS} results]} {
                if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: \n$results"] == 1} {
                    APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
                }
            }
        }
	if {$conditionBM} {
	    APSMpStep "Checking to see if the Dipole is conditioning" -desc "(On or Condition) Check to see if the dipole is conditioning"
	    if {[catch {WaitForTaskToFinish \
			  -cavget "APScavget -list=S:BM:StandardizeSUB.PROC -pend=30 -label" \
			  -waitLimit 1 \
			  -updateInterval 1 -message 0 \
			  -checkExpression {($value < -.01) || ($value > .01)}} results]} {
		if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: Problem occurred while checking if the dipole is conditioning \n$results"] == 1} {
		    APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
		}
	    }
	}
	if {${conditionC:BM}} {
            if {[llength $CUDipoleList]} {
                APSMpStep "Checking to see if the C:BM PS are conditioning" -desc "(On or Condition) Check to see if the C:BM PS are conditioning"
                if {[catch {WaitForTaskToFinish \
                              -cavget "APScavget -list=[join [set activeSelectedC:BMConverters] ,] -list=:StandardizeSUB.PROC -pend=30 -label" \
                              -waitLimit 1 \
                              -updateInterval 1 -message 0 \
                              -checkExpression {($value < -.01) || ($value > .01)}} results]} {
                    if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: Problem occurred while checking if the dipole is conditioning \n$results"] == 1} {
                        APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
                    }
                }
            }
        }
	foreach standardize "standardizeQ standardizeS conditionQS" family "Q S QS" {
	    if {([set $standardize]) && ([set $family])} {
		APSMpStep "Checking to see if the $family are conditioning" -desc "(On or Condition) Check to see if Q and S are conditioning"
		if {[catch {WaitForTaskToFinish \
			      -cavget "APScavget -list=[join [set activeSelected${family}Converters] ,] -list=:StandardizeSUB.PROC -pend=30 -label" \
			      -waitLimit 1 \
			      -updateInterval 1 -message 0 \
			      -checkExpression {($value < -.01) || ($value > .01)}} results]} {
		    if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: Problem occurred while checking if the $family power supplies are conditioning \n$results"] == 1} {
			APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
		    }
		}
	    }
	}

	if {$degauss} {
	    APSMpStep "Checking to see if the HCorrs are conditioning" -desc "(On or Condition) Check to see if the H correctors are conditioning"
	    if {[catch {WaitForTaskToFinish \
			  -cavget "APScavget -list=[join $activeSelectedHConverters ,] -list=:StandardizeSUB.PROC -pend=30 -label" \
			  -waitLimit 1 \
			  -updateInterval 1 -message 0 \
			  -checkExpression {($value < -.01) || ($value > .01)}} results]} {
		if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: Problem occurred while checking if the H power supplies are conditioning \n$results"] == 1} {
		    APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
		}
	    }
	}
	
	#Checking state of raw power supplies.
        if {[llength $sectors]} {
            APSMpStep "Checking state of raw power supplies" -desc "(On or Condition) Check the state of the raw power supplies"
            if {[catch {APScavget -list=S -list=[join $sectorPairWithColon ,] -list=:R -list=1,2,3,4 -list=:StatusCALC -pend=30 -label} results]} {
                APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
            }
            set nameAndValue ""
            set resetCabNameAndValue ""
            set resetFlowNameAndValue ""
            foreach "name value" $results {
                if {$value != 0} {
                    regsub -all {:StatusCALC} $name {} name
                    lappend nameAndValue ${name}:ResetSEQ=1
                    lappend nameAndValue ${name}:ONSEQ=1
                    lappend resetCabNameAndValue [string range $name 0 end-2]MagIntlkResetBO=1
                    lappend resetFlowNameAndValue [string range $name 0 end-2]PSIntlkResetBO=1
                }
            }
            if {[llength $nameAndValue]} {
                if {$state == "condition"} {
                    if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: At least one raw power supply is not on"] == 1} {
                        APSMpReturn error "APSMpSRUpDownDoubleSector: At least one raw power supply is not on"
                    }
                }
                
                #Reset MPEOBs for raw power supplies.
                APSMpStep "Reset MPEOBs for corresponding raw power supplies" -desc "(On) Reset MPEOBs for corresponding raw power supplies"
                if {[catch {APScavput -list=[join $resetCabNameAndValue ,] -blunderAhead -pend=30} results]} {
                    if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: Problem occurred while resetting BPEOBs for raw power supplies \n$results"] == 1} {
                        APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
                    }
                } 
                #Reset Flow signals for raw power supplies.
                APSMpStep "Reset Flow signals for corresponding raw power supplies" -desc "(On) Reset Flow signals for corresponding raw power supplies"
                if {[catch {APScavput -list=[join $resetFlowNameAndValue ,] -blunderAhead -pend=30} results]} {
                    if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: Problem occurred while resetting Flow signals for raw power supplies \n$results"] == 1} {
                        APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
                    }
                } 
                #Turn on corresponding raw power supplies.
                APSMpStep "Turning on corresponding raw power supplies" -desc "(On) Turn on corresponding raw power supplies"
                if {[catch {APScavput -list=[join $nameAndValue ,] -blunderAhead -pend=30} results]} {
                    if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: Problem occurred while turning on raw power supplies \n$results"] == 1} {
                        APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
                    }
                } 
                #Wait for corresponding raw power supplies to turn on by checking the status of the LED.
                #Check status every few seconds until completed or timed out.
                APSMpStep "Waiting for raw power supplies to turn on" -desc "(On) Waiting for raw power supplies to turn on"
                if {[catch {WaitForTaskToFinish \
                              -cavget "APScavget -list=S -list=[join $sectorPairWithColon ,] -list=:R -list=1,2,3,4 -list=:StatusCALC -pend=30 -label" \
                              -waitLimit 5 \
                                -updateInterval 5 \
                              -checkExpression {($value != 0)}} results]} {
                    if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: Problem occurred while waiting for raw power supplies to turn on\n$results"] == 1} {
                        APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
                    }
                }
            }
        }
        if {$state == "on"} {
            if {([string compare $apsNetworkDomain "accel.ntw0rk"] == 0) || ([string compare $apsNetworkDomain "aps4.anl.gov"] == 0)} {
                foreach sectorPair $sectorPairWithColon {
                    if {[catch {APScavput -list=S${sectorPair}:PSIntlkResetBO=1,S${sectorPair}:MagIntlkResetBO=1 -blunderAhead -pend=30} results]} {
                        if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: Problem occurred while reseting PS and magnet interlocks\n$results"] == 1} {
                            APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
                        }
                    }
                }
            }
            #Find power supplies that are off and set the current to 0
            APSMpStep "Finding power supplies that are off and setting the current to 0" -desc "(On) Find power supplies that are off and set the current to 0"
            if {[catch {APScavget -list=[join $activeSelectedConverters ,] -list=:StatusCALC -pend=30} results]} {
                APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
            }
            set nameAndValue ""
            foreach name $activeSelectedConverters value $results {
                if {($value != 0) && ($value != 1)} {
                    lappend nameAndValue ${name}:CurrentAO=0
                }
            }
            if {[llength $nameAndValue]} {
                if {[catch {APScavput -list=[join $nameAndValue ,] -blunderAhead -pend=30} results]} {
                    if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: Problem occurred while setting supplies that are off to 0 \n$results"] == 1} {
                        APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
                    }
                }
            }
            
            set retry 1
            while {$retry} {
                #Turn on corresponding power supplies for all families.
                foreach family "BM Q QS S H V QS4 C:BM C:H C:V" {
                    if {[set $family]} {
                        set sectors1 $sectors
                        if {$family=="QS4"} {
                            set sectors1 $QS4sectorList
                        }
                        if {($family != "BM") && ([llength $sectors1] == 0)} {
                            continue
                        }
                        APSMpStep "Turning $family family power supplies on" -desc "(On) Turn power supplies on"
                        if {[catch {APSSrDcpsAction \
                                      -family $family \
                                      -action on \
                                      -value 0 \
                                      -sectors $sectors1 \
                                      -pause 250 \
                                      -configFile NULL} results]} {
                            if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: Problem occurred while turning $family power supplies on \n$results"] == 1} {
                                APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
                            }
                        }
                    }
                }
                if {$BM} {
                    APSMpStep "Turning non-XAXSTF MT family power supplies on" -desc "(On) Turn non-XAXSTF MT power supplies on"
                    if {[catch {APSSrDcpsAction \
                                  -family MT \
                                  -action on \
                                  -value 0 \
                                  -sectors $nonXAXSTFsectors \
                                  -pause 250 \
                                  -configFile NULL} results]} {
                        if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: Problem occurred while turning MT power supplies on \n$results"] == 1} {
                            APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
                        }
                    }
                }
                
                #Wait for corresponding power supplies to turn on by checking the status of the LED.
                #Check status every few seconds until completed or timed out.
                APSMpStep "Waiting for power supplies to turn on" -desc "(On) Waiting for power supplies to turn on"
                if {[catch {WaitForTaskToFinish \
                              -cavget "APScavget -list=[join $activeSelectedConverters ,] -list=:StatusCALC -pend=30 -label" \
                              -waitLimit 2 \
                              -updateInterval 5 -message 0 \
                              -checkExpression {($value != 0) && ($value != 1)}} results]} {
                    incr retry
                } else {
                    set retry 0
                }
                if {$retry == 8} {
                    if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: Some of the power supplies failed to turn on"] == 1} {
                        APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
                    }
                    set retry 0
                }
                set DipoleOnTime [clock seconds]
            }
        } elseif {$state == "condition"} {
            APSMpStep "Checking state of power supplies" -desc "(Condition) Check state of power supplies"
            if {[catch {APScavget -list=[join $activeSelectedConverters ,] -list=:StatusCALC -pend=30 -label} results]} {
                APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
            }
            set nameAndValue ""
            foreach "name value" $results {
                if {($value != 0) && ($value != 1)} {
                    if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: $name is not on"] == 1} {
                        APSMpReturn error "APSMpSRUpDownDoubleSector: $name is not on"
                    }
                }
                if {$value == 1} {
                    if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: $name is in alarm mode"] == 1} {
                        APSMpReturn error "APSMpSRUpDownDoubleSector: $name is in alarm mode"
                    }
                }
            }
        }
        if {$apsCalibrate} {
            APSMpStep "Launching APSCalibrateDCPS" -desc "(On or Condition) Lanuch APSCalibrateDCPS"
            catch {exec APSCalibrateDCPS -system sr &} DCPSPID
            global execMode
            set oldMode $execMode
            set execMode Manual
            APSSetVarAndUpdate status "Press continue when done calibrating..."
            APSMpStep "Continue when ready" -desc "Continue when done calibrating"
            set execMode $oldMode
            catch {exec kill $DCPSPID}
        }

        #Disabling BControl
        #        APSMpStep "Disabling BControl" -desc "(On or Condition) Disable BControl"
        #        if {([string compare $apsNetworkDomain "accel.ntw0rk"] == 0) || ([string compare $apsNetworkDomain "aps4.anl.gov"] == 0)} {
        #            if {[catch {APSMpBoosterPSStartKillBControl -action1 kill} results]} {
        #                if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: Problem occurred while disabling BControl \n$results"] == 1} {
        #                    APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
        #                }
        #            }
        #        }
        
        if {$configure} {
            APSMpStep "Aborting DC Orbit Controllaws" -desc "(On or Condition) Abort DC Orbit Controllaws"
            if {[catch {exec cavput -list=DP:S:OrbitControlLawYSDDS.CMND=0,DP:S:OrbitControlLawYSDDS.CMND=0} results]} {
                if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: Problem occurred while aborting DC Orbit Controllaws \n$results"] == 1} {
                    APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
                }                
            }
            
            set standardizeFile /tmp/[APSTmpString]
            APSAddToTempFileList $standardizeFile
            if {[catch {exec sddsconvert \
                          /home/helios/oagData/sr/magnetConditioning/template.std \
                          -pipe=out -delete=column,ValueString \
                          | sddsxref -pipe -noWarning \
                          /home/helios/oagData/SCR/snapshots/SR/[file tail $ShortFilename] \
                          -match=PVName=ControlName -take=ValueString \
                          | sddsconvert -pipe -dele=col,Finish \
                          | sddsprocess -pipe \
                          "-scan=column,Finish,ValueString,%le" \
                          "-redefine=column,To,To Finish > ? To : Finish $ " \
                          | sddsconvert -pipe=in $standardizeFile \
                          -delete=column,ValueString} results]} {
                APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
            }
        }
        #If Dipole configuration option is chosen then condition the Dipole using 
        #the dipole.std configuration file.
        if {$conditionBM} {
            #Set all corresponding MT power supplies to zero.
            APSMpStep "Set non-XAXSTF MT family power supplies to 0" -desc "(On or Condition) Set non-XAXSTF MT power supplies to 0"
            if {[catch {APSSrDcpsAction \
                          -family MT \
                          -action set \
                          -value 0 \
                          -sectors $nonXAXSTFsectors \
                          -pause 10 \
                          -configFile NULL} results]} {
                if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: Problem occurred while setting MT power supplies to 0 \n$results"] == 1} {
                    APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
                }
            }
            
            #Wait for corresponding current readbacks to reach zero.
            #Check status every few seconds until completed or timed out.
            APSMpStep "Waiting for MT power supplies to reach 0" -desc "(On or Condition) Wait for MT power supplies to reach 0"
            if {[catch {WaitForTaskToFinish \
                          -cavget "APScavget -list=[join $MTcavgetOptions ,] -list=:CurrentAI -pend=30 -label" \
                          -waitLimit 3 \
                          -updateInterval 10 \
                          -checkExpression {($value < -.2) || ($value > .2)}} results]} {
                if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: Problem occurred while waiting for MT power supplies to reach 0 \n$results"] == 1} {
                    APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
                }
            }
            APSMpStep "Turning XAXSTF MT family power supplies off" -desc "(Off) Turn off XAXSTF MT power supplies"
            if {[catch {APSSrDcpsAction \
                          -family MT \
                          -action off \
                          -value 0 \
                          -sectors $unusedMTList \
                          -pause 10 \
                          -configFile NULL} results]} {
                if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: Problem occurred while turning XAXSTF MT power supplies off \n$results"] == 1} {
                    APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
                }
            }
                
            if {$configure} {
                APSMpStep "Configuring S:BM Dipole" -desc "(On or Condition) Configure S:BM dipole"
                if {([string compare $apsNetworkDomain "accel.ntw0rk"] == 0) || ([string compare $apsNetworkDomain "aps4.anl.gov"] == 0)} {
                    if {[catch {APSSrDcpsAction \
                                  -family BM \
                                  -action configure \
                                  -value 0 \
                                  -sectors $sectors \
                                  -pause 10 \
                                  -configFile $standardizeFile} results]} {
                        if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: Problem occurred while configuring BM for conditioning \n$results"] == 1} {
                            APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
                        }
                    }
                }
            }
            APSMpStep "Setting cycles for S:BM Dipole" -desc "(On or Condition) Set cycles for S:BM Dipole"
            if {[catch {APSSrDcpsAction \
                          -family BM \
                          -action setcycles \
                          -value $cycles \
                          -sectors $sectors \
                          -pause 10 \
                          -configFile NULL} results]} {
                if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: Problem occurred while setting conditioning cycles for BM \n$results"] == 1} {
                    APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
                }
            }
            APSWaitWithUpdate -waitSeconds 1
            
            if {$state == "on"} {
                set ws [expr [clock seconds] - $DipoleOnTime]
                if {$ws > 0} {
                    APSMpStep "Waiting for S:BM Dipole to stabilize" -desc "(On or Condition) Waiting for S:BM Dipole to stabilize"
                    APSWaitWithUpdate -waitSeconds $ws
                }
            }
            
            APSMpStep "Conditioning S:BM Dipole" -desc "(On or Condition) Condition S:BM dipole"
            if {[catch {APSSrDcpsAction \
                          -family BM \
                          -action $condition \
                          -value 0 \
                          -sectors $sectors \
                          -pause 10 \
                          -configFile NULL} results]} {
                if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: Problem occurred while conditioning BM \n$results"] == 1} {
                    APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
                }
            }
        }
        
        if {${conditionC:BM}} {
            if {[llength $CUDipoleList]} {
                if {$configure} {
                    APSMpStep "Configuring Canted Undulator Dipoles" -desc "(On or Condition) Configure Canted Undulator Dipoles"
                    if {([string compare $apsNetworkDomain "accel.ntw0rk"] == 0) || ([string compare $apsNetworkDomain "aps4.anl.gov"] == 0)} {
                        if {[catch {APSSrDcpsAction \
                                      -family C:BM \
                                      -action configure \
                                      -value 0 \
                                      -sectors $CUDipoleList \
                                      -pause 10 \
                                      -configFile /home/helios/oagData/sr/magnetConditioning/template.std} results]} {
                            if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: Problem occurred while configuring canted undulator dipoles for conditioning \n$results"] == 1} {
                                APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
                            }
                        }
                    }
                }
                
                APSMpStep "Setting cycles for the Canted Undulator Dipoles" -desc "(On or Condition) Set cycles for the Canted Undulator Dipoles"
                if {[catch {APSSrDcpsAction \
                              -family C:BM \
                              -action setcycles \
                              -value $cycles \
                              -sectors $CUDipoleList \
                              -pause 10 \
                              -configFile NULL} results]} {
                    if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: Problem occurred while setting conditioning cycles for canted undulator dipoles \n$results"] == 1} {
                        APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
                    }
                }
                
                APSWaitWithUpdate -waitSeconds 1
                APSMpStep "Conditioning the Canted Undulator Dipoles" -desc "(On or Condition) Condition Canted Undulator Dipoles"
                if {[catch {APSSrDcpsAction \
                              -family C:BM \
                              -action $condition \
                              -value 0 \
                              -sectors $CUDipoleList \
                              -pause 10 \
                              -configFile NULL} results]} {
                    if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: Problem occurred while conditioning canted undulator dipoles \n$results"] == 1} {
                        APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
                    }
                }
            }
        }
        
        

        #If Standardize configuration option is chosen then condition the QUADs and
        #the SEXTs using the /home/helios/oagData/sr/magnetConditioning/template.std configuration file.
        foreach standardize "standardizeQ standardizeS" family "Q S" {
            if {([set $standardize]) && ([set $family])} {
                if {$configure} {
                    if {([string compare $apsNetworkDomain "accel.ntw0rk"] == 0) || ([string compare $apsNetworkDomain "aps4.anl.gov"] == 0)} {
                        APSMpStep "Configuring $family power supplies" -desc "(On or Condition) Configure Q and S power supplies"
                        if {[catch {APSSrDcpsAction \
                                        -family $family \
                                        -action configure \
                                        -value 0 \
                                        -sectors $sectors \
                                        -pause 10 \
                                        -configFile $standardizeFile} results]} {
                            if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: Problem occurred while configuring $family power supplies for conditioning \n$results"] == 1} {
                                APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
                            }
                        }
                    }
                }
                APSMpStep "Setting cycles for $family power supplies" -desc "(On or Condition) Set cycles for Q and S power supplies"
                if {[catch {APSSrDcpsAction \
                                -family $family \
                                -action setcycles \
                                -value $cycles \
                                -sectors $sectors \
                                -pause 10 \
                                -configFile NULL} results]} {
                    if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: Problem occurred while setting conditioning cycles for $family power supplies \n$results"] == 1} {
                        APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
                    }
                }
                APSWaitWithUpdate -waitSeconds 1
                APSMpStep "Conditioning $family power supplies" -desc "(On or Condition) Condition Q and S power supplies"
                if {[catch {APSSrDcpsAction \
                                -family $family \
                                -action $condition \
                                -value 0 \
                                -sectors $sectors \
                                -pause 10 \
                                -configFile NULL} results]} {
                    if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: Problem occurred while conditioning $family power supplies \n$results"] == 1} {
                        APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
                    }
                }
            }
        }

        if {$conditionQS} {            
            if {$configure} {
                if {([string compare $apsNetworkDomain "accel.ntw0rk"] == 0) || ([string compare $apsNetworkDomain "aps4.anl.gov"] == 0)} {
                    APSMpStep "Configuring QS power supplies" -desc "(On or Condition) Configure QS power supplies"
                    if {[catch {APSSrDcpsAction \
                                    -family QS \
                                    -action configure \
                                    -value 0 \
                                    -sectors $sectors \
                                    -pause 10 \
                                    -configFile "/home/helios/oagData/sr/magnetConditioning/SQS_degauss.sdds"} results]} {
                        if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: Problem occurred while configuring QS power supplies for conditioning \n$results"] == 1} {
                            APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
                        }
                    }
                }
            }

            APSMpStep "Degauss QS power supplies" -desc "(On or Condition) Degauss QS power supplies"
            if {[catch {APSSrDcpsAction \
                            -family QS \
                            -action $condition \
                            -value 0 \
                            -sectors $sectors \
                            -pause 10 \
                            -configFile NULL} results]} {
                if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: Problem occurred while degaussing the QS power supplies \n$results"] == 1} {
                    APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
                }
            }
        }

        if {$configure} {
            if {([string compare $apsNetworkDomain "accel.ntw0rk"] == 0) || ([string compare $apsNetworkDomain "aps4.anl.gov"] == 0)} {
                APSMpStep "Setting correctors to Operation mode and to Scalar mode" \
                  -desc "Setting H,V power supplies to Operation mode and Scalar mode"
                if {[catch {APSSetCorrMode -plane both -sourceMode Operation -corrMode scalar} results]} {
                    if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: Problem occurred while setting correctors into operations and scalar mode \n$results"] == 1} {
                        APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
                    }
                }
                # Use checkStatus = 0 because we don't care whether the PS is on or not.
                # During the pem, some PS's will be turned off.
                if {[catch {APSSetRTFBCorrectorMode -all 1 -mode Operation -checkStatus 0 } results]} {
                    if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: Problem occurred while setting RTFB correctors into operations and scalar mode \n$results"] == 1} {
                        APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
                    }
                }
                # allow several seconds for the changes to take place.
                APSWaitWithUpdate -waitSeconds 4
            }
        }

        #If Degauss configuration option is chosen then condition the HCORRs using
        #the S_degauss.std configuration file.
        if {$degauss} {
            #Set all corresponding vertical power supplies to zero.
            if {([string compare $apsNetworkDomain "accel.ntw0rk"] == 0) || ([string compare $apsNetworkDomain "aps4.anl.gov"] == 0)} {
                if {[llength [set activeSelectedHConverters]]} {
                    
                    APSMpStep "Set V family power supplies to 0" -desc "(On or Condition) Set V power supplies to 0"
                    if {[catch {APScavput -list=SFB:S -list=[join $sectors ,] \
                                  "-list=A:,B:" \
                                  -list=V \
                                  -list=1,2,3,4 \
                                  -list=:CurrentAO=0 \
                                  -pend=30} results]} {
                        if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: Problem occurred while setting V power supplies to 0 \n$results"] == 1} {
                            APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
                        }
                    }
                    if [llength $QS4sectorList] {
                        APSMpStep "Set QS4 family power supplies to 0" -desc "(On or Condition) Set QS4 power supplies to 0"
                        if {[catch {APScavput -list=S -list=[join $QS4sectorList ,] \
                                      -list=A:QS4:CurrentAO=0  -pend=30} results]} {
                            if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: Problem occurred while setting QS4 power supplies to 0 \n$results"] == 1} {
                                APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
                            }
                        }
                    }
                }
                if {[llength $CUCorrList]} {
                    APSMpStep "Set Canted Undulator V family power supplies to 0" -desc "(On or Condition) Set Canted Undulator V power supplies to 0"
                    if {[catch {APScavput -list=SFB:S -list=[join $CUCorrList ,] \
                                  -list=C:V1:CurrentAO=0 \
                                  -pend=30} results]} {
                        if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: Problem occurred while setting canted undulator V power supplies to 0 \n$results"] == 1} {
                            APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
                        }
                    }
                }
            }
            
            if {[llength [set activeSelectedVConverters]]} {
                #Wait for corresponding current readbacks to reach zero.
                #Check status every few seconds until completed or timed out.
                APSMpStep "Waiting for vertical power supplies to reach 0" -desc "(On or Condition) Wait for V power supplies to reach 0"
                if {[catch {WaitForTaskToFinish \
                              -cavget "APScavget -list=[join $activeSelectedVConverters ,] -list=:CurrentAI -pend=30 -label" \
                              -waitLimit 3 \
                              -updateInterval 10 \
                              -checkExpression {($value < -3.0) || ($value > 1.5)}} results]} {
                    if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: Problem occurred while waiting for V power supplies to reach 0 \n$results"] == 1} {
                        APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
                    }
                }
            }
            if {[llength [set activeSelectedC:VConverters]]} {
                #Wait for corresponding current readbacks to reach zero.
                #Check status every few seconds until completed or timed out.
                APSMpStep "Waiting for vertical canted undulator power supplies to reach 0" -desc "(On or Condition) Wait for V canted undulator power supplies to reach 0"
                if {[catch {WaitForTaskToFinish \
                              -cavget "APScavget -list=[join ${activeSelectedC:VConverters} ,] -list=:CurrentAI -pend=30 -label" \
                              -waitLimit 3 \
                              -updateInterval 10 \
                              -checkExpression {($value < -3.0) || ($value > 1.5)}} results]} {
                    if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: Problem occurred while waiting for V canted undulator power supplies to reach 0 \n$results"] == 1} {
                        APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
                    }
                }
            }
            
            if {$configure} {
                if {([string compare $apsNetworkDomain "accel.ntw0rk"] == 0) || ([string compare $apsNetworkDomain "aps4.anl.gov"] == 0)} {
                    APSMpStep "Configuring H power supplies" -desc "(On or Condition) Configure H power supplies"
                    if {[catch {APSSrDcpsAction \
                                    -family H \
                                    -action configure \
                                    -value 0 \
                                    -sectors $sectors \
                                    -pause 10 \
                                    -configFile "/home/helios/oagData/sr/magnetConditioning/S_degauss.sdds"} results]} {
                        if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: Problem occurred while configuring H power supplies for conditioning \n$results"] == 1} {
                            APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
                        }
                    }
                    if {[llength $CUCorrList]} {
                        APSMpStep "Configuring Canted Undulator H power supplies" -desc "(On or Condition) Configure Canted Undulator H power supplies"
                        if {[catch {APSSrDcpsAction \
                                        -family C:H \
                                        -action configure \
                                        -value 0 \
                                        -sectors $CUCorrList \
                                        -pause 10 \
                                        -configFile "/home/helios/oagData/sr/magnetConditioning/S_degauss.sdds"} results]} {
                            if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: Problem occurred while configuring canted undulator H power supplies for conditioning \n$results"] == 1} {
                                APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
                            }
                        }
                    }
                }
            }

            APSMpStep "Degaussing H power supplies" -desc "(On or Condition) Degauss H power supplies"
            if {[catch {APSSrDcpsAction \
                            -family H \
                            -action $condition \
                            -value 0 \
                            -sectors $sectors \
                            -pause 10 \
                            -configFile NULL} results]} {
                if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: Problem occurred while degaussing the H power supplies \n$results"] == 1} {
                    APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
                }
            }
            if {[llength $CUCorrList]} {
                APSMpStep "Degaussing Canted Undulator H power supplies" -desc "(On or Condition) Degauss Canted Undulator H power supplies"
                if {[catch {APSSrDcpsAction \
                                -family C:H \
                                -action $condition \
                                -value 0 \
                                -sectors $CUCorrList \
                                -pause 10 \
                                -configFile NULL} results]} {
                    if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: Problem occurred while degaussing the canted undulator H power supplies \n$results"] == 1} {
                        APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
                    }
                }
            }
        }

        APSMpStep "Checking state of power supplies" -desc "(On or Condition) Check state of power supplies"
        if {[catch {APScavget -list=[join $activeSelectedConverters ,] -list=:StatusCALC -pend=30 -label} results]} {
            APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
        }
        set nameAndValue ""
        foreach "name value" $results {
            if {($value != 0) && ($value != 1)} {
                if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: $name is not on"] == 1} {
                    APSMpReturn error "APSMpSRUpDownDoubleSector: $name is not on"
                }
            }
#            if {$value == 1} {
#                if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: $name is in alarm mode"] == 1} {
#                    APSMpReturn error "APSMpSRUpDownDoubleSector: $name is in alarm mode"
#                }
#            }
        }

        if {$conditionBM} {
            APSMpStep "Checking to see if the dipole is conditioning" -desc "(On or Condition) Check to see if the dipole is conditioning"
            if {[catch {WaitForTaskToFinish \
                            -cavget "APScavget -list=S:BM:StandardizeSUB.PROC -pend=30 -label" \
                            -waitLimit 1 \
                            -updateInterval 1 -message 0 \
                            -checkExpression {($value < -.5) || ($value > .5)}} results]} {
            } else {
		if {$forcecondition} {
		    if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: Problem occurred while checking if the dipole is conditioning \n$results"] == 1} {
			APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
		    }
		}
            }
        }
        if {${conditionC:BM}} {
            if {[llength $CUDipoleList]} {
                APSMpStep "Checking to see if canted undulator dipoles are conditioning" -desc "(On or Condition) Check to see if the canted undulator dipoles are conditioning"
                if {[catch {WaitForTaskToFinish \
                                -cavget "APScavget -list=[join [set {activeSelectedC:BMConverters}] ,] -list=:StandardizeSUB.PROC -pend=30 -label" \
                                -waitLimit 1 \
                                -updateInterval 1 -message 0 \
                                -checkExpression {($value < -.5) || ($value > .5)}} results]} {
                } else {
		    if {$forcecondition} {
			if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: Problem occurred while checking if the canted undulator dipoles are conditioning \n$results"] == 1} {
			    APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
			}
		    }
                }
            }
        }


        foreach standardize "standardizeQ standardizeS conditionQS" family "Q S QS" {
            if {([set $standardize]) && ([set $family])} {
                APSMpStep "Checking to see if $family is conditioning" -desc "(On or Condition) Check to see if Q, S and QS are conditioning"
                if {[catch {WaitForTaskToFinish \
                                -cavget "APScavget -list=[join [set activeSelected${family}Converters] ,] -list=:StandardizeSUB.PROC -pend=30 -label" \
                                -waitLimit 1 \
                                -updateInterval 1 -message 0 \
                                -checkExpression {($value < -.5) || ($value > .5)}} results]} {
                } else {
		    if {$forcecondition} {
			if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: Problem occurred while checking if the $family power supplies are conditioning \n$results"] == 1} {
			    APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
			}
		    }
		}
            }
        }
        if {$degauss} {
            APSMpStep "Checking to see if HCorrs are conditioning" -desc "(On or Condition) Check to see if the H correctors are conditioning"
            if {[catch {WaitForTaskToFinish \
                            -cavget "APScavget -list=[join $activeSelectedHConverters ,] -list=:StandardizeSUB.PROC -pend=30 -label" \
                            -waitLimit 1 \
                            -updateInterval 1 -message 0 \
                            -checkExpression {($value < -.5) || ($value > .5)}} results]} {
            } else {
		if {$forcecondition} {
		    if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: Problem occurred while checking if the H power supplies are conditioning \n$results"] == 1} {
			APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
		    }
		}
            }
        }
        if {$conditionBM} {
            #split dipole wait into two so that it can check that it has not tripped.
            APSMpStep "Waiting for dipole conditioning to finish" -desc "(On or Condition) Wait for dipole conditioning to finish"
            if {[catch {WaitForTaskToFinish \
                          -cavget "APScavget -list=S:BM:StandardizeSUB.PROC -pend=30 -label" \
                          -waitLimit 6 \
                          -updateInterval 10 \
                          -message 0 \
                          -checkExpression {($value < -.01) || ($value > .01)}} results]} {
            }
            APSMpStep "Checking state of power supplies" -desc "(On or Condition) Check state of power supplies"
            if {[catch {APScavget -list=[join $activeSelectedConverters ,] -list=:StatusCALC -pend=30 -label} results]} {
                APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
            }
            set nameAndValue ""
            foreach "name value" $results {
                if {($value != 0) && ($value != 1)} {
                    if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: $name is not on"] == 1} {
                        APSMpReturn error "APSMpSRUpDownDoubleSector: $name is not on"
                    }
                }
            }
            APSMpStep "Waiting for dipole conditioning to finish" -desc "(On or Condition) Wait for dipole conditioning to finish"
            if {[catch {WaitForTaskToFinish \
                            -cavget "APScavget -list=S:BM:StandardizeSUB.PROC -pend=30 -label" \
                            -waitLimit [expr int($cycles2 * 24)] \
                            -updateInterval 10 \
                            -checkExpression {($value < -.01) || ($value > .01)}} results]} {
		if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: Problem occurred while waiting for the dipole to finish conditioning \n$results"] == 1} {
		    APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
		}
            }
            APSMpStep "Checking MT power supplies to ensure they are set to 0" -desc "(On or Condition) Check MT power supplies to ensure they are set to 0"
            if {[catch {WaitForTaskToFinish \
                            -cavget "APScavget -list=[join $MTcavgetOptions ,] -list=:CurrentAI -pend=30 -label" \
                            -waitLimit 1 \
                            -updateInterval 1 -message 0 \
                            -checkExpression {($value < -.2) || ($value > .2)}} results]} {
                if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: Problem occurred while checking if the MT power supplies are at 0 amps \n$results"] == 1} {
                    APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
                }
            }
        }
        if {${conditionC:BM}} {
            if {[llength $CUDipoleList]} {
                APSMpStep "Waiting for canted undulator dipole conditioning to finish" -desc "(On or Condition) Wait for canted undulator dipole conditioning to finish"
                if {[catch {WaitForTaskToFinish \
                                -cavget "APScavget -list=[join [set {activeSelectedC:BMConverters}] ,] -list=:StandardizeSUB.PROC -pend=30 -label" \
                                -waitLimit [expr int($cycles2 * 24 + 6)] \
                                -updateInterval 10 \
                                -checkExpression {($value < -.01) || ($value > .01)}} results]} {
                    if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: Problem occurred while waiting for the canted dipole power supplies to finish conditioning \n$results"] == 1} {
                        APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
                    }
                }
            }
        }
        foreach standardize "standardizeQ standardizeS conditionQS" family "Q S QS" {
            if {([set $standardize]) && ([set $family])} {
                APSMpStep "Waiting for $family conditioning to finish" -desc "(On or Condition) Wait for Q, S and QS conditioning to finish"
                if {[catch {WaitForTaskToFinish \
                                -cavget "APScavget -list=[join [set activeSelected${family}Converters] ,] -list=:StandardizeSUB.PROC -pend=30 -label" \
                                -waitLimit [expr int($cycles2 * 24 + 6)] \
                                -updateInterval 10 \
                                -checkExpression {($value < -.01) || ($value > .01)}} results]} {
                    if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: Problem occurred while waiting for the $family power supplies to finish conditioning \n$results"] == 1} {
                        APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
                    }
                }
            }
        }

        if {($conditionBM) && ($standardizeQ)} {
            if {[catch {APSSrDcpsAction \
                          -family Q \
                          -action setcycles \
                          -value 0 \
                          -sectors $sectors \
                          -pause 10 \
                          -configFile NULL} results]} {
                if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: Problem occurred while setting conditioning cycles for Q power supplies \n$results"] == 1} {
                    APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
                }
            }
            APSWaitWithUpdate -waitSeconds 1
            APSMpStep "Conditioning Q power supplies 0 cycles after BM finished conditioning" -desc "(On or Condition) Condition Q power supplies 0 cycles after BM finished conditioning"
            if {[catch {APSSrDcpsAction \
                          -family Q \
                          -action forcecondition \
                          -value 0 \
                          -sectors $sectors \
                          -pause 10 \
                          -configFile NULL} results]} {
                if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: Problem occurred while conditioning Q power supplies \n$results"] == 1} {
                    APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
                }
            }
            APSWaitWithUpdate -waitSeconds 1
            APSMpStep "Waiting for Q conditioning to finish" -desc "(On or Condition) Wait for Q conditioning to finish"
            if {[catch {WaitForTaskToFinish \
                          -cavget "APScavget -list=[join [set activeSelectedQConverters] ,] -list=:StandardizeSUB.PROC -pend=30 -label" \
                          -waitLimit [expr int(1 * 24 + 6)] \
                          -updateInterval 10 \
                          -checkExpression {($value < -.01) || ($value > .01)}} results]} {
                if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: Problem occurred while waiting for the Q power supplies to finish conditioning \n$results"] == 1} {
                    APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
                }
            }
        }


        if {$degauss} {
            APSMpStep "Waiting for HCorrs conditioning to finish" -desc "(On or Condition) Wait for H corrector conditioning to finish"
            # waitLimit may need to be changed in the future. 6 stands for the number of cycles
            if {[catch {WaitForTaskToFinish \
                            -cavget "APScavget -list=[join $activeSelectedHConverters ,] -list=:StandardizeSUB.PROC -pend=30 -label" \
                            -waitLimit [expr int(6 * 9 + 12)] \
                            -updateInterval 10 \
                            -checkExpression {($value < -.01) || ($value > .01)}} results]} {
                if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: Problem occurred while waiting for the H power supplies to finish conditioning \n$results"] == 1} {
                    APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
                }
            }
        }

        APSMpStep "Checking state of power supplies" -desc "(On or Condition) Check state of power supplies"
        if {[catch {APScavget -list=[join $activeSelectedConverters ,] -list=:StatusCALC -pend=30 -label} results]} {
            APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
        }
        set nameAndValue ""
        foreach "name value" $results {
            if {($value != 0) && ($value != 1)} {
                if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: $name is not on"] == 1} {
                    APSMpReturn error "APSMpSRUpDownDoubleSector: $name is not on"
                }
            }
#            if {$value == 1} {
#                if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: $name is in alarm mode"] == 1} {
#                    APSMpReturn error "APSMpSRUpDownDoubleSector: $name is in alarm mode"
#                }
#            }
        }


        #Setting corresponding power supplies to values stored in the SCR File
        set nameAndValue ""
        APSMpStep "Setting power supplies to values stored in the SCR File" -desc "(On or Condition) Set power supplies to values stored in the SCR File"
        foreach name $activeSelectedConverters value $activeSelectedConverterValues {
            if {![regexp {S.*[HV].*} $name] && ![regexp {S.*QS4.*} $name]} {
                lappend nameAndValue ${name}:CurrentAO=${value}
            }
        }
        if {[llength $nameAndValue]} {
            if {[catch {APScavput -list=[join $nameAndValue ,] -blunderAhead -pend=30} results]} {
                if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: Problem occurred while setting non-corrector PVs to final current value \n$results"] == 1} {
                    APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
                }
            }
            APSWaitWithUpdate -waitSeconds 5
        }
        
        APSMpStep "Ramp HV correctors to final current value..."
        if {[catch {APSRampPVs -PVList "$activeSelectedHConverters $activeSelectedVConverters" \
                        -prefix "SFB:" -suffix ":CurrentAO" \
                        -valueList "$activeSelectedHValues $activeSelectedVValues" \
                        -steps 20 -pause 0.5} results]} {
            if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: Problem occurred while ramping corrector PVs to final current value \n$results"] == 1} {
                APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
            }
        }
        if [llength $activeSelectedQS4Converters] {
            #ramp QS4 too
            APSMpStep "Ramp QS4 correctors to final current value..."
            if {[catch {APSRampPVs -PVList "$activeSelectedQS4Converters" \
                          -prefix "" -suffix ":CurrentAO" \
                          -valueList "$activeSelectedQS4Values" \
                          -steps 20 -pause 0.5} results]} {
                if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: Problem occurred while ramping QS4 PVs to final current value \n$results"] == 1} {
                    APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
                }
            }
        }
        APSMpStep "Doing 100Hz Reset Procedure" -desc "(Condition) Do 100Hz Reset Procedure"
        if {([string compare $apsNetworkDomain "accel.ntw0rk"] == 0) || ([string compare $apsNetworkDomain "aps4.anl.gov"] == 0)} {
            set activeSelectedConvertersNoBM "$activeSelectedQConverters $activeSelectedQSConverters $activeSelectedSConverters $activeSelectedHConverters $activeSelectedVConverters $activeSelectedMTConverters" 
	    if {[llength $activeSelectedConvertersNoBM] > 0}  {
		set pass 0
		while {$pass != 1} {
		    if {[catch {exec cavget -printErrors -list=[join $activeSelectedConvertersNoBM ,] -list=:CurrentAI.ASLO -pend=60} results]} {
			if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: Problem occurred while doing 100Hz reset procedure \n$results" -strings {"Retry" "Abort Procedure"}] == 1} {
			    APSMpReturn error "APSMpSRUpDownDoubleSector: Problem occurred while doing 100Hz reset procedure \n$results"
			} else {
			    continue
			}
		    }
                    
		    set also ""
		    foreach ps $activeSelectedConvertersNoBM value $results {
			append aslo "$ps=$value "
		    }
		    if {[catch {exec cavget -printErrors -list=[join $activeSelectedConvertersNoBM ,] -list=:CurrentAI.ESLO,:MeanAI.ESLO,:CurrentAI.AOFF -pend=60} results]} {
			if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: Problem occurred while doing 100Hz reset procedure \n$results" -strings {"Retry" "Abort Procedure"}] == 1} {
			    APSMpReturn error "APSMpSRUpDownDoubleSector: Problem occurred while doing 100Hz reset procedure \n$results"
			} else {
			    continue
			}
		    }
		    
		    set aoff ""
		    foreach ps $activeSelectedConvertersNoBM "value1 value2 value3" $results {
			append aoff "$ps=[expr {$value1 / $value2 * $value3 * 1.0}] "
		    }
		    set pass 1
		}
		set pass 0
		while {$pass != 1} {
		    if {[catch {
			exec cavput -list=[join $aoff ,] -list=:MeanAI.AOFF -pend=60
			exec cavput -list=[join $aslo ,] -list=:MeanAI.ASLO,:MADAI.ASLO,:PeakAbsDevAI.ASLO -pend=60
			exec cavput -list=[join $activeSelectedConvertersNoBM ,] -list=:FilterCoeffAO=9,:GThreshAO=.25,:GLookawayAO=200,:SPLookawayAO=500 -pend=60
			if {[llength $activeSelectedQConverters]} {
			    exec cavput -list=[join $activeSelectedQConverters ,] -list=:ADCTypeMBBO=0 -pend=60
			}
			if {[llength $activeSelectedQSConverters]} {
			    exec cavput -list=[join $activeSelectedQSConverters ,] -list=:ADCTypeMBBO=2 -pend=60
			}
                        if {[llength $activeSelectedQS4Converters]} {
			    exec cavput -list=[join $activeSelectedQS4Converters ,] -list=:ADCTypeMBBO=1 -pend=60
			}
			if {[llength $activeSelectedHConverters]} {
			    exec cavput -list=[join $activeSelectedHConverters ,] -list=:ADCTypeMBBO=1 -pend=60
			}
			if {[llength $activeSelectedVConverters]} {
			    exec cavput -list=[join $activeSelectedVConverters ,] -list=:ADCTypeMBBO=1 -pend=60
			}
			if {[llength $activeSelectedMTConverters]} {
			    exec cavput -list=[join $activeSelectedMTConverters ,] -list=:ADCTypeMBBO=2 -pend=60
			}
			if {[llength $activeSelectedSConverters]} {
			    exec cavput -list=[join $activeSelectedSConverters ,] -list=:ADCTypeMBBO=0 -pend=60
			}
			exec cavput -list=[join $activeSelectedConvertersNoBM ,] -list=:ResetStatsBO=1,:NoiseMonEnaBO=1 -pend=60
		    } results]} {
			set answer [APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: Problem occurred while doing 100Hz reset procedure \n$results" -strings {"Retry" "Continue without Retrying" "Abort Procedure"}]
			
			if {$answer == 2} {
			    APSMpReturn error "APSMpSRUpDownDoubleSector: Problem occurred while doing 100Hz reset procedure \n$results"
			} elseif {$answer == 1} {
			    continue
			}
		    }
		    set pass 1
                    APSMpStep "pass inside:  $pass"
		}
	    }
        }

#        APSMpStep "Starting BControl" -desc "(On or Condition) Start BControl"
#        if {([string compare $apsNetworkDomain "accel.ntw0rk"] == 0) || ([string compare $apsNetworkDomain "aps4.anl.gov"] == 0)} {
#            if {[catch {APSMpBoosterPSStartKillBControl -BM 1 -QF 1 -QD 1 -SF 1 -SD 1 -slope 1 -zero 1 -action1 start -action2 enable} results]} {
#                if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: Problem occurred while starting BControl \n$results"] == 1} {
#                    APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
#                }
#            }
#        }
        APSMpStep "set S:BM:CalibrateBO to 1"
        if {[catch {APScavput -list=S:BM:CalibrateBO=1 -blunderAhead -pend=30} results]} {
            if {[APSPEMTKDialog .warningDialog -text "APSMpSRUpDownDoubleSector: Problem occurred while setting calibrating the dipole \n$results"] == 1} {
                APSMpReturn error "APSMpSRUpDownDoubleSector: $results"
            }
        }
    }
    APSMpStep "APSMpSRUpDownDoubleSector end."
    if {$degauss && $corrMode=="Vector"} {
        APSAlertBox .warning -errorMessage "Warning: the original corrector is vector mode! Use controllaw start button to return back to Vector mode for both plane so that the setpoints and offsets can be reinitialized."
    }
    #note that the corrector mode is set to scalar for conditioning,
    #need run controllaw start button to return back to vector mode
    APSMpReturn ok
}

proc WaitForTaskToFinish {args} {
    set cavget ""
    set checkExpression ""
    set waitLimit 3
    set updateInterval 10
    set message 1
    APSStrictParseArguments {cavget waitLimit updateInterval checkExpression message}
    if {![llength $cavget]} {
        return -code error "WaitForTaskToFinish: invalid cavget option"
    }
    if {![llength $checkExpression]} {
        return -code error "WaitForTaskToFinish: invalid checkExpression option"
    }
    set wait 1
    while {$wait} {
        set wait 0
        if {[incr waitLimit -1] < 0} {
            if {$message} {
                set waitLimit 1
                set answer [APSPEMTKDialog .warningDialog -text "[join $badList \n]\n\n1. Fix the problem manually then press Re-test Condition.\n\n2. Ignore the problem, i.e. do not re-test, and resume the PEM procedure. There is a risk involved as the problem may actually be important, and the PEM has no way of determining this.\n\n3. Press the Abort Procedure to which will cause the eventual exit of the PEM procedure." -strings {"Re-test Condition" "Continue without Re-testing" "Abort Procedure"}]
                
                #set answer [tk_dialog .warningDialog "Warning Message" "$pvName = $pvValue\n\nChoose one of the following:\n\nFix the problem manually then press Continue.\n\nPress Cancel to stop the procedure" warning -1 Continue Cancel "Continue at\nyour own risk"]
                case $answer {
                    0 { 
                    } 
                    1 {
                        return -code ok
                    }
                    2 {
                        return -code error "WaitForTaskToFinish: Timed out, $pvName = $pvValue"
                    } 
                }
            } else {
                return -code error "WaitForTaskToFinish: Timed out, $pvName = $pvValue"
            }
        }
        if {[catch {eval $cavget} results]} {
            return -code error "WaitForTaskToFinish: $results"
        }
        set badList ""
        foreach {name value} $results {
             if $checkExpression {
                if {$waitLimit == 0} {
                    set pvName $name
                    set pvValue $value
                    lappend badList $pvName=$value
                }
                set wait 1
                #break
            }
        }
        if {$wait} {
            APSWaitWithUpdate -waitSeconds $updateInterval
        }
    }
}

proc APSFindSCROperationFile {args} {
    set machine ""
    set preference ""
    set arrayName ""
    APSParseArguments {machine preference arrayName}
    if {![string length $preference]} {
        set preference UserBeamPreferred
    }
    global $arrayName
    set oldDir [pwd]
    set dir /home/helios/oagData/SCR/snapshots/$machine
    cd $dir
    if {![file exist $machine-$preference.gz]} {
        cd $oldDir
        return -code error "$machine-$preference.gz does not exist!"
    }
    set file [file readlink $machine-$preference.gz]
    if [catch {file readlink $file} file1] {
	#file is not a link, do nothing
    } else {
	set file $file1
    }
    set ${arrayName}(Description) [exec sdds2stream $file -par=SnapshotDescription]
    cd $oldDir
    return  [file tail $file]
}

proc APSRampPVs {args} {
    set PVList ""
    set valueList ""
    set steps 30
    set pause 1
    set prefix ""
    set suffix ""
    APSParseArguments {PVList valueList steps pause prefix suffix}

    if {[llength $PVList] != [llength $valueList]} {
        APSMpReturn error "APSRampPVs: mismatched length of list arguments."
    }
    if {$steps <=0 } {
        APSMpReturn error "APSRampPVs: invalid step value."
    }
    if {$pause <=0 } {
        APSMpReturn error "APSRampPVs: invalid pause value."
    }
    if {![llength $PVList]} return
    # get original values
    set name ""
    foreach PV $PVList {
        lappend name ${prefix}${PV}${suffix}
    }
    if {[catch {APScavget -list=[join $name ,] -pend=30} origValues]} {
        APSMpReturn error "APSRampPVs: $origValues"
    }
    for {set i 1} {$i <= $steps} {incr i} {
        set nameAndValue ""
        foreach PV $PVList target $valueList orig $origValues {
            set delta [expr 1.0 *($target - $orig)/ $steps]
            set value [expr 1.0 *$orig + $i * $delta]
            lappend nameAndValue ${prefix}${PV}${suffix}=$value
        }
        if {[catch {APScavput -list=[join $nameAndValue ,] -blunderAhead -pend=30} results]} {
            APSMpReturn error "APSRampPVs: $results"
        }
        if {$i < $steps} {
            after [expr int(1000 * $pause)]
        }
    }
}

proc APSMpSRDumpBeam {args} {
    if {[catch {APScavput -list=S:MPS:dumpBeamSQ=1 -pend=10} results]} {
        return -code error "Problem with setting S:MPS:dumpBeamSQ: $results"
    }
    APSWaitWithUpdate -waitSeconds 1
}

proc APSMpSRResetMPS {args} {
    # Commented out resetting AGC 
    # because there is a small but non-zero chance of conflict 
    # with rf group.
    if {0} {
        if {[catch {APScavput -list=SRF:AGC:FB_SoftEngage -list=1,2 -list=BO=1 -pend=30} results]} {
            return -code error "Problem with setting SRF:AGC:FB_SoftEngage1BO and SRF:AGC:FB_SoftEngage2BO: $results"
        }
    }
    
    if {[catch {exec resetMPS} results]} {
        return -code error "$results"
    }
}
