#
# $Log: not supported by cvs2svn $
# Revision 1.12  2009/12/03 22:52:24  shang
# added Par extraction septum PV to the PARSeptumList and added PARKickerSeptum option to SetPulsedMagnetEnables
# proc to disbale/enable the par kicker and septum at the same time.
#
# Revision 1.11  2006/09/08 16:30:27  soliday
# timingResets.tcl is not a program so I removed the exec oagwish and
# auto_path commands.
#
# Revision 1.10  2003/12/16 03:26:00  emery
# Added locations BoosterExtSpecial GunToBoosterExtSpecial where
# the booster extraction kicker is excluded from the list.
# This is used in conjunction with injecting into the SR
# with the booster extraction kicker "included" with the SR kickers.
#
# Revision 1.9  2002/05/30 15:50:24  borland
# Now correctly sets the gun for gun-to-booster extraction.
#
# Revision 1.8  2002/04/01 14:10:08  borland
# Added "GunTo..." in addition to "Gunto..." locations.
#
# Revision 1.7  2002/03/26 17:21:34  emery
# Replaced last instance of PARlist variable.
#
# Revision 1.6  2002/03/26 17:06:23  emery
# Changed location names from PARto... to Gunto....
#
# Revision 1.5  2002/03/26 16:59:47  emery
# Added error messages. Split PAR kicker and septum PV lists.
#
# Revision 1.4  2002/03/26 15:16:29  emery
# Added PAR kicker mode.
#
# Revision 1.3  1997/08/14 00:37:32  borland
# Reinstated ES2 to same function as all others, since this script wasn't involved
# in the problems after all.
#
# Revision 1.2  1997/08/13 16:37:42  borland
# Modified to not toggle or turn off B:ES2, because of problems with
# that supply.
#
# Revision 1.1  1996/04/07  22:57:54  borland
# Moved here from oagapp directory.
#
# Revision 1.2  1996/03/18  23:17:38  borland
# Added exec command before cavputs.
#
# Revision 1.1  1996/03/18  23:13:48  borland
# First version.
#
#

proc TogglePulsedMagnetEnables {args} {
    set location All
    APSStrictParseArguments {location}
    if [catch {exec cavget -list=LI:iLeavePermitCC -printErrors -pend=10 } interleaving] {
        return -code error "Error reading LI:iLeavePermitCC: $interleaving"
    }
    set GunList "L1:RFG:KIK:chargeTimePD.GATE=<value>"
    set PARSeptumList "It:Par:PSPchargeBO.VAL=<value>,It:ParInjChargeDDG.GATE=<value>,It:ParExtChargeDDG.GATE=<value>"
    set PARKickerList "It:Ddg2chan4.GATE=<value>,It:ER1:er.OTL2=<value>"
    set BoosterInjList "It:Ddg3chan2.GATE=<value>,It:Ddg2chan6.GATE=<value>"
    set BoosterExtList "Mt:Ddg1chan4.GATE=<value>,Mt:Ddg1chan5.GATE=<value>,Mt:Ddg1chan0.GATE=<value>"
    set BoosterExtSpecialList "Mt:Ddg1chan4.GATE=<value>,Mt:Ddg1chan5.GATE=<value>"
    set changeAlpha 1
    set pvList ""
    switch $location {
        Gun {
            set pvList $GunList
        }
        PAR {
            set pvList $GunList,$PARKickerList,$PARSeptumList
        }
        PARKicker {
            set pvList $PARKickerList
            set changeAlpha 0
        }
        BoosterInj {
            set pvList $BoosterInjList
            set changeAlpha 0
        }
        GunToBoosterInj -
        GuntoBoosterInj {
            set pvList $GunList,$PARKickerList,$PARSeptumList,$BoosterInjList
        }
        BoosterExt {
            set pvList $BoosterExtList
            set changeAlpha 0
        }
        BoosterExtSpecial {
            set pvList $BoosterExtSpecialList
            set changeAlpha 0
        }
        GunToBoosterExtSpecial -
        GuntoBoosterExtSpecial {
            if $interleaving {
                set pvList $PARKickerList,$PARSeptumList,$BoosterInjList
            } else {
                set pvList $GunList,$PARKickerList,$PARSeptumList,$BoosterInjList,$BoosterExtSpecialList
            }
        }
        GunToBoosterExt -
        GuntoBoosterExt -
        All {
            set pvList $GunList,$PARKickerList,$PARSeptumList,$BoosterInjList,$BoosterExtList
        }
        default {
            return -code error -errorinfo "TogglePulsedMagnetEnables: Invalid location: $location" \
              "TogglePulsedMagnetEnables: Invalid location: $location"
        }
    }
    if {$interleaving && $changeAlpha} {
        #set RG2 alpha magnet to 140 to turn on gun kicker automatically
        if [catch {exec cavput -list=L1:RG2:LFA:CurrentAO=140 -pend=10} result] {
            return -code error -errorinfo "TogglePulsedMagnetEnables: Error setting RG2 alpha magnet to 140A: $result."
        }
    }
    if [string length $pvList] {
        if [catch {regsub -all "<value>" $pvList 0 pvList0} result] {
            return -code error -errorinfo "TogglePulsedMagnetEnables: Error substituting values regsub." \
              "TogglePulsedMagnetEnables: Error substituting values regsub."
        }
        if [catch {regsub -all "<value>" $pvList 1 pvList1} result] {
            return -code error -errorinfo "TogglePulsedMagnetEnables: Error substituting values regsub." \
              "TogglePulsedMagnetEnables: Error substituting values regsub."
        }
        if [catch {exec cavput -list=$pvList0 -pend=10} result] {
            return -code error -errorinfo "TogglePulsedMagnetEnables: Error trying to turn off timing enables." \
              "TogglePulsedMagnetEnables: Error trying to turn off timing enables."
        }
        after 1000
        if [catch {exec cavput -list=$pvList1 -pend=10} result] {
            return -code error -errorinfo "TogglePulsedMagnetEnables: Error trying to turn on timing enables." \
              "TogglePulsedMagnetEnables: Error trying to turn on timing enables."
        }
    }
    return -code ok
}

proc SetPulsedMagnetEnables {args} {
    set location All
    set state 0
    APSStrictParseArguments {location state}
    if [catch {exec cavget -list=LI:iLeavePermitCC -printErrors -pend=10 } interleaving] {
        return -code error "Error reading ACIS:LAL:INTRLV_MODE: $interleaving"
    }
    set GunList "L1:RFG:KIK:chargeTimePD.GATE=<value>"
    set PARSeptumList "It:Par:PSPchargeBO.VAL=<value>,It:ParInjChargeDDG.GATE=<value>,It:ParExtChargeDDG.GATE=<value>"
    set PARKickerList "It:Ddg2chan4.GATE=<value>,It:ER1:er.OTL2=<value>"
    set BoosterInjList "It:Ddg3chan2.GATE=<value>,It:Ddg2chan6.GATE=<value>"
    set BoosterExtList "Mt:Ddg1chan4.GATE=<value>,Mt:Ddg1chan5.GATE=<value>,Mt:Ddg1chan0.GATE=<value>"
    set BoosterExtSpecialList "Mt:Ddg1chan4.GATE=<value>,Mt:Ddg1chan5.GATE=<value>"
    set pvList ""
    set changeAlpha 1
    switch $location {
        Gun {
            set pvList $GunList
        }
        PAR {
            set pvList $GunList,$PARKickerList,$PARSeptumList
        }
        PARKicker {
            set pvList $PARKickerList
            set changeAlpha 0
        }
        PARKickerSeptum {
            set pvList $PARKickerList,$PARSeptumList
            set changeAlpha 0
        }
        BoosterInj {
            set pvList $BoosterInjList
            set changeAlpha 0
        }
        GunToBoosterInj -
        GuntoBoosterInj {
            set pvList $GunList,$PARKickerList,$PARSeptumList,$BoosterInjList
        }
        BoosterExt {
            set pvList $BoosterExtList
            set changeAlpha 0
        }
        BoosterExtSpecial {
            set pvList $BoosterExtSpecialList
            set changeAlpha 0
        }
        GunToBoosterExtSpecial -
        GuntoBoosterExtSpecial {
            set pvList $GunList,$PARKickerList,$PARSeptumList,$BoosterInjList,$BoosterExtSpecialList
        }
        GunToBoosterExt -
        GuntoBoosterExt -
        All {
            set pvList $GunList,$PARKickerList,$PARSeptumList,$BoosterInjList,$BoosterExtList
        }
        default {
            return -code error -errorinfo "SetPulsedMagnetEnables: Invalid location: $location" \
              "SetPulsedMagnetEnables: Invalid location: $location"
        }
    }
    if {$interleaving && $changeAlpha} {
        #set RG2 alpha magnet (141 -turn on RG gun kicker, 0 - turn off RG gun kicker)
        switch $state {
            0 {
                set value 0
            }
            1 {
                set value 140
            }
            default {
                return -code error  -errorinfo "SetPulsedMagnetEnables: Invalid state: $state" \
                  "SetPulsedMagnetEnables: Invalid state: $state"
            }
        }
        if [catch {exec cavput -list=L1:RG2:LFA:CurrentAO=$value -pend=10} result] {
            return -code error  -errorinfo "SetPulsedMagnetEnables: error setting RG2 alpha magent to $value: $result" \
              "SetPulsedMagnetEnables: error setting RG2 alpha magnet to $value: $result"
        }
    }
    if [string length $pvList] {
        regsub -all "<value>" $pvList $state pvList0
        if [catch {exec cavput -list=$pvList0 -pend=10} result] {
            return -code error -errorinfo "SetPulsedMagnetEnables: Error trying to change timing enables." \
              "SetPulsedMagnetEnables: Error trying to change timing enables."
        }
    }
    return -code ok
}

