#
# $Log: not supported by cvs2svn $
# Revision 1.23  2009/07/28 17:57:18  soliday
# Updated to play the audio files on both Solaris and Linux.
#
# Revision 1.22  2009/07/28 15:40:41  borland
# Added backoff beep mode.  Also added volume control.
#
# Revision 1.21  2009/07/23 21:57:38  borland
# Added -q option to play to avoid printouts to screen.
#
# Revision 1.20  2006/09/08 16:51:07  soliday
# Fixed APSAlertBoxHelp so that it has the proper backslashes.
#
# Revision 1.19  2006/05/16 20:08:27  shang
# Updated the APSAlertBox to use APSScrolledText if the error message has more than 30 lines or is longer than 2000 characters.
#
# Revision 1.18  2006/04/04 19:47:10  soliday
# Updated the APSAlertBox to use the APSScrolledText if the error message
# is longer than 2000 characters.
#
# Revision 1.17  2006/04/04 18:46:30  shang
# customized the alert tone which now is different for sr, linac, booster, par and default
# type applications and made it optional to announce the workstation name.
#
# Revision 1.16  2006/03/10 21:46:54  shang
# added widget argument to APSAlertBeep to stop the beep as soon as the widget disappears
# and modified APSAlertBox to pass widget argument when it calls APSAlertBeep.
#
# Revision 1.15  2006/01/18 15:43:38  borland
# If apsScriptCommand is not defined, doesn't attempt to use it.
#
# Revision 1.14  2005/10/20 22:23:36  shang
# fixed bug that occurred when dialog box is modal with beep feature and changed
# the background color of the title according to alert type.
#
# Revision 1.13  2005/09/26 16:24:32  shang
# added beep option to APSAlertBox
#
# Revision 1.12  2003/10/17 21:14:31  shang
# made name as an argument of APSAlertBox, so that programmer can name the alert box
# instead of "alert" only.
#
# Revision 1.11  2003/02/17 18:16:30  soliday
# The alert box now display the path to the script command.
#
# Revision 1.10  2001/02/06 19:44:20  soliday
# Fixed some tcl warning messages found with procheck
#
# Revision 1.9  2001/01/31 16:35:35  soliday
# Fixed a small problem with last change.
#
# Revision 1.8  2001/01/31 16:32:20  soliday
# Added type option to APSAlertBox.
#
# Revision 1.7  2000/06/27 16:20:01  soliday
# Added catch statements around all the grab commands.
#
# Revision 1.6  1998/10/27 22:30:56  soliday
# Fixed bug before grab command by adding tkwait visibility $parent$widget
#
# Revision 1.5  1998/10/20 20:47:14  borland
# Fixed a bug with emailing of messages that occured when the message had
# quotation marks in it.
#
# Revision 1.4  1996/10/04 19:52:59  saunders
# Added "print", "save as..", and "email..." buttons to APSAlertBox.
# In doing so, I removed functionally duplicate procedures, and created
# generic APSPrint and APSSaveDialog. Also addes "save as.." to APSExecLog.
# Removed "Enscript" buttons, since "Print" is adequate.
#
# Revision 1.3  1996/01/16  21:19:52  saunders
# Added Log for CVS. Should have done this a long time ago.
#
#

proc APSAlertBoxHelp {} {
    return "Usage: APSAlertBox widget \n\[-parent <string>\]\t\n\[-noPack 1\]\t\n\[-packOption <list>\]\t\n\[-contextHelp <string>\]\t\n\[-errorMessage <string>\]\t\n\[-modeless 1\]\t\n\[-type {error | warning | done}\]\t\n\[-name <string>\]\t\n\[-beep {no | once | continuous | backoff}\]\t\n\[-volume <fraction>\]\t\n\[-announceWorkstion {0|1}\]\t\n\[-toneType {sr|booster|linac|par|default}\]\t\n\nCreates\t\$parent\$widget.\n\tmsg\n\tbuttonRow.ok.button\n\t"
}

proc APSAlertBox {widget args} {
    global apsContextHelp apsScriptCommand
    set parent ""
    set noPack 0
    set packOption "-side top -fill x"
    set name "Alert"
    set errorMessage "No Message Given"
    set modeless 0
    set contextHelp ""
    set type "error"
    set beep no
    set announceWorkstation 1
    set toneType default
    set volume 1
    APSStrictParseArguments {parent noPack packOption errorMessage modeless \
			 contextHelp type name beep announceWorkstation toneType volume}
    set beep [string tolower $beep]
    if {$contextHelp != ""} {
        set apsContextHelp($parent$widget) $contextHelp
    }
    set color red
    switch [string tolower $type] {
        warning {
            set color yellow
        }
        done {
            set color green
        }
    }
    
    if {$parent == ""} {
        toplevel $widget -bg $color
        wm title $widget $name
    } else {
        frame $parent$widget -bg $color
        if {!$noPack} {
            eval pack $parent$widget $packOption
        }
    }
    if [info exists apsScriptCommand] {
        set errorMessage "${apsScriptCommand}\n$errorMessage"
    } else {
        set errorMessage "$errorMessage"
    }

    label $parent$widget.label -relief ridge -text $type -bg $color -fg black
    pack $parent$widget.label -side top -fill x
    
    set tmpText [split $errorMessage "\n"]
    set lines [llength $tmpText]
    if {$lines>30 || [string length $errorMessage] > 2000} {
        APSScrolledText .msg -parent $parent$widget -width 50 -height 40
        $parent$widget.msg.text insert end "$errorMessage\n"
    } else {
        message $parent$widget.msg -relief ridge -justify center -bd 2 \
            -width 400 -text "$errorMessage\n                            "
        pack $parent$widget.msg -side top -fill both -expand 1
    }

    frame $parent$widget.buttonRow -relief raised -bd 2
    pack $parent$widget.buttonRow -side bottom -fill x

    APSButton .ok -parent $parent$widget.buttonRow -text OK \
      -command "destroy $parent$widget" \
      -highlight 1 -contextHelp \
      "Dismiss this warning and continue with application."
    
    regsub -all {"} $errorMessage {\\"} errorMessage1
    APSButton .print -parent $parent$widget.buttonRow -size small \
      -text Print -command "APSPrint -text \"$errorMessage1\"" -contextHelp \
      "Print this $type message to local printer."
    
    APSButton .save -parent $parent$widget.buttonRow -size small \
      -text "Save As..."\
      -command "APSSaveDialog .alertSave -text \"$errorMessage1\" -modal 1" \
      -contextHelp "Save this $type message to a file."

    APSButton .email -parent $parent$widget.buttonRow -size small \
      -text "Email..." \
      -command "APSEMailDialog .alertEmail -message \"$errorMessage1\" -modal 1" \
      -contextHelp "Email $type message to someone."
   # update
    tkwait visibility $parent$widget
    if {$beep!="no"} {
        global apsAlertStopBeep
        set apsAlertStopBeep($parent$widget) 0
        set apsAlertStopBeep($parent$widget.count) 0
        if {$beep=="continuous" || $beep=="backoff"} {
            set stopBeep "global apsAlertStopBeep; set apsAlertStopBeep($parent$widget) 1"
            APSButton .silent -parent $parent$widget.buttonRow -text "Silent" \
                -command $stopBeep
        }
        APSAlertBeep -beep $beep -widget $parent$widget \
            -announceWorkstation $announceWorkstation -toneType $toneType -volume $volume
        update
    }
    if {!$modeless} {
        set grabList [grab current $parent$widget]
        focus $parent$widget
        catch {grab $parent$widget}
        tkwait window $parent$widget
        catch {grab set $grabList}
    }
}    

proc APSAlertBeep {args} {
    set beep once
    set interval 1
    set widget "" 
    set announceWorkstation 1
    set toneType default
    set volume 1.0
    global apsAlertStopBeep apsScriptHost
    APSStrictParseArguments {beep interval widget announceWorkstation toneType volume}
    set soundDir /usr/local/oag/apps/sounds
    if {$beep=="no"} {
        return
    }
    if ![info exist apsAlertStopBeep($widget.count)] {
        set apsAlertStopBeep($widget.count) 0
    } 
    if ![info exist apsAlertStopBeep($widget)] {
        set apsAlertStopBeep($widget) 0
    }
    set toneType [string tolower $toneType]
    switch $toneType {
        linac -
        par -
        booster -
        sr {
            set beepType $toneType
        }
        default {
            set beepType default
        }
    } 
    if {[string length $widget] && [winfo exist $widget] && !$apsAlertStopBeep($widget)} {
        global tcl_platform
        if {$announceWorkstation && ![expr $apsAlertStopBeep($widget.count)%60]} {
            set name $apsScriptHost
            if [file exist $soundDir/$name.wav] {
                if {$tcl_platform(os) == "SunOS"} {
                    catch {exec audioplay -i -v 100 $soundDir/$name.wav &}
                } else {
                    catch {exec play -q -v $volume $soundDir/$name.wav &}
                }
            }
            after 500
        } 
        if {$tcl_platform(os) == "SunOS"} {
            catch {exec audioplay -i -v 100 $soundDir/$beepType.wav &}
        } else {
            catch {exec play -q -v $volume $soundDir/$beepType.wav &}
        }
        #bell
        incr apsAlertStopBeep($widget.count)
        if {$beep=="continuous"} {
            after [expr $interval * 1000] "APSAlertBeep -beep continuous -interval $interval -widget $widget -announceWorkstation $announceWorkstation -toneType $toneType -volume $volume"
        }
        if {$beep=="backoff"} {
            if [expr $interval>60] {
                set interval 60
            }
            after [expr int($interval * 1000)] "APSAlertBeep -beep backoff -interval [expr $interval*1.2] -widget $widget -announceWorkstation $announceWorkstation -toneType $toneType -volume $volume"
        }
    } else {
        set apsAlertStopBeep($widget) 0
        set apsAlertStopBeep($widget.count) 0
    }
    return
}



